/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 ** 
 ** \brief This example shows how to use the ADC Priority conversion with 
 ** conversion end polling
 **
 ** In this example the ADC0 channel 0 is set to priority conversion mode.
 ** The conversion is started by software trigger and the function
 ** Adc_PrioStatus() is used for polling of the end of conversion. In
 ** main() the result is stored in the gloabl variable u32AdcResult.
 **
 ** History:
 **   - 2014-01-15  1.0  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

uint32_t u32Result              ;		// global ADC result variable
stc_prio_fifo_info_t stcFifoInfo;		// global ADC FIFO data information

/**
 ******************************************************************************
 ** \brief  ADC0 initialization and single conversion start
 **
 ** \return  uint32_t   ADC result
 ******************************************************************************/
void Main_adc_polling(void)
{
    stc_adcn_t*      pstcAdc = (stc_adcn_t*) &ADC0;
    stc_adc_config_t stcConfig;

    PDL_ZERO_STRUCT(stcConfig);		// Clear local configuration to zero.

    /* Common setting */
    stcConfig.bLsbAlignment = TRUE;
    stcConfig.u32SamplingTimeSelect.AD_CH_0 = 1u;
    stcConfig.enSamplingTimeN0 = Value32;
    stcConfig.u8SamplingTime0 = 30u;
    stcConfig.enSamplingTimeN1 = Value32;
    stcConfig.u8SamplingTime1 = 30u;
    stcConfig.u8SamplingMultiplier = 5u;
    stcConfig.u8EnableTime = 10u;   
    /* Scan conversion configuration */
    stcConfig.bScanInit = FALSE;
    /* Priority converison is not used, so don't initialize it */
    stcConfig.bPrioInit = TRUE;
    stcConfig.stcPrioInit.u8PrioFifoDepth = 0;
    stcConfig.stcPrioInit.u8PrioLevel2AnalogChSel = 0u; //select ch.0 as software trigger
    stcConfig.stcPrioInit.u8PrioLevel1AnalogChSel = 1u;
    stcConfig.stcPrioInit.bPrioExtTrigStartEnable = FALSE;
    stcConfig.stcPrioInit.bPrioTimerStartEnable = FALSE;
    stcConfig.stcPrioInit.enPrioTimerTrigger = AdcNoTimer;
    /* Comparison function is not used, so don't initialize it */
    stcConfig.bComparisonEnable = FALSE;
    /* Range comparison function is not used, so don't initialize it */
    stcConfig.bRangeComparisonEnable = FALSE;
  
    if (Ok == Adc_Init(&ADC0, &stcConfig))        // Init ADC0
    {
        pstcAdc = (stc_adcn_t*) &ADC0;

        Adc_Enable(pstcAdc);               // Enable ADC0 and wait for ready
        Adc_SwTriggerPrio(pstcAdc);        // Trigger ADC0

        while (TRUE != Adc_GetIntFlag(pstcAdc, AdcPrioInt))      // Poll for conversion done
        {}
        Adc_ClrIntFlag(pstcAdc, AdcPrioInt);
        
        u32Result = Adc_GetPrioFifo(pstcAdc);
        Adc_ExtractPrioFifo(pstcAdc, u32Result, &stcFifoInfo);
    }

}

/**
 ******************************************************************************
 ** \brief  Main function
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
  Main_adc_polling();
  
  while(1)
  {}
}
