/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 **
 ** \brief This example shows how to use the ADC triggered by Base Timer.
 **
 ** Before using this exampel, user can connect P10/AN00 with AVCC and
 ** P11/AN01 with GND. ADC samples the analog signal from AN00 and AN11.
 ** The ADC result data will output to terminal window.
 **
 ** BT ch.0 is configured to trigger ADC scan conversion. Every 2ms interval,
 ** a ADC start signal will be generated by BT.
 **
 ** History:
 **   - 2014-01-15  0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/******************************************************************************/
/* Local definition                                                           */
/******************************************************************************/
/* ADC */
/*! \brief ADC unit  */
#define USER_ADC_UNIT                    ADC0
/*! \brief Scan channel number  */
#define USER_ADC_SCAN_CH_NUM             (2)
/*! \brief ADC scan channel 0  */
#define USER_ADC_SCAN_CH0                 0
/*! \brief ADC scan channel 1  */
#define USER_ADC_SCAN_CH1                 1
/* BT */
/*! \brief User BT unit */
#define USER_BT_CH      BT0
/*! \brief User BT index */
#define USER_BT_CH_INDEX  BtInstanceIndexBt0

/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/
/*! \brief Read finish flag  */
boolean_t bReadFinished = FALSE;
/*! \brief Scan data  */
uint16_t ScanData[20][USER_ADC_SCAN_CH_NUM];
/*! \brief Scan data  */
uint16_t ScanDataCnt = 0;

/**
 ******************************************************************************
 ** \brief  Result callback function
 **
 ** \param  u32AdcArgument  ADC FIFO value
 ******************************************************************************/
void AdcScanIntCallback(uint32_t* pu32AdcArgument)
{
    uint32_t AdcData, Cnt;
    stc_scan_fifo_info_t stcFifoInfo;
    Cnt = USER_ADC_SCAN_CH_NUM;

    if(ScanDataCnt >= sizeof(ScanData)/sizeof(ScanData[0]))
    {
        ScanDataCnt = 0;
        bReadFinished = 1; /* Finish timer trigger */
        Bt_Rt_DisableCount(&USER_BT_CH); /* Disable Reload timer counter */
        Adc_ClrScanFifo((stc_adcn_t*)&USER_ADC_UNIT); /* Give up FIFO data */

        return;
    }

    while(Cnt--)
    {
        AdcData = *pu32AdcArgument;
        Adc_ExtractScanFifo((stc_adcn_t*)&USER_ADC_UNIT, AdcData, &stcFifoInfo);

        if(stcFifoInfo.u8Ch == USER_ADC_SCAN_CH0)
        {
            ScanData[ScanDataCnt][0] = stcFifoInfo.u16Data;
        }
        else if(stcFifoInfo.u8Ch == USER_ADC_SCAN_CH1)
        {
            ScanData[ScanDataCnt][1] = stcFifoInfo.u16Data;
        }
    }
    ScanDataCnt++;
}
/**
 ******************************************************************************
 ** \brief  Error callback function
 ******************************************************************************/
void AdcErrorCallback(void)
{
  // Error handling ... Should not happen.
}

/**
 ******************************************************************************
 ** \brief  Result callback function
 ******************************************************************************/
void InitAdc(void)
{
    stc_adcn_t*      pstcAdc = (stc_adcn_t*) &USER_ADC_UNIT;
    stc_adc_config_t stcConfig;
    stc_adc_int_sel_t stcIntSel;
    stc_adc_int_callback_t stcIntCallback;

    PDL_ZERO_STRUCT(stcConfig);		// Clear local configuration to zero.

    /* Common setting */
    stcConfig.bLsbAlignment = TRUE;
    stcConfig.u32SamplingTimeSelect.u32AD_CHn = 0xFFFFFFFFu;
    stcConfig.enSamplingTimeN0 = Value32;
    stcConfig.u8SamplingTime0 = 30u;
    stcConfig.enSamplingTimeN1 = Value32;
    stcConfig.u8SamplingTime1 = 30u;
    stcConfig.u8SamplingMultiplier = 5u;
    stcConfig.u8EnableTime = 10u;
    /* Scan conversion configuration */
    stcConfig.bScanInit = TRUE;
    stcConfig.stcScanInit.u32ScanCannelSelect.u32AD_CHn = ((1u << USER_ADC_SCAN_CH0) |
                                                           (1u << USER_ADC_SCAN_CH1) );
    stcConfig.stcScanInit.enScanMode = ScanSingleConversion;
    stcConfig.stcScanInit.bScanTimerStartEnable = TRUE;
    stcConfig.stcScanInit.u8ScanFifoDepth = (USER_ADC_SCAN_CH_NUM - 1);       // Only one conversion
    stcConfig.stcScanInit.enScanTimerTrigger = (en_adc_timer_select_t)((uint8_t)USER_BT_CH_INDEX + (uint8_t)AdcBt0);
    /* Priority converison is not used, so don't initialize it */
    stcConfig.bPrioInit = FALSE;
    /* Comparison function is not used, so don't initialize it */
    stcConfig.bComparisonEnable = FALSE;
    /* Range comparison function is not used, so don't initialize it */
    stcConfig.bRangeComparisonEnable = FALSE;

    if (Ok == Adc_Init(pstcAdc, &stcConfig))        // Init ADC0
    {
        /* Configure interrupt  */
        PDL_ZERO_STRUCT(stcIntSel);
        PDL_ZERO_STRUCT(stcIntCallback);
        stcIntSel.bScanIrq = 1;
        stcIntSel.bFifoOverrunIrq = 1;
        stcIntCallback.pfnScanCallback = (func_ptr_adc_parg32_t)AdcScanIntCallback;
        stcIntCallback.pfnScanErrorCallback = AdcErrorCallback;
        Adc_EnableInt(pstcAdc, &stcIntSel, &stcIntCallback);

        Adc_Enable(pstcAdc);               // Enable ADC0 and wait for ready
        Adc_SwTriggerScan(pstcAdc);        // Trigger ADC0
    }
}

/**
 ******************************************************************************
 ** \brief  ADC intialization function
 ******************************************************************************/
void InitBt(void)
{
    stc_bt_rt_config_t stcRtConfig;
    // Set BT IO mode
    Bt_ConfigIOMode(&USER_BT_CH, BtIoMode0);

    // Select reload timer function
    Bt_SelTimerMode(&USER_BT_CH, BtRtMode);

    // Initialize BT
    stcRtConfig.enPres = RtPres1Div4; // BT_CLK_DIV_1: T = 0.1us @ PCLK = 40MHz
    stcRtConfig.enSize = RtSize16Bit;
    stcRtConfig.enMode = RtReload;
    stcRtConfig.enOutputPolarity = RtPolarityLow;
    stcRtConfig.enExtTrig = RtExtTiggerDisable;

    if(Ok == Bt_Rt_Init(&USER_BT_CH, &stcRtConfig))
    {
        Bt_Rt_WriteCycleVal(&USER_BT_CH, 19999);  // Cycle = T*(m+1)=2ms
    }
}

/**
 ******************************************************************************
 ** \brief  Print ADC result data in the terminal window
 ******************************************************************************/
void PrintAdcData(void)
{
    uint16_t i , j, u16Times, u16Size;
    uint8_t au8Ch[USER_ADC_SCAN_CH_NUM] = {USER_ADC_SCAN_CH0, USER_ADC_SCAN_CH1};

    u16Size = USER_ADC_SCAN_CH_NUM;
    u16Times = sizeof(ScanData)/sizeof(ScanData[0]);
    i = 0;
    j = 0;
    while(i != u16Times)
    {
    #ifdef DEBUG_PRINT
        printf("------------ Sample %d-------------\n\r", (i+1));
    #endif
        while(j != u16Size)
        {
            /*Print Result */
        #ifdef DEBUG_PRINT
            printf( "<Ch%d> ADC value:%d\n\r", au8Ch[j], ScanData[i][j]);
        #endif
            j++;
        }
    #ifdef DEBUG_PRINT
        printf("\n\r");
    #endif
        i++;
        j = 0;
    }
}



/**
 ******************************************************************************
 ** \brief  Main function
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    InitAdc();
    InitBt();
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("ADC sample (triggered by Base Timer) Start \n");
    printf("==================================================\n");
#endif
    // Start Reload timer by software trigger
    Bt_Rt_EnableCount(&USER_BT_CH);
    Bt_Rt_EnableSwTrig(&USER_BT_CH);

    while(1)
    {
        if(bReadFinished)
        {
            bReadFinished = 0;
            PrintAdcData();
            break;
        }
    }

    Adc_DeInit((stc_adcn_t*) &USER_ADC_UNIT);

    while(1)
    {}

}
