/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 ** 
 ** \brief This example shows how to use the ADC with conversion end polling
 **
 ** In this example the ADC0 channel 0 is set to single conversion mode.
 ** The conversion is started by software trigger and the function
 ** Adc_GetScanIntFlag() is used for polling of the end of conversion. In
 ** main() the result is stored in the gloabl variable stcFifoInfo.
 **
 ** History:
 **   - 2014-01-14  1.0  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

volatile stc_scan_fifo_info_t stcFifoInfo ;		// global ADC result variable

/**
 ******************************************************************************
 ** \brief  ADC0 initialization and single conversion start
 **
 ** \return  uint32_t   ADC result
 ******************************************************************************/
stc_scan_fifo_info_t Main_adc_polling(void)
{
    stc_adcn_t*      pstcAdc = (stc_adcn_t*) &ADC0;
    stc_adc_config_t stcConfig;
    uint32_t u32Result;
    stc_scan_fifo_info_t stcInfo;

    PDL_ZERO_STRUCT(stcConfig);		// Clear local configuration to zero.

    /* Common setting */
    stcConfig.bLsbAlignment = TRUE;
    stcConfig.u32SamplingTimeSelect.AD_CH_0 = 1u;
    stcConfig.enSamplingTimeN0 = Value32;
    stcConfig.u8SamplingTime0 = 30u;
    stcConfig.enSamplingTimeN1 = Value32;
    stcConfig.u8SamplingTime1 = 30u;
    stcConfig.u8SamplingMultiplier = 5u;
    stcConfig.u8EnableTime = 10u;   
    /* Scan conversion configuration */
    stcConfig.bScanInit = TRUE;
    stcConfig.stcScanInit.u32ScanCannelSelect.AD_CH_0 = 1u;
    stcConfig.stcScanInit.enScanMode = ScanSingleConversion;
    stcConfig.stcScanInit.bScanTimerStartEnable = FALSE;
    stcConfig.stcScanInit.u8ScanFifoDepth = 0u;       // Only one conversion
    stcConfig.stcScanInit.enScanTimerTrigger = AdcNoTimer;
    /* Priority converison is not used, so don't initialize it */
    stcConfig.bPrioInit = FALSE;
    /* Comparison function is not used, so don't initialize it */
    stcConfig.bComparisonEnable = FALSE;
    /* Range comparison function is not used, so don't initialize it */
    stcConfig.bRangeComparisonEnable = FALSE;
    
    if (Ok == Adc_Init(pstcAdc, &stcConfig))        // Init ADC0
    {
        Adc_Enable(pstcAdc);               // Enable ADC0 and wait for ready
        Adc_SwTriggerScan(pstcAdc);        // Trigger ADC0
        
        while(1)
        {
            if(TRUE == Adc_GetIntFlag(pstcAdc, AdcScanInt))
            {
                Adc_ClrIntFlag(pstcAdc, AdcScanInt);
                break;
            }
        }
    }
  
    u32Result = Adc_GetScanFifo(pstcAdc);
    Adc_ExtractScanFifo(pstcAdc, u32Result, &stcInfo);
    return stcInfo;
}

/**
 ******************************************************************************
 ** \brief  Main function
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
  stcFifoInfo = Main_adc_polling();
  
  while(1)
  {}
}
