/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Demonstrate FCS (Anomalous Frequency Detection) interrupt when Main clock 
 ** frequency is abnormal.
 **
 ** History:
 **   - 2014-02-11  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/
 
/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"


/******************************************************************************/
/* Local Functions                                                            */
/******************************************************************************/
static void Delay(uint32_t Cnt);

/******************************************************************************/
/* Local variable                                                             */
/******************************************************************************/
static uint16_t m_u16MainClockCycle;
static uint8_t m_u8FcsIntFlag;
static void FcsIntCallback(void);

/*
 ******************************************************************************
 ** \brief Clock supervisor mode main function
 ******************************************************************************
 */
int32_t main(void)
{        
    stc_reset_result_t stcResetCause;
    stc_csv_status_t stcCsvStatus ; 
      
    Hwwdg_Stop();
    
    /* Initialize Green LED IO */
    Gpio1pin_InitOut(GPIO1PIN_P61, Gpio1pin_InitDirectionOutput); 
    Gpio1pin_Put(GPIO1PIN_P61, 1);
    
    /* Read causes of resets, reading the register clears all bits */
    Reset_GetCause(&stcResetCause);
    /* Read CSV status */
    Csv_GetCsvFailCause(&stcCsvStatus);
   
    Csv_DisableMainCsv();    
    Csv_DisableSubCsv();   
     
    /* Clock Failure Detection(CSV) reset is generated */    
    if (stcResetCause.bClockSupervisor == TRUE)
    {
        /* Sub Clock Failure Detection(CSV) reset is generated.*/
        if (stcCsvStatus.bCsvMainClockStatus == TRUE) 
        { 
            while(1);
        }
        
        /* Main Clock Failure Detection(CSV) reset is generated */
        if (stcCsvStatus.bCsvSubClockStatus == TRUE)
        {  
            while(1);
        }
    } 
    
    if(stcResetCause.bAnomalousFrequency == TRUE)
    {
        while(1);
    }
    
    /* FCS and CSV reset are not generated */
    if ((stcCsvStatus.bCsvMainClockStatus != TRUE) && 
        (stcCsvStatus.bCsvSubClockStatus != TRUE)) 
    {
        /* Assume High speed CR = 4MHz, the expected main clock cycle is 1024  */
        Csv_SetFcsCrDiv(FcsCrDiv1024);
        
        /* Assume High speed CR = 4MHz, this is a correct range */
        Csv_SetFcsDetectRange(1000, 1100);
        /* Assume High speed CR = 4MHz, this is a wrong range to simulate a FCS reset */
        //Csv_SetFcsDetectRange(800, 900);
        
        /* Enable FCS interrupt */
        Csv_EnableFcsInt(FcsIntCallback);
        
        /* Enable FCS function */
        Csv_EnableFcs();
        
        while(1)
        {
            if(m_u8FcsIntFlag)
            {
                m_u8FcsIntFlag = 0;
             
                /* Read detected main clock cycle */
                m_u16MainClockCycle = Csv_GetFcsDetectCount();
                if(m_u16MainClockCycle) ;
                
                while(1)
                {
                    Gpio1pin_Put(GPIO1PIN_P61, 1);
                    Delay(1);
                    Gpio1pin_Put(GPIO1PIN_P61, 0);
                    Delay(1);
                }
            }
        }
    
    } 
    while(1){;}
    
}

/*
 ******************************************************************************
 ** \brief Time delay
 ******************************************************************************
 */
static void Delay(uint32_t u32Cnt)
{
    uint32_t u32i;
    for(; u32Cnt; u32Cnt--)
        for(u32i=SystemCoreClock/5; u32i; u32i--);
}

/*
 ******************************************************************************
 ** \brief FCS interrupt callback function
 ******************************************************************************
 */
static void FcsIntCallback(void)
{
    /* Disable FCS interrupt */
    Csv_DisableFcsInt();
    
    /* Disable FCS function */
    Csv_DisableFcs();
    
    m_u8FcsIntFlag = 1;
}

/*****************************************************************************/
/* END OF FILE */


