/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates communication between CSIO0 and CSIO1 with 
 ** interrupt mode using DMA. 
 ** Make following connection before using this example:
 ** Master             Slave
 ** SIN0_1  -------    SOT1_1
 ** SCK0_0  -------    SCK1_1
 ** SOT0_0  -------    SIN1_1
 ** 
 ** History:
 **   - 2014-03-17  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
#define SAMPLE_CSIO_SLAVE_RX_BUFFSIZE   (8u)
#define SAMPLE_CSIO_MASTER_TX_BUFFSIZE  (8u)

#define SAMPLE_CSIO_MASTER_RX_BUFFSIZE  (8u)
#define SAMPLE_CSIO_SLAVE_TX_BUFFSIZE   (8u)

#define InitCsio0Io(void)  {SetPinFunc_SIN0_1();SetPinFunc_SOT0_0();SetPinFunc_SCK0_0();}
#define InitCsio1Io(void)  {SetPinFunc_SIN1_1();SetPinFunc_SOT1_1();SetPinFunc_SCK1_1();}   

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
volatile stc_mfsn_csio_t* CsioCh0 = &CSIO0;
volatile stc_mfsn_csio_t* CsioCh1 = &CSIO1;
stc_csio_int_sel_t stcCsio0IntSel, stcCsio1IntSel;
stc_csio_int_cb_t stcCsio0IntCb, stcCsio1IntCb;
static boolean_t bDma0Finished, bDma1Finished;
static uint8_t au8CsioMasterTxBuf[SAMPLE_CSIO_MASTER_TX_BUFFSIZE];
static uint8_t au8CsioSlaveTxBuf[SAMPLE_CSIO_SLAVE_TX_BUFFSIZE];
static uint8_t au8CsioMasterRxBuf[SAMPLE_CSIO_MASTER_RX_BUFFSIZE];
static uint8_t au8CsioSlaveRxBuf[SAMPLE_CSIO_SLAVE_RX_BUFFSIZE];

/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2 
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint8_t u8Length)
{
    while(u8Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  DMA Error callback function
 ******************************************************************************/
void Main_dma0_error_callback(uint8_t u8ErrorCode)
{
    // Error handling ... Should never happen.
}

/**
 ******************************************************************************
 ** \brief  DMNA finish callback function
 ******************************************************************************/
void Main_dma0_finish_callback(void)
{  
    stcCsio0IntSel.bTxInt = TRUE;
    stcCsio0IntSel.bRxInt = TRUE;
    Mfs_Csio_DisableInt(CsioCh0, &stcCsio0IntSel);
    
    bDma0Finished = TRUE;         // Set DMA finished notification flag
}

/**
 ******************************************************************************
 ** \brief  DMA Error callback function
 ******************************************************************************/
void Main_dma1_error_callback(uint8_t u8ErrorCode)
{
    // Error handling ... Should never happen.
}

/**
 ******************************************************************************
 ** \brief  DMNA finish callback function
 ******************************************************************************/
void Main_dma1_finish_callback(void)
{    
    stcCsio1IntSel.bRxInt = TRUE;
    stcCsio1IntSel.bTxInt = TRUE;
    Mfs_Csio_DisableInt(CsioCh1, &stcCsio1IntSel);
  
    bDma1Finished = TRUE;         // Set DMA finished notification flag
}

/**
 ******************************************************************************
 ** \brief Dummy callback function
 ******************************************************************************/
static void DummyIntCallback(void)
{
    // Should never enter
}

/**
 ******************************************************************************
 ** \brief  Initialize and enable DMA ch.0 and ch.1 for CSIO master TX and
 **         CSIO slave RX
 ******************************************************************************/
void MasterTxSlaveRxDmaInit(void)
{
    stc_dma_config_t stcDma0Config, stcDma1Config;
    stc_dma_int_sel_t stcDma0IntSel, stcDma1IntSel;
    stc_dma_int_cb_t  stcDma0IntCb, stcDma1IntCb;

    // Clear local configuration structure to zero.
    PDL_ZERO_STRUCT(stcDma0Config);	
    PDL_ZERO_STRUCT(stcDma0IntSel);	
    PDL_ZERO_STRUCT(stcDma0IntCb);	
    PDL_ZERO_STRUCT(stcDma1Config);	
    PDL_ZERO_STRUCT(stcDma1IntSel);	
    PDL_ZERO_STRUCT(stcDma1IntCb);

    // DMAC0 configuration
    stcDma0Config.enDmaIdrq = MfsTx0;
    stcDma0Config.u8BlockCount = 1u;
    stcDma0Config.u16TransferCount = SAMPLE_CSIO_MASTER_TX_BUFFSIZE ;   
    stcDma0Config.enTransferMode = DmaDemandTransfer;
    stcDma0Config.enTransferWdith = Dma8Bit;
    stcDma0Config.u32SourceAddress = (uint32_t)&(au8CsioMasterTxBuf[0u]); // UART data address
    stcDma0Config.u32DestinationAddress = MFS0_DATA_REG_ADDR;	   // Destination array's address
    stcDma0Config.bFixedSource = FALSE;
    stcDma0Config.bFixedDestination = TRUE;
    stcDma0Config.bReloadCount = TRUE;       // Reload count for next data package
    stcDma0Config.bReloadSource = TRUE;      // Reload source address for next data package
    stcDma0Config.bReloadDestination = FALSE; 
    stcDma0Config.bEnableBitMask = TRUE;    // Don't clear enable bit after transfer completion

    if (Ok == Dma_Init_Channel(0u, &stcDma0Config))      // Initialize DMA channel 0
    {
        /* Configure interrupt */
        stcDma0IntSel.bCompleteInt = 1;
        stcDma0IntSel.bErrorInt = 1;
        stcDma0IntCb.pfnCompleteCallback = Main_dma0_finish_callback;
        stcDma0IntCb.pfnErrorCallback = Main_dma0_error_callback; 
        Dma_EnableInt(0u, &stcDma0IntSel, &stcDma0IntCb);

        Dma_SetChannel(0u, TRUE, FALSE, FALSE);	      // Enable channel
    }

    // DMAC1 configuration
    stcDma1Config.enDmaIdrq = MfsRx1;
    stcDma1Config.u8BlockCount = 1u;
    stcDma1Config.u16TransferCount = SAMPLE_CSIO_SLAVE_RX_BUFFSIZE ;   
    stcDma1Config.enTransferMode = DmaDemandTransfer;
    stcDma1Config.enTransferWdith = Dma8Bit;
    stcDma1Config.u32SourceAddress = MFS1_DATA_REG_ADDR ; // UART data address
    stcDma1Config.u32DestinationAddress = (uint32_t)&(au8CsioSlaveRxBuf[0u]);	 // Destination array's address
    stcDma1Config.bFixedSource = TRUE;
    stcDma1Config.bFixedDestination = FALSE;
    stcDma1Config.bReloadCount = TRUE;       // Reload count for next data package
    stcDma1Config.bReloadSource = FALSE;
    stcDma1Config.bReloadDestination = TRUE; // Reload destination for next data package
    stcDma1Config.bEnableBitMask = TRUE;     // Don't clear enable bit after transfer completion

    if (Ok == Dma_Init_Channel(1u, &stcDma1Config))      // Initialize DMA channel 1
    {
        /* Configure interrupt */
        stcDma1IntSel.bCompleteInt = 1;
        stcDma1IntSel.bErrorInt = 1;
        stcDma1IntCb.pfnCompleteCallback = Main_dma1_finish_callback;
        stcDma1IntCb.pfnErrorCallback = Main_dma1_error_callback; 
        Dma_EnableInt(1u, &stcDma1IntSel, &stcDma1IntCb);

        Dma_Enable();            // Overall enable of DMA

        Dma_SetChannel(1u, TRUE, FALSE, FALSE);	      // Enable channel
    }
    
    Dma_Enable();            // Overall enable of DMA
}

/**
 ******************************************************************************
 ** \brief  De-Initialize DMA ch.0 and ch.1 for CSIO master RX and
 **         CSIO slave TX
 ******************************************************************************/
void MasterTxSlaveRxDmaDeInit(void)
{
    Dma_Disable();            // Overall enable of DMA
    Dma_DeInit_Channel(0u);
    Dma_DeInit_Channel(1u);
}

/**
 ******************************************************************************
 ** \brief  Initialize and enable DMA ch.0 and ch.1 for CSIO master RX and
 **         CSIO slave TX
 ******************************************************************************/
void MasterRxSlaveTxDmaInit(void)
{
    stc_dma_config_t stcDma0Config, stcDma1Config;
    stc_dma_int_sel_t stcDma0IntSel, stcDma1IntSel;
    stc_dma_int_cb_t  stcDma0IntCb, stcDma1IntCb;

    // Clear local configuration structure to zero.
    PDL_ZERO_STRUCT(stcDma0Config);	
    PDL_ZERO_STRUCT(stcDma0IntSel);	
    PDL_ZERO_STRUCT(stcDma0IntCb);	
    PDL_ZERO_STRUCT(stcDma1Config);	
    PDL_ZERO_STRUCT(stcDma1IntSel);	
    PDL_ZERO_STRUCT(stcDma1IntCb);

    // DMAC0 configuration
    stcDma0Config.enDmaIdrq = MfsRx0;
    stcDma0Config.u8BlockCount = 1u;
    stcDma0Config.u16TransferCount = SAMPLE_CSIO_MASTER_RX_BUFFSIZE ;   
    stcDma0Config.enTransferMode = DmaDemandTransfer;
    stcDma0Config.enTransferWdith = Dma8Bit;
    stcDma0Config.u32SourceAddress = MFS0_DATA_REG_ADDR;	   // CSIO data register address
    stcDma0Config.u32DestinationAddress = (uint32_t)&(au8CsioMasterRxBuf[0u]); // destination array address
    stcDma0Config.bFixedSource = TRUE;
    stcDma0Config.bFixedDestination = FALSE;
    stcDma0Config.bReloadCount = TRUE;       // Reload count for next data package
    stcDma0Config.bReloadSource = FALSE;      
    stcDma0Config.bReloadDestination = TRUE; // Reload destination address for next data package
    stcDma0Config.bEnableBitMask = TRUE;     // Don't clear enable bit after transfer completion

    if (Ok == Dma_Init_Channel(0u, &stcDma0Config))      // Initialize DMA channel 0
    {
        /* Configure interrupt */
        stcDma0IntSel.bCompleteInt = 1;
        stcDma0IntSel.bErrorInt = 1;
        stcDma0IntCb.pfnCompleteCallback = Main_dma0_finish_callback;
        stcDma0IntCb.pfnErrorCallback = Main_dma0_error_callback; 
        Dma_EnableInt(0u, &stcDma0IntSel, &stcDma0IntCb);

        Dma_SetChannel(0u, TRUE, FALSE, FALSE);	      // Enable channel
    }

    // DMAC1 configuration
    stcDma1Config.enDmaIdrq = MfsTx1;
    stcDma1Config.u8BlockCount = 1u;
    stcDma1Config.u16TransferCount = SAMPLE_CSIO_SLAVE_TX_BUFFSIZE ;   
    stcDma1Config.enTransferMode = DmaDemandTransfer;
    stcDma1Config.enTransferWdith = Dma8Bit;
    stcDma1Config.u32SourceAddress = (uint32_t)&(au8CsioSlaveTxBuf[0u]);	 // Source array's address
    stcDma1Config.u32DestinationAddress =  MFS1_DATA_REG_ADDR ; // CSIO data register address
    stcDma1Config.bFixedSource = FALSE;
    stcDma1Config.bFixedDestination = TRUE;
    stcDma1Config.bReloadCount = TRUE;       // Reload count for next data package
    stcDma1Config.bReloadSource = TRUE;
    stcDma1Config.bReloadDestination = FALSE; // Reload destination for next data package
    stcDma1Config.bEnableBitMask = TRUE;     // Don't clear enable bit after transfer completion

    if (Ok == Dma_Init_Channel(1u, &stcDma1Config))      // Initialize DMA channel 1
    {
        /* Configure interrupt */
        stcDma1IntSel.bCompleteInt = 1;
        stcDma1IntSel.bErrorInt = 1;
        stcDma1IntCb.pfnCompleteCallback = Main_dma1_finish_callback;
        stcDma1IntCb.pfnErrorCallback = Main_dma1_error_callback; 
        Dma_EnableInt(1u, &stcDma1IntSel, &stcDma1IntCb);

        Dma_Enable();            // Overall enable of DMA

        Dma_SetChannel(1u, TRUE, FALSE, FALSE);	      // Enable channel
    }
    
    Dma_Enable();            // Overall enable of DMA
}

/**
 ******************************************************************************
 ** \brief  De-Initialize DMA ch.0 and ch.1 for CSIO master RX and
 **         CSIO slave TX
 ******************************************************************************/
void MasterRxSlaveTxDmaDeInit(void)
{
    Dma_Disable();            // Overall enable of DMA
    Dma_DeInit_Channel(0u);
    Dma_DeInit_Channel(1u);
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{   
    stc_mfs_csio_config_t stcCsio0Config, stcCsio1Config;
    uint8_t u8i;
    
    /* Clear configuration structure */
    PDL_ZERO_STRUCT(stcCsio0Config);
    PDL_ZERO_STRUCT(stcCsio1Config);
    
    /* Initialize CSIO function I/O */    
    InitCsio0Io();
    InitCsio1Io();
    
    /* Initialize CSIO master  */
    stcCsio0Config.enMsMode = CsioMaster;
    stcCsio0Config.enActMode = CsioActNormalMode;
    stcCsio0Config.bInvertClk = FALSE;
    stcCsio0Config.u32BaudRate = 100000;
    stcCsio0Config.enDataLength = CsioEightBits;
    stcCsio0Config.enBitDirection = CsioDataMsbFirst;
    stcCsio0Config.enSyncWaitTime = CsioSyncWaitZero;
    stcCsio0Config.bEnableFifo = FALSE;
    stcCsio0Config.bEnChipSelection = FALSE;
    stcCsio0Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh0, &stcCsio0Config);
    
    /* Initialize CSIO slave  */
    stcCsio1Config.enMsMode = CsioSlave;
    stcCsio1Config.enActMode = CsioActNormalMode;
    stcCsio1Config.bInvertClk = FALSE;
    stcCsio1Config.u32BaudRate = 100000;
    stcCsio1Config.enDataLength = CsioEightBits;
    stcCsio1Config.enBitDirection = CsioDataMsbFirst;
    stcCsio1Config.bEnableFifo = FALSE;
    stcCsio1Config.bEnChipSelection = FALSE;
    stcCsio1Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh1, &stcCsio1Config);
    
    /*************************************************************************/
    /*                Master sends data to slave                             */
    /*************************************************************************/
    // Initialize and enable DMA
    MasterTxSlaveRxDmaInit();
    
    /* Enable RX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioRx);
    /* Enable TX function of CSIO0   */
    Mfs_Csio_EnableFunc(CsioCh0, CsioTx);
///////////// 1st test   
    // Initialize send data
    for(u8i=0; u8i<SAMPLE_CSIO_MASTER_TX_BUFFSIZE; u8i++)
    {
        au8CsioMasterTxBuf[u8i] = u8i;
    }
    // Clear DMA transfer finish flag
    bDma0Finished = FALSE;
    bDma1Finished = FALSE;
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    stcCsio1IntSel.bRxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    stcCsio0IntSel.bTxInt = TRUE;
    stcCsio0IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    // Wait for TX finish
    while(bDma0Finished != TRUE)
    {};
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle)); // wait for TX idle
    
    // Wait for RX finish
    while(bDma1Finished != TRUE)
    {};
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterTxBuf, au8CsioSlaveRxBuf, SAMPLE_CSIO_SLAVE_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }

///////////// 2nd test 
    // Initialize send data
    for(u8i=0; u8i<SAMPLE_CSIO_MASTER_TX_BUFFSIZE; u8i++)
    {
        au8CsioMasterTxBuf[u8i] = ~u8i;
    }
    
    // Clear DMA transfer finish flag
    bDma0Finished = FALSE;
    bDma1Finished = FALSE;
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    stcCsio1IntSel.bRxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    stcCsio0IntSel.bTxInt = TRUE;
    stcCsio0IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    // Wait for TX finish
    while(bDma0Finished != TRUE)
    {};
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle)); // wait for TX idle
    
    // Wait for RX finish
    while(bDma1Finished != TRUE)
    {};
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterTxBuf, au8CsioSlaveRxBuf, SAMPLE_CSIO_SLAVE_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(CsioCh0, CsioTx);
    /* Disable RX function of CSIO1   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioRx);
    
    MasterTxSlaveRxDmaDeInit();
    
    /*************************************************************************/
    /*                Master receives data from slave                        */
    /*************************************************************************/ 
    // Initialize and enable DMA
    MasterRxSlaveTxDmaInit();

    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioTx);
    
    /* Enable TX and RX function of CSIO0   */
    Mfs_Csio_EnableFunc(CsioCh0, CsioTx);
    Mfs_Csio_EnableFunc(CsioCh0, CsioRx);
    
////////// 1st test       
    // Initialize send data
    for(u8i=0; u8i<SAMPLE_CSIO_SLAVE_TX_BUFFSIZE; u8i++)
    {
        au8CsioSlaveTxBuf[u8i] = u8i;
    }
    // Clear DMA transfer finish flag
    bDma0Finished = FALSE;
    bDma1Finished = FALSE;
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    stcCsio0IntSel.bRxInt = TRUE;
    stcCsio0IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    // Wait for TX finish
    while(bDma1Finished != TRUE)
    {
        while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxEmpty));
        Mfs_Csio_SendData(CsioCh0, 0x00u, FALSE); // Dummy write until DMA interrupt occurs
    };
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle)); // wait for TX idle
    
    // Wait for RX finish
    while(bDma0Finished != TRUE)
    {};
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterRxBuf, au8CsioSlaveTxBuf, SAMPLE_CSIO_MASTER_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }

////////// 2nd test       
    // Initialize send data
    for(u8i=0; u8i<SAMPLE_CSIO_SLAVE_TX_BUFFSIZE; u8i++)
    {
        au8CsioSlaveTxBuf[u8i] = ~u8i;
    }
    // Clear DMA transfer finish flag
    bDma0Finished = FALSE;
    bDma1Finished = FALSE;
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    stcCsio0IntSel.bRxInt = TRUE;
    stcCsio0IntCb.pfnRxIntCb = DummyIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    // Wait for TX finish
    while(bDma1Finished != TRUE)
    {
        while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxEmpty));
        Mfs_Csio_SendData(CsioCh0, 0x00u, FALSE); // Dummy write until DMA interrupt occurs
    };
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle)); // wait for TX idle
    
    // Wait for RX finish
    while(bDma0Finished != TRUE)
    {};
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterRxBuf, au8CsioSlaveTxBuf, SAMPLE_CSIO_MASTER_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    /* Disable TX and RX function of CSIO0   */
    Mfs_Csio_DisableFunc(CsioCh0, CsioTx);
    Mfs_Csio_DisableFunc(CsioCh0, CsioRx);
    /* Disable RX function of CSIO1   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioRx);
    
    MasterRxSlaveTxDmaDeInit();
    
    Mfs_Csio_DeInit(CsioCh0);
    Mfs_Csio_DeInit(CsioCh1);
    
    while(1); /* Data is normally sent and received */
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
