/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates communication between CSIO0 and CSIO1 with 
 ** interrupt mode using FIFO. 
 ** Make following connection before using this example:
 ** Master             Slave
 ** SIN0_1  -------    SOT1_1
 ** SCK0_0  -------    SCK1_1
 ** SOT0_0  -------    SIN1_1
 ** 
 ** History:
 **   - 2014-03-10  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
#define SAMPLE_CSIO_SLAVE_RX_BUFFSIZE   sizeof(au8CsioMasterTxBuf)/sizeof(char)
#define SAMPLE_CSIO_MASTER_TX_BUFFSIZE  SAMPLE_CSIO_SLAVE_RX_BUFFSIZE

#define SAMPLE_CSIO_MASTER_RX_BUFFSIZE  sizeof(au8CsioSlaveTxBuf)/sizeof(char)
#define SAMPLE_CSIO_SLAVE_TX_BUFFSIZE   SAMPLE_CSIO_MASTER_RX_BUFFSIZE

#define SAMPLE_CSIO_FIFO_MAX_CAPACITY         (128u)
#define SAMPLE_CSIO_FIFO_RX_CNT               (8u)

#define InitCsio0Io(void)  {SetPinFunc_SIN0_1();SetPinFunc_SOT0_0();SetPinFunc_SCK0_0();}
#define InitCsio1Io(void)  {SetPinFunc_SIN1_1();SetPinFunc_SOT1_1();SetPinFunc_SCK1_1();}   
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
/// CSIO TX FIFO information structure
typedef struct stc_tx_fifo_info
{
    uint32_t u32TxCnt;
    uint8_t* pTxBuf;
    boolean_t bTxFinish;
    
}stc_tx_fifo_info_t;

/// UART RX FIFO information structure
typedef struct stc_rx_fifo_info
{
    uint32_t u32RxCnt;
    uint8_t* pRxBuf;
    boolean_t bRxFinish;
    
}stc_rx_fifo_info_t;

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/

/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
static volatile stc_mfsn_csio_t* CsioCh0 = &CSIO0;
static volatile stc_mfsn_csio_t* CsioCh1 = &CSIO1;
static stc_csio_int_sel_t stcCsio0IntSel, stcCsio1IntSel;
static stc_csio_int_cb_t stcCsio0IntCb, stcCsio1IntCb;
static uint8_t au8CsioMasterTxBuf[] = "CSIO master sends data to CSIO Slave";
static uint8_t au8CsioSlaveTxBuf[] = "CSIO master reads data from CSIO Slave";
static uint8_t au8CsioMasterRxBuf[SAMPLE_CSIO_MASTER_RX_BUFFSIZE];
static uint8_t au8CsioSlaveRxBuf[SAMPLE_CSIO_SLAVE_RX_BUFFSIZE];
static stc_tx_fifo_info_t stcTxFifoInfo = {0};
static stc_rx_fifo_info_t stcRxFifoInfo = {0};

/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2 
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint8_t u8Length)
{
    while(u8Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  CSIO master FIFO transfer interrupt callback function
 ******************************************************************************/
static void CsioMasterFifoTxIntCallback(void)
{
    uint8_t u8i = 0;
    if(stcTxFifoInfo.u32TxCnt > SAMPLE_CSIO_FIFO_MAX_CAPACITY)
    {
        while(u8i < SAMPLE_CSIO_FIFO_MAX_CAPACITY)
        {
            Mfs_Csio_SendData(CsioCh0, *stcTxFifoInfo.pTxBuf++, TRUE);
            u8i++;
        }
        stcTxFifoInfo.u32TxCnt -= SAMPLE_CSIO_FIFO_MAX_CAPACITY;
        return;
    }
    
    while(u8i < stcTxFifoInfo.u32TxCnt)
    {
        Mfs_Csio_SendData(CsioCh0, *stcTxFifoInfo.pTxBuf++, TRUE);
        u8i++;
    }
  
    stcCsio0IntSel.bTxFifoInt = TRUE;
    Mfs_Csio_DisableInt(CsioCh0, &stcCsio0IntSel);
    
    stcTxFifoInfo.bTxFinish = TRUE;
}

/**
 ******************************************************************************
 ** \brief  CSIO slave receive interrupt callback function
 ******************************************************************************/
static void CsioSlaveRxIntCallback(void)
{
    uint8_t u8i = 0;
    if(stcRxFifoInfo.u32RxCnt > SAMPLE_CSIO_FIFO_RX_CNT) 
    {
        /* Receive data when RX FIFO count match with SAMPLE_UART_FIFO_RX_CNT */
        while(u8i < SAMPLE_CSIO_FIFO_RX_CNT)
        {
            *stcRxFifoInfo.pRxBuf++ = Mfs_Csio_ReceiveData(CsioCh1); 
            u8i++;
        }
        stcRxFifoInfo.u32RxCnt -= SAMPLE_CSIO_FIFO_RX_CNT;
        return;
    }
    
    /* Receive data when RX FIFO is idle */
    /* idle means FIFO count is less than SAMPLE_UART_FIFO_RX_CNT and
       RX FIFO don't receive data from then on for a short time. */
    while(u8i < stcRxFifoInfo.u32RxCnt)
    {
        *stcRxFifoInfo.pRxBuf++ = Mfs_Csio_ReceiveData(CsioCh1);
        u8i++;
    }
    
    stcCsio1IntSel.bRxInt = TRUE;
    Mfs_Csio_DisableInt(CsioCh1, &stcCsio1IntSel);
    
    stcRxFifoInfo.bRxFinish = TRUE;
}

/**
 ******************************************************************************
 ** \brief  CSIO Master receive interrupt callback function
 ******************************************************************************/
static void CsioMasterRxIntCallback(void)
{
    uint8_t u8i = 0;
    if(stcRxFifoInfo.u32RxCnt > SAMPLE_CSIO_FIFO_RX_CNT) 
    {
        /* Receive data when RX FIFO count match with SAMPLE_UART_FIFO_RX_CNT */
        while(u8i < SAMPLE_CSIO_FIFO_RX_CNT)
        {
            *stcRxFifoInfo.pRxBuf++ = Mfs_Csio_ReceiveData(CsioCh0); 
            u8i++;
        }
        stcRxFifoInfo.u32RxCnt -= SAMPLE_CSIO_FIFO_RX_CNT;
        return;
    }
    
    /* Receive data when RX FIFO is idle */
    /* idle means FIFO count is less than SAMPLE_UART_FIFO_RX_CNT and
       RX FIFO don't receive data from then on for a short time. */
    while(u8i < stcRxFifoInfo.u32RxCnt)
    {
        *stcRxFifoInfo.pRxBuf++ = Mfs_Csio_ReceiveData(CsioCh0);
        u8i++;
    }
    
    stcCsio0IntSel.bRxInt = TRUE;
    Mfs_Csio_DisableInt(CsioCh0, &stcCsio0IntSel);
    
    stcRxFifoInfo.bRxFinish = TRUE;
}

/**
 ******************************************************************************
 ** \brief  CSIO slave FIFO transfer interrupt callback function
 ******************************************************************************/
static void CsioSlaveFifoTxIntCallback(void)
{
    uint8_t u8i = 0;
    if(stcTxFifoInfo.u32TxCnt > SAMPLE_CSIO_FIFO_MAX_CAPACITY)
    {
        while(u8i < SAMPLE_CSIO_FIFO_MAX_CAPACITY)
        {
            Mfs_Csio_SendData(CsioCh1, *stcTxFifoInfo.pTxBuf++, TRUE);
            u8i++;
        }
        stcTxFifoInfo.u32TxCnt -= SAMPLE_CSIO_FIFO_MAX_CAPACITY;
        return;
    }
    
    while(u8i < stcTxFifoInfo.u32TxCnt)
    {
        Mfs_Csio_SendData(CsioCh1, *stcTxFifoInfo.pTxBuf++, TRUE);
        u8i++;
    }
  
    stcCsio1IntSel.bTxFifoInt = TRUE;
    Mfs_Csio_DisableInt(CsioCh1, &stcCsio1IntSel);
    
    stcTxFifoInfo.bTxFinish = TRUE;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{   
    stc_mfs_csio_config_t stcCsio0Config, stcCsio1Config;
    uint8_t u8i;
    
    /* Clear configuration structure */
    PDL_ZERO_STRUCT(stcCsio0Config);
    PDL_ZERO_STRUCT(stcCsio1Config);
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    /* Initialize CSIO function I/O */    
    InitCsio0Io();
    InitCsio1Io();
    
    /* Initialize CSIO master  */
    stcCsio0Config.enMsMode = CsioMaster;
    stcCsio0Config.enActMode = CsioActNormalMode;
    stcCsio0Config.bInvertClk = FALSE;
    stcCsio0Config.u32BaudRate = 100000;
    stcCsio0Config.enDataLength = CsioEightBits;
    stcCsio0Config.enBitDirection = CsioDataMsbFirst;
    stcCsio0Config.enSyncWaitTime = CsioSyncWaitZero;
    stcCsio0Config.bEnableFifo = TRUE;
    stcCsio0Config.stcFifoConfig.enFifoSel = MfsTxFifo1RxFifo2;
    stcCsio0Config.stcFifoConfig.u8ByteCount1 = 0u;
    stcCsio0Config.stcFifoConfig.u8ByteCount2 = SAMPLE_CSIO_FIFO_RX_CNT;
    stcCsio0Config.bEnChipSelection = FALSE;
    stcCsio0Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh0, &stcCsio0Config);
    
    /* Initialize CSIO slave  */
    stcCsio1Config.enMsMode = CsioSlave;
    stcCsio1Config.enActMode = CsioActNormalMode;
    stcCsio1Config.bInvertClk = FALSE;
    stcCsio1Config.u32BaudRate = 100000;
    stcCsio1Config.enDataLength = CsioEightBits;
    stcCsio1Config.enBitDirection = CsioDataMsbFirst;
    stcCsio1Config.bEnableFifo = TRUE;
    stcCsio1Config.stcFifoConfig.enFifoSel = MfsTxFifo1RxFifo2;
    stcCsio1Config.stcFifoConfig.u8ByteCount1 = 0u;
    stcCsio1Config.stcFifoConfig.u8ByteCount2 = SAMPLE_CSIO_FIFO_RX_CNT;
    stcCsio1Config.bEnChipSelection = FALSE;
    stcCsio1Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh1, &stcCsio1Config);
    
    /*************************************************************************/
    /*                Master sends data to slave                             */
    /*************************************************************************/
    /* Initialize the FIFO information */
    // Master
    stcTxFifoInfo.u32TxCnt = SAMPLE_CSIO_MASTER_TX_BUFFSIZE;
    stcTxFifoInfo.pTxBuf = au8CsioMasterTxBuf;
    stcTxFifoInfo.bTxFinish = FALSE;
    // Slave
    stcRxFifoInfo.u32RxCnt = SAMPLE_CSIO_SLAVE_RX_BUFFSIZE;
    stcRxFifoInfo.pRxBuf = au8CsioSlaveRxBuf;
    stcRxFifoInfo.bRxFinish = FALSE;
    
    /* Enable RX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioRx);
    /* Enable TX function of CSIO0   */
    Mfs_Csio_EnableFunc(CsioCh0, CsioTx);
    
    /* Configure interrupt */
    stcCsio1IntSel.bRxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = CsioSlaveRxIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    stcCsio0IntSel.bTxFifoInt = TRUE;
    stcCsio0IntCb.pfnTxFifoIntCb = CsioMasterFifoTxIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    while(stcTxFifoInfo.bTxFinish != TRUE) // wait for Master TX finish
    {};
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle)); // wait TX idle
    
    while(stcRxFifoInfo.bRxFinish != TRUE) // wait for Slave RX finish
    {};
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterTxBuf, au8CsioSlaveRxBuf, SAMPLE_CSIO_SLAVE_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(CsioCh0, CsioTx);
    /* Disable RX function of CSIO1   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioRx);
    
    /*************************************************************************/
    /*                Master receives data from slave                        */
    /*************************************************************************/
    /* Initialize the FIFO information */
    // Master
    stcRxFifoInfo.u32RxCnt = SAMPLE_CSIO_MASTER_RX_BUFFSIZE;
    stcRxFifoInfo.pRxBuf = au8CsioMasterRxBuf;
    stcRxFifoInfo.bRxFinish = FALSE;
    
    // Slave
    stcTxFifoInfo.u32TxCnt = SAMPLE_CSIO_SLAVE_TX_BUFFSIZE;
    stcTxFifoInfo.pTxBuf = au8CsioSlaveTxBuf;
    stcTxFifoInfo.bTxFinish = FALSE;
    
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioTx);
    
    /* Enable TX and RX function of CSIO0   */
    Mfs_Csio_EnableFunc(CsioCh0, CsioTx);
    Mfs_Csio_EnableFunc(CsioCh0, CsioRx);
    
    /* Configure interrupt */
    stcCsio0IntSel.bRxInt = TRUE;
    stcCsio0IntCb.pfnRxIntCb = CsioMasterRxIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    stcCsio1IntSel.bTxFifoInt = TRUE;
    stcCsio1IntCb.pfnTxFifoIntCb = CsioSlaveFifoTxIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until slave finish writing FIFO */
    while(stcTxFifoInfo.bTxFinish != TRUE)
    {};
    
    /* Now data has been prepared in the Slave side */
    /* Read the data by writing data synchronously */
    u8i = 0;
    while(u8i < SAMPLE_CSIO_SLAVE_TX_BUFFSIZE)
    {
        while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxEmpty)); // wait TX idle
        Mfs_Csio_SendData(CsioCh0, 0x00u, FALSE);   /* Dummy write */
        u8i++;
    }
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle)); // wait TX idle
    
    /* Wait until Master finish reading FIFO */
    while(stcRxFifoInfo.bRxFinish != TRUE)
    {};
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterRxBuf, au8CsioSlaveTxBuf, SAMPLE_CSIO_MASTER_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    /* Disable TX and RX function of CSIO0   */
    Mfs_Csio_DisableFunc(CsioCh0, CsioTx);
    Mfs_Csio_DisableFunc(CsioCh0, CsioRx);
    /* Disable TX function of CSIO1   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioTx);
    
    Mfs_Csio_DeInit(CsioCh0);
    Mfs_Csio_DeInit(CsioCh1);
    
    while(1); /* Data is normally sent and received */
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
