/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates communication between CSIO0 and CSIO1 with 
 ** interrupt mode. 
 ** Make following connection before using this example:
 ** Master             Slave
 ** SIN0_1  -------    SOT1_1
 ** SCK0_0  -------    SCK1_1
 ** SOT0_0  -------    SIN1_1
 ** 
 ** History:
 **   - 2014-02-17  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
#define SAMPLE_CSIO_SLAVE_RX_BUFFSIZE   sizeof(au8CsioMasterTxBuf)/sizeof(char)
#define SAMPLE_CSIO_MASTER_TX_BUFFSIZE  SAMPLE_CSIO_SLAVE_RX_BUFFSIZE

#define SAMPLE_CSIO_MASTER_RX_BUFFSIZE  sizeof(au8CsioSlaveTxBuf)/sizeof(char)
#define SAMPLE_CSIO_SLAVE_TX_BUFFSIZE   SAMPLE_CSIO_MASTER_RX_BUFFSIZE

#define InitCsio0Io(void)  {SetPinFunc_SIN0_1();SetPinFunc_SOT0_0();SetPinFunc_SCK0_0();}
#define InitCsio1Io(void)  {SetPinFunc_SIN1_1();SetPinFunc_SOT1_1();SetPinFunc_SCK1_1();}   

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
volatile stc_mfsn_csio_t* CsioCh0 = &CSIO0;
volatile stc_mfsn_csio_t* CsioCh1 = &CSIO1;
stc_csio_int_sel_t stcCsio0IntSel, stcCsio1IntSel;
stc_csio_int_cb_t stcCsio0IntCb, stcCsio1IntCb;
uint32_t u32SendCnt = 0, u32ReceiveCnt = 0;
static uint8_t au8CsioMasterTxBuf[] = "CSIO master sends data to CSIO Slave";
static uint8_t au8CsioSlaveTxBuf[] = "CSIO master reads data from CSIO Slave";
static uint8_t au8CsioMasterRxBuf[SAMPLE_CSIO_MASTER_RX_BUFFSIZE];
static uint8_t au8CsioSlaveRxBuf[SAMPLE_CSIO_SLAVE_RX_BUFFSIZE];

/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2 
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint8_t u8Length)
{
    while(u8Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  CSIO master transfer interrupt callback function
 ******************************************************************************/
static void CsioMasterTxIntCallback(void)
{
    if(u32SendCnt == SAMPLE_CSIO_MASTER_TX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio0IntSel.bTxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh0, &stcCsio0IntSel);
        return;
    }
  
    Mfs_Csio_SendData(CsioCh0, au8CsioMasterTxBuf[u32SendCnt], TRUE);  
    u32SendCnt++;
}

/**
 ******************************************************************************
 ** \brief  CSIO slave receive interrupt callback function
 ******************************************************************************/
static void CsioSlaveRxIntCallback(void)
{
    au8CsioSlaveRxBuf[u32ReceiveCnt] = Mfs_Csio_ReceiveData(CsioCh1);
    u32ReceiveCnt++;
    
    if(u32ReceiveCnt == SAMPLE_CSIO_SLAVE_RX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio0IntSel.bRxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh1, &stcCsio1IntSel);
    }
}

/**
 ******************************************************************************
 ** \brief  CSIO Master receive interrupt callback function
 ******************************************************************************/
static void CsioMasterRxIntCallback(void)
{
    au8CsioMasterRxBuf[u32ReceiveCnt] = Mfs_Csio_ReceiveData(CsioCh0);
    u32ReceiveCnt++;
    
    if(u32ReceiveCnt == SAMPLE_CSIO_MASTER_RX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio0IntSel.bRxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh0, &stcCsio0IntSel);
        return;
    }
}

/**
 ******************************************************************************
 ** \brief  CSIO slave transfer interrupt callback function
 ******************************************************************************/
static void CsioSlaveTxIntCallback(void)
{
    if(u32SendCnt == SAMPLE_CSIO_SLAVE_TX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio1IntSel.bTxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh1, &stcCsio1IntSel);
        return;
    }
  
    Mfs_Csio_SendData(CsioCh1, au8CsioSlaveTxBuf[u32SendCnt], TRUE);  
    u32SendCnt++;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{   
    stc_mfs_csio_config_t stcCsio0Config, stcCsio1Config;
    
    /* Clear configuration structure */
    PDL_ZERO_STRUCT(stcCsio0Config);
    PDL_ZERO_STRUCT(stcCsio1Config);
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    /* Initialize CSIO function I/O */    
    InitCsio0Io();
    InitCsio1Io();

    u32SendCnt = 0;
    u32ReceiveCnt = 0;
    
    /* Initialize CSIO master  */
    stcCsio0Config.enMsMode = CsioMaster;
    stcCsio0Config.enActMode = CsioActNormalMode;
    stcCsio0Config.bInvertClk = FALSE;
    stcCsio0Config.u32BaudRate = 100000;
    stcCsio0Config.enDataLength = CsioEightBits;
    stcCsio0Config.enBitDirection = CsioDataMsbFirst;
    stcCsio0Config.enSyncWaitTime = CsioSyncWaitZero;
    stcCsio0Config.bEnableFifo = FALSE;
    stcCsio0Config.bEnChipSelection = FALSE;
    stcCsio0Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh0, &stcCsio0Config);
    
    /* Initialize CSIO slave  */
    stcCsio1Config.enMsMode = CsioSlave;
    stcCsio1Config.enActMode = CsioActNormalMode;
    stcCsio1Config.bInvertClk = FALSE;
    stcCsio1Config.u32BaudRate = 100000;
    stcCsio1Config.enDataLength = CsioEightBits;
    stcCsio1Config.enBitDirection = CsioDataMsbFirst;
    stcCsio1Config.bEnableFifo = FALSE;
    stcCsio1Config.bEnChipSelection = FALSE;
    stcCsio1Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh1, &stcCsio1Config);
    
    /*************************************************************************/
    /*                Master sends data to slave                             */
    /*************************************************************************/
    /* Configure interrupt */
    stcCsio1IntSel.bRxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = CsioSlaveRxIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    stcCsio0IntSel.bTxInt = TRUE;
    stcCsio0IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    /* Enable RX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioRx);
    /* Enable TX function of CSIO0   */
    Mfs_Csio_EnableFunc(CsioCh0, CsioTx);
    
    /* Wait for TX finish */
    while(u32SendCnt < SAMPLE_CSIO_MASTER_TX_BUFFSIZE)
    {
        ;
    }
    /* Wait for TX bus idle (until last data is sent) */
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle));
    
    /* Wait for RX finish */
    while(u32ReceiveCnt < SAMPLE_CSIO_SLAVE_RX_BUFFSIZE)
    {
        ;
    }
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterTxBuf, au8CsioSlaveRxBuf, SAMPLE_CSIO_SLAVE_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(CsioCh0, CsioTx);
    /* Disable RX function of CSIO1   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioRx);
    
    /*************************************************************************/
    /*                Master receives data from slave                        */
    /*************************************************************************/
    u32SendCnt = 0;
    u32ReceiveCnt = 0;
    PDL_ZERO_STRUCT(stcCsio0IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio0IntCb);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    
    /* Configure interrupt */
    stcCsio0IntSel.bRxInt = TRUE;
    stcCsio0IntCb.pfnRxIntCb = CsioMasterRxIntCallback;
    Mfs_Csio_EnableInt(CsioCh0, &stcCsio0IntSel, &stcCsio0IntCb);
    
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioSlaveTxIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioTx);
    
    /* Enable TX and RX function of CSIO0   */
    Mfs_Csio_EnableFunc(CsioCh0, CsioTx);
    Mfs_Csio_EnableFunc(CsioCh0, CsioRx);
 
    /* Send dummy data */
    while(u32SendCnt < SAMPLE_CSIO_SLAVE_TX_BUFFSIZE)
    {
        while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxEmpty));
        Mfs_Csio_SendData(CsioCh0, 0x00u, FALSE);   /* Dummy write */
    }
        
    /* Wait for Master TX bus idle (until last data is sent) */
    while(TRUE != Mfs_Csio_GetStatus(CsioCh0, CsioTxIdle));
    
    /* Wait for receive finish */
    while(u32ReceiveCnt < SAMPLE_CSIO_MASTER_RX_BUFFSIZE)
    {
        ;
    }
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterRxBuf, au8CsioSlaveTxBuf, SAMPLE_CSIO_MASTER_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    Mfs_Csio_DeInit(CsioCh0);
    Mfs_Csio_DeInit(CsioCh1);
    
    while(1); /* Data is normally sent and received */
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
