/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates access free run timer with peak/zero match
 ** interrupt
 ** History:
 **   - 2014-01-06  0.0.1  Edison Zhang    First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/// MFT channel
#define USER_MFT_CH      MFT0_FRT
/// FRT channel
#define USER_FRT_CH      MFT_FRT_CH0
/// FRT mode
#define FRT_MODE         FrtUpCount
/// FRT cycle time = 15000/(1/1024*40MHz)=1.536s
#define FRT_CYCLE        15000UL

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/*! \brief Interrupt count value */
static uint32_t m_u32CntInt;

/*! \brief Peak match flag */
static uint8_t m_u8PeakMatchFlag = 0;
static uint8_t m_u8ZeroMatchFlag = 0;

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
/*!
 ******************************************************************************
 ** \brief FRT peak match interrupt handler
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void FrtPeakIntCallback(void)
{
    m_u8PeakMatchFlag = 1;
}

/*!
 ******************************************************************************
 ** \brief FRT zero match interrupt handler
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void FrtZeroIntCallback(void)
{
    m_u8ZeroMatchFlag = 1;
}


/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series.
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    stc_mft_frt_config_t stcFrtConfig;
    stc_frt_int_sel_t stcFrtIntSel;
    stc_frt_int_cb_t stcFrtCallBack;

#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("FRT Sample Start \n");
    printf("==================================================\n");
#endif
    //configure the FRT parameter
    stcFrtConfig.enFrtMode = FrtUpCount;
    stcFrtConfig.enFrtClockDiv = FrtPclkDiv1024;
    stcFrtConfig.bEnExtClock = FALSE;
    stcFrtConfig.bEnBuffer = TRUE;

    //configure frt interrupt mode
    stcFrtIntSel.bFrtZeroMatchInt = TRUE; //enable zero match interrupt
    stcFrtIntSel.bFrtPeakMatchInt = TRUE; //enable peak match interrupt

    //configure callback function of frt
    stcFrtCallBack.pfnFrtPeakCallback = FrtPeakIntCallback;
    stcFrtCallBack.pfnFrtZeroCallback = FrtZeroIntCallback;

    Mft_Frt_Stop(&USER_MFT_CH ,USER_FRT_CH);
    //init frt module
    Mft_Frt_Init(&USER_MFT_CH ,USER_FRT_CH,&stcFrtConfig);
    //set frt cycle value
    Mft_Frt_SetCountCycle(&USER_MFT_CH ,USER_FRT_CH,FRT_CYCLE);
    //enable frt interrupt
    Mft_Frt_EnableInt(&USER_MFT_CH ,USER_FRT_CH,&stcFrtIntSel,&stcFrtCallBack);
    //start frt
    Mft_Frt_Start(&USER_MFT_CH ,USER_FRT_CH);

    m_u32CntInt = 0;
    while(1)
    {
        //zero match mode
        if(m_u8ZeroMatchFlag)
        {
            m_u32CntInt ++;
            m_u8ZeroMatchFlag = 0;
        #ifdef DEBUG_PRINT
            printf("Time %d: FRT Zero Match Interrupt occurs\n", m_u32CntInt);
        #endif
        }

        //peak match mode
        if(m_u8PeakMatchFlag)
        {
            m_u32CntInt ++;
            m_u8PeakMatchFlag = 0;
        #ifdef DEBUG_PRINT
            printf("Time %d: FRT Peak Match Interrupt occurs\n", m_u32CntInt);
        #endif
        }

        //stop FRT after occurred 10 times interrupt
        if(10 == m_u32CntInt)
        {
            Mft_Frt_Stop(&USER_MFT_CH ,USER_FRT_CH);
            Mft_Frt_DisableInt(&USER_MFT_CH ,USER_FRT_CH,&stcFrtIntSel);
            m_u8PeakMatchFlag = 0;
            m_u8ZeroMatchFlag = 0;
            m_u32CntInt = 0;
        #ifdef DEBUG_PRINT
            printf("==================================================\n");
            printf("FRT Sample Stop \n");
            printf("==================================================\n");
        #endif
        }
    }
}
