/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates ICU capture function with interrupt. 
 ** A pulse generated by a GPIO is input to ICU input pin. 
 **
 ** History:
 **   - 2014-02-23  0.1.0  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"
/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
//FRT
/*! \brief define Mft Frt Unit */
#define  USER_MFT_FRT            MFT0_FRT
/*! \brief define Mft Frt Channel */
#define  USER_MFT_FRT_CH         MFT_FRT_CH2
/*! \brief define Mft Frt cycle value */
#define  FRT_CYCLE_VAL           60000

//ICU
/*! \brief define Mft Icu Unit */
#define  USER_MFT_ICU            MFT0_ICU
/*! \brief define Mft Icu Channel */
#define  USER_MFT_ICU_CH         MFT_ICU_CH2
/*! \brief define GPIO function */
#define  InitIcuIo()             SetPinFunc_IC02_0() 
/*! \brief define Mft Frt Channelx to connect Icu */
#define  USER_ICU_SEL_FRT_CH     USER_MFT_FRT_CH

// Use GPIO to simulate a pulse
/*! \brief Initialize GPIO input to ICU pin */
#define  InitWaveGenIo()         Gpio1pin_InitOut(GPIO1PIN_P3A, Gpio1pin_InitDirectionOutput)
/*! \brief Polling GPIO pin */
#define  PollingWaveGenIo()      Gpio1pin_Put(GPIO1PIN_P3A, ~(Gpio1pin_Get(GPIO1PIN_P3A)))

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/*! \brief Mft Icu Interrupt count value */
static uint32_t m_u32CntIcuIntTrg;
/*! \brief Mft Frt Peak Interrupt count value */
static uint32_t m_u32CntFrtPeakIntTrg;
/*! \brief Mft Frt Peak Interrupt count value */
static uint32_t m_u32CntFrtZeroIntTrg;
/*! \brief Mft Icu captured rising edge number */
static uint32_t m_u32CntRisingEdgeTrg;
/*! \brief Mft Icu captured falling edge number */
static uint32_t m_u32CntFallingEdgeTrg;
/*! \brief Mft Icu captured edge type buffer */
static uint8_t m_au8EdgeTypeBuf[50];
/*! \brief Mft Icu captured data value buffer */
static uint32_t m_au16CapDataValBuf[50];
/*! \brief Mft Icu captured edge number */
static uint16_t m_u16CntCapEdgeNum;
/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/           
static void MftIcuIntHandler(void);
static void MftFrtPeakIntHandler(void);
static void MftFrtZeroIntHandler(void);
static void MftFrtParaConfig(void);
static void Delay(void);
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/*
 ******************************************************************************
 ** \brief Configure Mft Frt parameter values
 ******************************************************************************
 */
static void MftFrtParaConfig(void)
{
    stc_mft_frt_config_t stcMftFrtConfig;
    stc_frt_int_sel_t   stcMftFrtIntSel;
    stc_frt_int_cb_t    stcMftFrtIntCallback;
    
    stcMftFrtConfig.enFrtClockDiv = FrtPclkDiv128;
    stcMftFrtConfig.enFrtMode = FrtUpCount;
    stcMftFrtConfig.bEnBuffer = FALSE;
    stcMftFrtConfig.bEnExtClock = FALSE;
    
    stcMftFrtIntSel.bFrtPeakMatchInt = TRUE;
    stcMftFrtIntSel.bFrtZeroMatchInt = TRUE;
    stcMftFrtIntCallback.pfnFrtPeakCallback = MftFrtPeakIntHandler;
    stcMftFrtIntCallback.pfnFrtZeroCallback = MftFrtZeroIntHandler;
    
    // Set initial parameter
    Mft_Frt_Init(&USER_MFT_FRT, USER_MFT_FRT_CH, &stcMftFrtConfig);
    // Set Frt count cycle
    Mft_Frt_SetCountCycle(&USER_MFT_FRT, USER_MFT_FRT_CH, FRT_CYCLE_VAL);
    // Enable interrupt
    Mft_Frt_EnableInt(&USER_MFT_FRT, USER_MFT_FRT_CH,
                      &stcMftFrtIntSel, &stcMftFrtIntCallback);
}
/*!
 ******************************************************************************
 ** \brief Mft Icu example code
 **
 ** 1. Set GPIO function to Icu
 ** 2. Configure Frt&Icu parameter
 ** 3. Enable Icu interrupt
 ** 4. Start Frt&Icu operation
 ** 5. Stop  Frt&Icu operation, disable interrupt
 ** 6. print Icu detected information
 ******************************************************************************
 */
int32_t main(void)
{
    uint16_t u16CntPrint = 0;
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Mft Icu Example Program Start \n");
    printf("==================================================\n");
    
#endif
    // Initialize Gpio to generate the pulse which inputs to ICU
    InitWaveGenIo();

    // Set Mft Icu IO port
    InitIcuIo();
    
    // Initialize Mft Frt module
    MftFrtParaConfig();
    
    // Select Frt channel
    Mft_Icu_SelFrt(&USER_MFT_ICU, USER_MFT_ICU_CH, (en_mft_icu_frt_t)USER_ICU_SEL_FRT_CH);
    // Disable Icu operation
    Mft_Icu_ConfigDetectMode(&USER_MFT_ICU, USER_MFT_ICU_CH, IcuDisable);
    // Enable Interrupt
    Mft_Icu_EnableInt(&USER_MFT_ICU, USER_MFT_ICU_CH, MftIcuIntHandler);

    // Initialize trig counters
    m_u32CntIcuIntTrg = 0;
    m_u32CntRisingEdgeTrg = 0;
    m_u32CntFallingEdgeTrg = 0;

#ifdef  DEBUG_PRINT
    printf("==================================================\n");
    printf("Enable Mft Icu Rising Edge Detection\n");
#endif
    // Start Frt operation
    Mft_Frt_Start(&USER_MFT_FRT,USER_MFT_FRT_CH);
    
    // Start Icu operation, Set detecting rising edge mode
    Mft_Icu_ConfigDetectMode(&USER_MFT_ICU, USER_MFT_ICU_CH, IcuRisingDetect);
    Delay();
    while(10 > m_u32CntIcuIntTrg)
    {
	PollingWaveGenIo();
        Delay();
    }

    // Stop FRT
    Mft_Frt_Stop(&USER_MFT_FRT,USER_MFT_FRT_CH);
    // Disable Mft Icu Operation and set mode to IcuDisable
    Mft_Icu_ConfigDetectMode(&USER_MFT_ICU,USER_MFT_ICU_CH,IcuDisable);

#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Disable Mft Icu Rising Edge Detection\n");
    printf("==================================================\n");
    printf("Interrupt request times:      %d\n", m_u32CntIcuIntTrg);
    printf("Mft Icu Rising Edge trigger : %d\n", m_u32CntRisingEdgeTrg);
    printf("Mft Icu Falling Edge trigger: %d\n", m_u32CntFallingEdgeTrg);
    for(u16CntPrint = 0; u16CntPrint < 10; u16CntPrint++)
    {
        printf("Captured Edge Type : %d\n", m_au8EdgeTypeBuf[u16CntPrint]);
        printf("Captured Data Value: %d\n", m_au16CapDataValBuf[u16CntPrint]);
    }
#endif
    // Clear print counter buffer and related variables
    for(u16CntPrint = 0; u16CntPrint < 10; u16CntPrint++)
    {
        m_au8EdgeTypeBuf[u16CntPrint] = 0;
    }
    m_u32CntIcuIntTrg = 0;
    m_u32CntRisingEdgeTrg = 0;
    m_u32CntFallingEdgeTrg = 0;
    m_u16CntCapEdgeNum = 0;
    
/********************************************************************/
#ifdef  DEBUG_PRINT
    printf("==================================================\n");
    printf("Enable Mft Icu Falling Edge Detection\n");
#endif
    // Start Frt operation
    Mft_Frt_Start(&USER_MFT_FRT,USER_MFT_FRT_CH);
    
    // Start Icu operation, Set detecting rising edge mode
    Mft_Icu_ConfigDetectMode(&USER_MFT_ICU, USER_MFT_ICU_CH, IcuFallingDetect);
    Delay();
    while(10 > m_u32CntIcuIntTrg)
    {
	    PollingWaveGenIo();
        Delay();
    }

    // Stop FRT
    Mft_Frt_Stop(&USER_MFT_FRT,USER_MFT_FRT_CH);
    // Disable Mft Icu Operation and set mode to IcuDisable
    Mft_Icu_ConfigDetectMode(&USER_MFT_ICU,USER_MFT_ICU_CH,IcuDisable);

#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Disable Mft Icu Falling Edge Detection\n");
    printf("==================================================\n");
    printf("Interrupt request times:      %d\n", m_u32CntIcuIntTrg);
    printf("Mft Icu Rising Edge trigger : %d\n", m_u32CntRisingEdgeTrg);
    printf("Mft Icu Falling Edge trigger: %d\n", m_u32CntFallingEdgeTrg);
    for(u16CntPrint = 0; u16CntPrint < 10; u16CntPrint++)
    {
        printf("Captured Edge Type : %d\n", m_au8EdgeTypeBuf[u16CntPrint]);
        printf("Captured Data Value: %d\n", m_au16CapDataValBuf[u16CntPrint]);
    }
#endif
    // Clear print counter buffer and related variables
    for(u16CntPrint = 0; u16CntPrint < 10; u16CntPrint++)
    {
        m_au8EdgeTypeBuf[u16CntPrint] = 0;
    }
    m_u32CntIcuIntTrg = 0;
    m_u32CntRisingEdgeTrg = 0;
    m_u32CntFallingEdgeTrg = 0;
    m_u16CntCapEdgeNum = 0;
    
/****************************************************************************/
#ifdef  DEBUG_PRINT
    printf("==================================================\n");
    printf("Enable Mft Icu Both Edge Detection\n");
#endif
    // Start Frt operation
    Mft_Frt_Start(&USER_MFT_FRT,USER_MFT_FRT_CH);
    
    // Start Icu operation, Set detecting rising edge mode
    Mft_Icu_ConfigDetectMode(&USER_MFT_ICU, USER_MFT_ICU_CH, IcuBothDetect);
    Delay();
    while(10 > m_u32CntIcuIntTrg)
    {
	    PollingWaveGenIo();
        Delay();
    }

    // Stop FRT
    Mft_Frt_Stop(&USER_MFT_FRT,USER_MFT_FRT_CH);
    // Disable Mft Icu Operation and set mode to IcuDisable
    Mft_Icu_ConfigDetectMode(&USER_MFT_ICU,USER_MFT_ICU_CH,IcuDisable);

#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Disable Mft Icu Both Edge Detection\n");
    printf("==================================================\n");
    printf("Interrupt request times:      %d\n", m_u32CntIcuIntTrg);
    printf("Mft Icu Rising Edge trigger : %d\n", m_u32CntRisingEdgeTrg);
    printf("Mft Icu Falling Edge trigger: %d\n", m_u32CntFallingEdgeTrg);
    for(u16CntPrint = 0; u16CntPrint < 10; u16CntPrint++)
    {
        printf("Captured Edge Type : %d\n", m_au8EdgeTypeBuf[u16CntPrint]);
        printf("Captured Data Value: %d\n", m_au16CapDataValBuf[u16CntPrint]);
    }
#endif
    // Clear print counter buffer and related variables
    for(u16CntPrint = 0; u16CntPrint < 10; u16CntPrint++)
    {
        m_au8EdgeTypeBuf[u16CntPrint] = 0;
    }
    m_u32CntIcuIntTrg = 0;
    m_u32CntRisingEdgeTrg = 0;
    m_u32CntFallingEdgeTrg = 0;

#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Mft Icu Example Program End \n");
    printf("==================================================\n");
#endif
        
    while(1);
    
}

/*!
 ******************************************************************************
 ** \brief Mft Icu trigger interrupt handler
 ******************************************************************************
 */
static void MftIcuIntHandler(void)
{
    m_u32CntIcuIntTrg++;

    m_au8EdgeTypeBuf[m_u16CntCapEdgeNum] = Mft_Icu_GetLastEdge(&USER_MFT_ICU, USER_MFT_ICU_CH);
    // this is just a demo
    // if absolute captured data value is needed
    // please add Frt counter value: m_u32CntFrtPeakIntTrg*65536
    m_au16CapDataValBuf[m_u16CntCapEdgeNum] = Mft_Icu_GetCaptureData(&USER_MFT_ICU, USER_MFT_ICU_CH);
    
    if(IcuFallingEdge == m_au8EdgeTypeBuf[m_u16CntCapEdgeNum])
    {
        m_u32CntFallingEdgeTrg++;
    }
    else if(IcuRisingEdge == m_au8EdgeTypeBuf[m_u16CntCapEdgeNum])
    {
        m_u32CntRisingEdgeTrg++;
    }
    m_u16CntCapEdgeNum++;
}
/*!
 ******************************************************************************
 ** \brief Mft Frt callback function
 ******************************************************************************
 */
static void MftFrtPeakIntHandler(void)
{
    m_u32CntFrtPeakIntTrg++;
    
}
/*!
 ******************************************************************************
 ** \brief Mft Frt callback function
 ******************************************************************************
 */
static void MftFrtZeroIntHandler(void)
{
    m_u32CntFrtZeroIntTrg++;
    
}
/*!
 ******************************************************************************
 ** \brief time delay function
 ******************************************************************************
 */
static void Delay(void)
{
    uint32_t u32Cnt = 10000;
    while (0 != (u32Cnt--));
}
/*****************************************************************************/
/* END OF FILE */
