/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates 2 channel linked mode of OCU.
 **
 ** History:
 **   - 2014-01-08  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/*  FRT module  */
// MFT channel
#define USER_MFT_FRT          MFT0_FRT
// FRT 0
#define USER_FRT_CH0          MFT_FRT_CH0
#define USER_FRT_CH0_MOD      FrtUpCount
// FRT cycle
#define FRT_CYCLE       60000  

/*  OCU module  */
// MFT channel
#define USER_MFT_OCU        MFT0_OCU
// OCU channel
#define USER_OCU_CH0    MFT_OCU_CH0
#define USER_OCU_CH1    MFT_OCU_CH1
// OCU compare value
#define USER_OCU_CH0_CMPVAL1 (20000)
#define USER_OCU_CH1_CMPVAL1 (40000)


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint8_t m_u8OcuMatchFlag[2] = {0};
static uint8_t m_u8Ch1Level = 0;

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
/**
 ******************************************************************************
 ** \brief  OCU channel 0 interrupt callback function   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu0IntCallback(void)
{
    if(m_u8Ch1Level != Mft_Ocu_GetRtPinLevel(&USER_MFT_OCU, USER_OCU_CH1))
    {
        m_u8OcuMatchFlag[0] = 1;
    }
    m_u8Ch1Level = Mft_Ocu_GetRtPinLevel(&USER_MFT_OCU, USER_OCU_CH1);
}

/**
 ******************************************************************************
 ** \brief  OCU channel 1 interrupt callback function  .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu1IntCallback(void)
{
    if(m_u8Ch1Level != Mft_Ocu_GetRtPinLevel(&USER_MFT_OCU, USER_OCU_CH1))
    {
        m_u8OcuMatchFlag[1] = 1;
    }
    m_u8Ch1Level = Mft_Ocu_GetRtPinLevel(&USER_MFT_OCU, USER_OCU_CH1);
}

/**
 ******************************************************************************
 ** \brief  Init user free run timer   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void InitFrt(void)
{
    stc_mft_frt_config_t stcFrtConfig0; //channel 1 configure structure
    
    //configure the FRT channel 0 parameter
    stcFrtConfig0.enFrtMode = USER_FRT_CH0_MOD;
    stcFrtConfig0.enFrtClockDiv = FrtPclkDiv1024;
    stcFrtConfig0.bEnExtClock = FALSE;
    stcFrtConfig0.bEnBuffer = TRUE;

    //stop frt
    Mft_Frt_Stop(&USER_MFT_FRT,USER_FRT_CH0);
    //init frt module
    Mft_Frt_Init(&USER_MFT_FRT,USER_FRT_CH0,&stcFrtConfig0);
    //set frt cycle value
    Mft_Frt_SetCountCycle(&USER_MFT_FRT,USER_FRT_CH0,FRT_CYCLE);
}

/**
 ******************************************************************************
 ** \brief  Init user output compare unit  .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void InitOcu(void)
{
    stc_mft_ocu_config_t stcOcuConfig0;
    stc_even_compare_config_t stcEvenCompareConfig;
    stc_odd_compare_config_t  stcOddCompareConfig;
    
    /*configure the OCU parameter*/
    stcOcuConfig0.enFrtConnect = Frt0ToOcu;
    stcOcuConfig0.bFm4 = TRUE;
    stcOcuConfig0.bCmod = FALSE;
    stcOcuConfig0.u8Mod = 0;
    stcOcuConfig0.enStatePin = RtLowLevel;
    stcOcuConfig0.enOccpBufMode = OccpBufDisable;
    stcOcuConfig0.enOcseBufMode = OcseBufDisable;

    //init OCU
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH0,&stcOcuConfig0);
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH1,&stcOcuConfig0);

    //write OCU compare value buffer
    stcEvenCompareConfig.enFrtZeroEvenMatchEvenChRtStatus = RtOutputHold;
    stcEvenCompareConfig.enFrtZeroEvenNotMatchEvenChRtStatus = RtOutputHold;
    stcEvenCompareConfig.enFrtUpCntEvenMatchEvenChRtStatus = RtOutputHold;
    stcEvenCompareConfig.enFrtPeakEvenMatchEvenChRtStatus = RtOutputHold;
    stcEvenCompareConfig.enFrtPeakEvenNotMatchEvenChStatus = RtOutputHold;
    stcEvenCompareConfig.enFrtDownCntEvenMatchEvenChRtStatus = RtOutputHold;
    stcEvenCompareConfig.enIopFlagWhenFrtZeroEvenMatch = IopFlagSet;
    stcEvenCompareConfig.enIopFlagWhenFrtUpCntEvenMatch = IopFlagSet;
    stcEvenCompareConfig.enIopFlagWhenFrtPeakEvenMatch = IopFlagSet;
    stcEvenCompareConfig.enIopFlagWhenFrtDownCntEvenMatch = IopFlagSet;
    
    stcOddCompareConfig.enFrtZeroOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtZeroOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtZeroOddNotMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcOddCompareConfig.enFrtUpCntOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtUpCntOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtUpCntOddNotMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtPeakOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtPeakOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtPeakOddNotMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcOddCompareConfig.enFrtDownOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtDownOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enFrtDownOddNotMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcOddCompareConfig.enIopFlagWhenFrtZeroOddMatch = IopFlagSet;
    stcOddCompareConfig.enIopFlagWhenFrtUpCntOddMatch = IopFlagSet;
    stcOddCompareConfig.enIopFlagWhenFrtPeakOddMatch = IopFlagSet;
    stcOddCompareConfig.enIopFlagWhenFrtDownCntOddMatch = IopFlagSet;
    
    Mft_Ocu_SetEvenChCompareMode(&USER_MFT_OCU, USER_OCU_CH0, &stcEvenCompareConfig);
    Mft_Ocu_SetOddChCompareMode(&USER_MFT_OCU, USER_OCU_CH1, &stcOddCompareConfig);
    
    //write OCU compare value
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH0,USER_OCU_CH0_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH1,USER_OCU_CH1_CMPVAL1);

    //Enable OCU interrupt
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH0,Ocu0IntCallback);
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH1,Ocu1IntCallback);
}
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series.
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
#ifdef DEBUG_PRINT
    printf("===============================================================\n");
    printf("          ocu_ch_linked Sample Start \n");
    printf("OCU0 and OCU1 connect to FRT0 and running \n");
    printf("with 2 channel linked mode\n");
    printf("===============================================================\n");
#endif
    //init FRT
    InitFrt();
    //init OCU
    InitOcu();

    //start FRT
    Mft_Frt_Start(&USER_MFT_FRT,USER_FRT_CH0);
    //enable OCU operation
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH0);
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH1);

    while(1)
    {
        if(m_u8OcuMatchFlag[0] == 1)
        { 
            m_u8OcuMatchFlag[0] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT0 counts at %d)\n", USER_OCU_CH1, USER_OCU_CH0_CMPVAL1);
        #endif
        }

        if(m_u8OcuMatchFlag[1] == 1)
        {
            m_u8OcuMatchFlag[1] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT0 counts at %d)\n", USER_OCU_CH1, USER_OCU_CH1_CMPVAL1);
        #endif
        }

    }
}
