/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates PC overflow trigger mode of Revolution Counter.
 **
 ** History:
 **   - 2014-01-28  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*------------------------- FRT ----------------------------*/
/*! \brief define Mft Frt Unit */
#define  USER_FRT             (MFT0_FRT)
/*! \brief define Mft Frt Channel */
#define  USER_FRT_CH          (MFT_FRT_CH0)

/*------------------------- Ocu ----------------------------*/
/*! \brief define Mft Ocu Unit */
#define  USER_OCU           (MFT0_OCU)
/*! \brief define Mft Ocu Channel */
#define  USER_OCU_CH0       (MFT_OCU_CH0)
#define  USER_OCU_CH1       (MFT_OCU_CH1)

#define  USER_OCU_CH0_CMPVAL1   (2000)  ///< OCU Ch0 compare value 1
#define  USER_OCU_CH0_CMPVAL2   (6000)  ///< OCU Ch0 compare value 2
#define  USER_OCU_CH1_CMPVAL1   (4000)  ///< OCU Ch1 compare value 1
#define  USER_OCU_CH1_CMPVAL2   (8000)  ///< OCU Ch1 compare value 2

/*------------------------ Wfg ----------------------------*/
/*! \brief define Mft Wfg Unit */
#define  USER_WFG             (MFT0_WFG)
/*! \brief define Mft Wfg Channel */
#define  USER_WFG_COUPLE_CH   (MFT_WFG_CH10)

/*------------------------ Qprc ---------------------------*/
/*! \brief define Qprc Unit */
#define  USER_QPRC            (QPRC0)
/*----------------------- GPIO -----------------------------*/
#define InitMftIo()         {SetPinFunc_RTO00_0(); SetPinFunc_RTO01_0();} 
#define InitQprcIo()        {SetPinFunc_AIN0_0();SetPinFunc_BIN0_0();SetPinFunc_ZIN0_0();}

/* Key */
#define InitKey()           Gpio1pin_InitIn(GPIO1PIN_P0F, Gpio1pin_InitDirectionInput)    
#define GetKey()            Gpio1pin_Get(GPIO1PIN_P0F) 

/*  ZIN trigger IO */
#define InitZinTriggerIo()  Gpio1pin_InitOut(GPIO1PIN_P14, Gpio1pin_InitDirectionOutput)
#define SetZinTriggerIo()   Gpio1pin_Put(GPIO1PIN_P14, 1)
#define ClrZinTriggerIo()   Gpio1pin_Put(GPIO1PIN_P14, 0)

/*---------------------------------------------------------------------------*/
/* Local data                                                                */
/*---------------------------------------------------------------------------*/
/*! \brief OCU match count */
static uint8_t  m_au8OcuMatchCnt[2] = {0};
/*! \brief PC overflow interrupt flag */
static uint8_t  m_u8PcOfFlag = 0;
/*! \brief PC underflow interrupt flag */
static uint8_t  m_u8PcUfFlag = 0;
/*! \brief PC underflow interrupt flag */
static uint8_t  m_u8PcMatchFlag = 0;
/*! \brief RC underflow interrupt flag */
static uint8_t  m_u8RcMatchFlag = 0;
/*! \brief RC comapre value */
static uint32_t m_u32RcCmpVal = 1;

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
 ******************************************************************************
 ** \brief FRT initialization
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void FrtConfig(void)
{
    stc_mft_frt_config_t stcMftFrtConfig;
    
    //Stop Frt count, and init count value  0
    Mft_Frt_Stop(&USER_FRT, USER_FRT_CH);
    Mft_Frt_SetCountVal(&USER_FRT, USER_FRT_CH, 0);

    stcMftFrtConfig.enFrtClockDiv = FrtPclkDiv64;
    stcMftFrtConfig.enFrtMode = FrtUpCount;
    stcMftFrtConfig.bEnBuffer = FALSE;
    stcMftFrtConfig.bEnExtClock = FALSE;

    // Set initial parameter
    Mft_Frt_Init(&USER_FRT, USER_FRT_CH, &stcMftFrtConfig);

    // Set Frt count cycle
    Mft_Frt_SetCountCycle(&USER_FRT, USER_FRT_CH, 10000);

    return;
}

/*!
 ******************************************************************************
 ** \brief OCU Ch0 interrupt callback
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void Ocu0IntCallback(void)
{
    if (0 == m_au8OcuMatchCnt[0])
    {
        Mft_Ocu_WriteOccp(&USER_OCU, USER_OCU_CH0, USER_OCU_CH0_CMPVAL2);
    }
    else
    {
        Mft_Ocu_WriteOccp(&USER_OCU, USER_OCU_CH0, USER_OCU_CH0_CMPVAL1);
    }

    m_au8OcuMatchCnt[0] = !m_au8OcuMatchCnt[0];

    return;
}

/*!
 ******************************************************************************
 ** \brief OCU Ch1 interrupt callback
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void Ocu1IntCallback(void)
{
    if (0 == m_au8OcuMatchCnt[1])
    {
        Mft_Ocu_WriteOccp(&USER_OCU, USER_OCU_CH1, USER_OCU_CH1_CMPVAL2);
    }
    else
    {
        Mft_Ocu_WriteOccp(&USER_OCU, USER_OCU_CH1, USER_OCU_CH1_CMPVAL1);
    }

    m_au8OcuMatchCnt[1] = !m_au8OcuMatchCnt[1];

    return;
}

/*!
 ******************************************************************************
 ** \brief OCU initialization
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void OcuConfig(void)
{
    stc_mft_ocu_config_t stcOcuConfig;
    stc_even_compare_config_t stcCh0CompareConfig;
    stc_odd_compare_config_t  stcCh1CompareConfig;

    stcOcuConfig.enFrtConnect = (en_mft_ocu_frt_t)USER_FRT_CH;  //select the FRT to be connected to OCU
    stcOcuConfig.bFm4 = TRUE;  // select FM0P mode or FM3-compatible mode
    stcOcuConfig.bCmod = FALSE; // FM3 compatible mode
    stcOcuConfig.u8Mod = 0;     // select OCU ch0~ch5 in FM3 compatible
    stcOcuConfig.enStatePin = RtLowLevel; // initial low level
    stcOcuConfig.enOccpBufMode = OccpBufDisable;  // buffer register function of OCCP
    stcOcuConfig.enOcseBufMode = OcseBufDisable;  // buffer register function of OCSE

    /* Disable&&Init   OCU_CH0&OCU_CH1 */
    Mft_Ocu_DisableOperation(&USER_OCU, USER_OCU_CH0);
    Mft_Ocu_DisableOperation(&USER_OCU, USER_OCU_CH1);
    Mft_Ocu_Init(&USER_OCU, USER_OCU_CH0, &stcOcuConfig);
    Mft_Ocu_Init(&USER_OCU, USER_OCU_CH1, &stcOcuConfig);

    //1 change mode
    //OCSE0:  0x00000FFF 
    //OCSE1:  0x0FF00FFF 
    stcCh0CompareConfig.enFrtZeroEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtZeroEvenNotMatchEvenChRtStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtUpCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenNotMatchEvenChStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtDownCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enIopFlagWhenFrtZeroEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtUpCntEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtPeakEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtDownCntEvenMatch = IopFlagSet;
    
    stcCh1CompareConfig.enFrtZeroOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtDownOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enIopFlagWhenFrtZeroOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtUpCntOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtPeakOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtDownCntOddMatch = IopFlagSet;
    
    Mft_Ocu_SetEvenChCompareMode(&USER_OCU, USER_OCU_CH0, &stcCh0CompareConfig);
    Mft_Ocu_SetOddChCompareMode(&USER_OCU, USER_OCU_CH1, &stcCh1CompareConfig);

    //write OCU compare value
    Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH0,USER_OCU_CH0_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH1,USER_OCU_CH1_CMPVAL1);

    //Enable OCU interrupt
    Mft_Ocu_EnableInt(&USER_OCU,USER_OCU_CH0,Ocu0IntCallback);
    Mft_Ocu_EnableInt(&USER_OCU,USER_OCU_CH1,Ocu1IntCallback);
    
    return;
}

/*!
 ******************************************************************************
 ** \brief WFG initialization
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void WfgConfig(void)
{
    stc_wfg_ctrl_bits_t stcCtrlBits;

    InitMftIo();

    /* Set through mode */
    stcCtrlBits.enGtenBits = GtenBits00B;
    stcCtrlBits.enPselBits = PselBits00B;
    stcCtrlBits.enPgenBits = PgenBits00B;
    stcCtrlBits.enDmodBit  = DmodBit0B;

    Mft_Wfg_ConfigCtrlBits(&USER_WFG, USER_WFG_COUPLE_CH, &stcCtrlBits);
    Mft_Wfg_ConfigMode(&USER_WFG, USER_WFG_COUPLE_CH, WfgThroughMode);

    return;
}

/*!
 ******************************************************************************
 ** \brief Pc match interrupt handler
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void PcMatchIntHandler(void)
{
    m_u8PcMatchFlag = 1;

    return;
}

/*!
 ******************************************************************************
 ** \brief  PC status interrupt function
 **
 ** \param [in]  u8IntType   QPRC_PC_OVERFLOW_INT
 **                                   QPRC_PC_UNDERFLOW_INT
 **                                   QPRC_PC_ZERO_INDEX_INT
 ** \retval None
 **
 ******************************************************************************
 */
static void PcUfOfZeroIntHandler(uint8_t u8IntType)
{
    if (QPRC_PC_OVERFLOW_INT == u8IntType)
    {
        m_u8PcOfFlag = 1;
    }
    else if (QPRC_PC_UNDERFLOW_INT == u8IntType)
    {
        m_u8PcUfFlag = 1;
    }

    return;
}

/*!
 ******************************************************************************
 ** \brief  RC Match interrupt function
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void RcMatchIntHandler(void)
{
    m_u8RcMatchFlag = 1;
    m_u32RcCmpVal++;
    Qprc_SetPcRcCompareValue(&USER_QPRC, m_u32RcCmpVal);

    return;
}

/*!
 ******************************************************************************
 ** \brief QPRC initialization
 **
 ** \param None
 **
 ** \retval None
 **
 ******************************************************************************
 */
static void QprcConfig(void)
{
    stc_qprc_config_t stcQprcConfig;
    stc_qprc_int_sel_t   stcQprcIntSel;
    stc_qprc_int_cb_t    stcQprcIntCallback;

    /* Enable AIN, BIN, ZIN */
    InitQprcIo();

    /*Initialize QPRC*/
    stcQprcConfig.bSwapAinBin   = FALSE; //TRUE: Swap AIN and BIN inputs
    stcQprcConfig.enComapreMode = QprcComapreWithRevolution;
    stcQprcConfig.enAinEdge = QprcAinRisingEdge;
    stcQprcConfig.enBinEdge = QprcBinRisingEdge;
    stcQprcConfig.enZinEdge = QprcZinFallingEdge;

    stcQprcConfig.enPcResetMask = QprcResetMaskDisable; // Position counter reset mask
    stcQprcConfig.b8KValue      = FALSE; //TRUE: Outrange mode from 0 to 0x7FFF, FALSE: Outrange mode from 0 to 0xFFFF:

    stcQprcConfig.stcAinFilter.bInputMask   = FALSE; //No input mask setting
    stcQprcConfig.stcAinFilter.bInputInvert = FALSE; //No invert mask setting
    stcQprcConfig.stcAinFilter.enWidth      = QprcNoFilter; //No filter

    stcQprcConfig.stcBinFilter.bInputMask   = FALSE;
    stcQprcConfig.stcBinFilter.bInputInvert = FALSE;
    stcQprcConfig.stcBinFilter.enWidth      = QprcNoFilter;

    stcQprcConfig.stcCinFilter.bInputMask   = FALSE;
    stcQprcConfig.stcCinFilter.bInputInvert = FALSE;
    stcQprcConfig.stcCinFilter.enWidth      = QprcNoFilter;

    Qprc_Init(&USER_QPRC, &stcQprcConfig);

    /* Disable position counter &&  revolution counter*/
    Qprc_ConfigPcMode(&USER_QPRC, QprcPcMode0); // PC_Mode0:Disable position counter
    Qprc_ConfigRcMode(&USER_QPRC, QprcRcMode0); // RC_Mode0: Disable revolution counter

    /* Set Value */
    Qprc_SetPcCompareValue(&USER_QPRC, 100);
    Qprc_SetPcRcCompareValue(&USER_QPRC, m_u32RcCmpVal);
    Qprc_SetPcMaxValue(&USER_QPRC, 200);

    /* Clear PC count, interrupt flag */
    Qprc_SetPcCount(&USER_QPRC, 0);
    Qprc_ClrIntFlag(&USER_QPRC, QprcPcOfUfZeroInt);
    Qprc_ClrIntFlag(&USER_QPRC, QprcPcMatchInt);
    Qprc_ClrIntFlag(&USER_QPRC, QprcPcRcMatchInt);
    Qprc_ClrIntFlag(&USER_QPRC, QprcPcMatchRcMatchInt);
    Qprc_ClrIntFlag(&USER_QPRC, QprcPcCountInvertInt);
    Qprc_ClrIntFlag(&USER_QPRC, QprcRcOutrangeInt);

    /* Enable PcMatch && Overflow, undowflow, zero match interrupt */
    stcQprcIntSel.bQprcPcOfUfZeroInt = TRUE; //Enable Overflow, undowflow, zero match interrupt
    stcQprcIntSel.bQprcPcMatchInt    = TRUE; //Enable PcMatch interrupt
    stcQprcIntSel.bQprcPcRcMatchInt  = TRUE;
    stcQprcIntSel.bQprcPcMatchRcMatchInt = FALSE;
    stcQprcIntSel.bQprcPcCountInvertInt  = FALSE;
    stcQprcIntSel.bQprcRcOutrangeInt     = FALSE;
    stcQprcIntCallback.pfnPcOfUfZeroIntCallback = PcUfOfZeroIntHandler; //Overflow, undowflow, zero match interrupt callback function
    stcQprcIntCallback.pfnPcMatchIntCallback    = PcMatchIntHandler;    //PC match interrupt callback function
    stcQprcIntCallback.pfnPcRcMatchIntCallback  = RcMatchIntHandler;
    stcQprcIntCallback.pfnPcMatchRcMatchIntCallback = NULL;
    stcQprcIntCallback.pfnPcCountInvertIntCallback  = NULL;
    stcQprcIntCallback.pfnRcOutrangeIntCallback     = NULL;

    Qprc_EnableInt(&USER_QPRC, &stcQprcIntSel, &stcQprcIntCallback);
    /* Set PC &&RC mode */
    Qprc_ConfigPcMode(&USER_QPRC, QprcPcMode1);// PC_Mode3: Directional count mode: Counts up/down with BIN active edge and AIN level
    Qprc_ConfigRcMode(&USER_QPRC, QprcRcMode2);//Up/down count of RC on over or underflow in position count match

    return;
}

/*!
 ******************************************************************************
 ** \brief Delay time
 **
 ** \param  [in]   u32Cnt
 **
 ** \retval   None
 **
 ******************************************************************************
 */
static void Delay(uint32_t u32Cnt)
{
    uint32_t i = 0;
    for(; u32Cnt; u32Cnt--)
        for(i=1000; i; i--);

    return;
}

/*!
 ******************************************************************************
 ** \brief ZIN trigger initialization
 **
 ** \param  None
 **
 ** \retval   None
 **
 ******************************************************************************
 */
static void InitZinTrig(void)
{
    InitZinTriggerIo();
    ClrZinTriggerIo();

    return;
}

/*!
 ******************************************************************************
 ** \brief ZIN trigger signal generation
 **
 ** \param  None
 **
 ** \retval   None
 **
 ******************************************************************************
*/
static void ZinTrigGen(void)
{
    Delay(2000);
    SetZinTriggerIo();
    Delay(2000);
    ClrZinTriggerIo();

    return;
}

/*!
 ******************************************************************************
 ** \brief QPRC example code
 **
  ** \return uint32_t return value, if needed
 ******************************************************************************
 */
int32_t main(void)
{
#ifdef DEBUG_PRINT
   Uart_Printf_Init();
#endif
     
#ifdef DEBUG_PRINT
    printf("******************************************************************************************\n\r");
    printf("*        QPRC Revolution Counter example with PC underflow and overflow trigger mode     *\n\r");
    printf("******************************************************************************************\n\n\r");

    printf("Connect RTO00_0 with AIN0_0, and ZIN0_0 with P14\n\r");
    printf("If PC count overflows, PC count clears to 0 and RC count adds by 1 \n\n\r");
#endif

    WfgConfig();
    OcuConfig();
    FrtConfig();
    InitKey();
    InitZinTrig();
    QprcConfig();

    /*Start FRT operation */
    Mft_Frt_Start(&USER_FRT, USER_FRT_CH);

    /*Start OCU operation */
    Mft_Ocu_EnableOperation(&USER_OCU, USER_OCU_CH0);
    Mft_Ocu_EnableOperation(&USER_OCU, USER_OCU_CH1);

    while(1)
    {
        if (0 != m_u8PcMatchFlag)
        {
        #ifdef DEBUG_PRINT
            printf("The count of Position Counter matchs with compare value!\n");      
        #endif          
            m_u8PcMatchFlag = 0;
        }

        if (0 != m_u8PcOfFlag)
        {
        #ifdef DEBUG_PRINT
            printf("The count of Position Counter overflows!\n");      
        #endif   
            m_u8PcOfFlag = 0;
        }

        if (0 != m_u8PcUfFlag)
        {
        #ifdef DEBUG_PRINT
            printf("The count of Position Counter underflows!\n");      
        #endif    
            m_u8PcUfFlag = 0;
        }

        if (0 != m_u8RcMatchFlag)
        {
        #ifdef DEBUG_PRINT
            printf("The count of Revolution Counter matchs!\n");      
        #endif  
            m_u8RcMatchFlag = 0;
        }

        if (FALSE == GetKey())
        {
            ZinTrigGen();
        }
    }
}
/*****************************************************************************/
/* END OF FILE */
