/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates timer function of Real Time Clock. Every
 ** 5s second the timer interrupt occurs, a IO connected with LED will inverse.
 **
 **
 ** History:
 **   - 2014-03-07  0.1  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/

/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/

/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
static uint8_t  u8TimerIntOccur = 0;
static uint8_t  u8IntSec = 0;
static uint8_t  u8BlinkPolling = 0;

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/

/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/

/*****************************************************************************/
/* Function implementation - global ('extern') and local ('static')          */
/*****************************************************************************/
/**
 ******************************************************************************
 ** \brief Initializatio GPIO
 **
 ******************************************************************************/
static void PortInit(void)
{
   Gpio1pin_InitOut(GPIO1PIN_P61, Gpio1pin_InitVal(1u));
}

/**
 ******************************************************************************
 ** \brief Set port for LED
 **
 ******************************************************************************/
static void LedPolling(void)
{
    Gpio1pin_Put(GPIO1PIN_P61, u8BlinkPolling);
    u8BlinkPolling = ~u8BlinkPolling;
}

/**
 ******************************************************************************
 ** \brief 0.5-Second Interrupt
 **
 ******************************************************************************/
static void SampleRtcTimerCb(void)
{
    u8TimerIntOccur = 1;
}

/**
 ******************************************************************************
 ** \brief One Second Interrupt
 **
 ******************************************************************************/
static void SampleRtcOneSencondCb(void)
{
    // Set the one second interruption flag
    u8IntSec = TRUE;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for MB9BF56X series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    en_result_t enResult;
    stc_rtc_config_t stcRtcConfig;
    stc_rtc_int_sel_t stcInt;
    stc_rtc_int_cb_t  stcIntCb;
    stc_rtc_time_t   stcTimeDate;

    // Clear structures
    PDL_ZERO_STRUCT(stcRtcConfig);
    PDL_ZERO_STRUCT(stcInt);
    PDL_ZERO_STRUCT(stcIntCb);
    // Initialize GPIO for LED
    PortInit();

    // Clear seconds interrupt flag
    u8IntSec = FALSE;
    
    stcRtcConfig.enClkSel = RtcUseSubClk;
    stcRtcConfig.bEnSuboutDivider = FALSE;
    stcRtcConfig.bInitFreqCorr = FALSE;
    stcRtcConfig.enRtccoSel = RtccoOutput1Hz;
    
    // Time setting (23:59:00 1st of January 2014)
    stcRtcConfig.bInitTimeDate = TRUE;
    stcRtcConfig.stcTimeDate.u8Second = 0;                    // Second      : 00
    stcRtcConfig.stcTimeDate.u8Minute = 59;                   // Minutes     : 59
    stcRtcConfig.stcTimeDate.u8Hour   = 23;                   // Hour        : 23
    stcRtcConfig.stcTimeDate.u8Day    = 31;                   // Date        : 31th
    stcRtcConfig.stcTimeDate.u8Month  = RtcJanuary;           // Month       : January
    stcRtcConfig.stcTimeDate.u16Year   = 2014;                // Year        : 2014
    (void)Rtc_SetDayOfWeek(&stcRtcConfig.stcTimeDate);        // Set Day of the Week in stcRtcTime

    // Alarm setting 
    stcRtcConfig.bInitAlarm = FALSE;

    // Intialize RTC timer
    stcRtcConfig.bInitTimer = TRUE;
    stcRtcConfig.stcTimer.enMode = RtcTimerPeriod;
    stcRtcConfig.stcTimer.u32TimerCycle = 5; // Generate interrupt every 5s
    
    // Initialize the RTC
    enResult = Rtc_Init(&stcRtcConfig);

    if (Ok != enResult)
    {
#ifdef DEBUG_PRINT
        printf("Initial error!\n");
#endif
        while(1);
    }

    // Configure interrupt 
    stcInt.TimerInt = TRUE;
    stcInt.OneSecondInt = TRUE;
    
    stcIntCb.pfnOneSecondCallback  = SampleRtcOneSencondCb;
    stcIntCb.pfnTimerCallback      = SampleRtcTimerCb;
    
    Rtc_EnableInt(&stcInt, &stcIntCb);
      
    // Start RTC counting 
    Rtc_EnableFunc(RtcCount);
    
    // Enable timer 
    Rtc_EnableFunc(RtcTimer);
    
    while (1)
    {
        // If one second interruption occurs
        if (TRUE == u8IntSec)
        {
            u8IntSec = FALSE;
            Rtc_ReadDateTime(&stcTimeDate);
        #ifdef DEBUG_PRINT
            // Print RTC to console
            printf("%04d/%02d/%02d %02d:%02d:%02d ", 
                   stcTimeDate.u16Year, 
                   stcTimeDate.u8Month, 
                   stcTimeDate.u8Day, 
                   stcTimeDate.u8Hour, 
                   stcTimeDate.u8Minute, 
                   stcTimeDate.u8Second);
            switch (stcTimeDate.u8DayOfWeek)
            {
                case RtcSunday:
                    printf("Sunday\n");
                    break;
                case RtcMonday:
                    printf("Monday\n");
                    break;
                case RtcTuesday:
                    printf("Tuesday\n");
                    break;
                case RtcWednesday:
                    printf("Wednesday\n");
                    break;
                case RtcThursday:
                    printf("Thursday\n");
                    break;
                case RtcFriday:
                    printf("Friday\n");
                    break;
                case RtcSaturday:
                    printf("Saturday\n");
                    break;
                default:
                    break;
            }
        #endif  
        }
        
        if(u8TimerIntOccur == 1)
        {
            u8TimerIntOccur = 0;
        #ifdef DEBUG_PRINT
            printf("Timer interrupt occurs!\n");
        #endif    
            LedPolling();
        }
        
    }
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
