/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module for Software watchdog sample (using window mode)
 **
 ** History:
 **   - 2014-02-15  0.0.1  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/
static volatile uint8_t u8SwwdtActive;
static const stc_swwdg_config_t stcSwwdgConfig = {
    20000000,                   /* Timer interval */
    TRUE,                       /* Enables SW watchdog reset */
    TRUE,                       /* Enables Window watchdog mode */
    FALSE,                      /* Interrupt when reload without timing window was occured */
    en_swwdg_timing_window_50   /* Reload can be executed at less than or equal to 50% of WdogLoad */
};

// Table to store status of window watchdog error
static volatile uint8_t au8ErrorStatus[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

// Timing table for reloading to watchdog timer
// Please use these values when debugging performs by flash and optimization is not used.
static const volatile uint32_t au32CountValue[10]  = {
#if defined (__ICCARM__)
    5757129,   // Interruption will occur
    5757130,   // Interruption will occur
    5757131,   // Interruption will occur
    5757132,   // Interruption will occur
    5757133,   // Interruption will occur
    8057135,   // Interruption will not occur
    8057136,   // Interruption will not occur
    8057137,   // Interruption will not occur
    8057138,   // Interruption will not occur
    8057139    // Interruption will not occur
#else
    2099980,   // Interruption will occur
    2099982,   // Interruption will occur
    2099984,   // Interruption will occur
    2099986,   // Interruption will occur
    2099988,   // Interruption will occur
   10099990,   // Interruption will not occur
   10099992,   // Interruption will not occur
   10099994,   // Interruption will not occur
   10099996,   // Interruption will not occur
   10099998    // Interruption will not occur
#endif
};

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
/**
 ******************************************************************************
 ** \brief  Software watchdog interrupt function
 **
 ** Interruption occurs if watchdog counter reloads when watchdog counter is
 ** less than 100%,75%,50% or 25% of WdogLoad value.
 ** This percentage is set by TWD bits of WdogControl.
 **
 ******************************************************************************/
static void WdgSwCallback(void)
{
    // Stop software watchdog (*)
    Swwdg_Stop();
    // Clear interrupt
    Swwdg_Feed();
    // Software watchdog in-acitve
    u8SwwdtActive = FALSE;

    // (*)Software watchdog has to be stopped since interruption occurs,
    //    when interruption cause was cleared and watchdog counter was reloaded.
}

/**
 ******************************************************************************
 ** \brief  Main function of project for MB9BF56X series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    volatile uint32_t u32Count;
    volatile uint8_t  u8Index;

    u8SwwdtActive = FALSE;
    // Initialize hardware watchdog
    if (Ok != Swwdg_Init((stc_swwdg_config_t *)&stcSwwdgConfig))
    {
#ifdef DEBUG_PRINT
        printf("Initial error!\n");
#endif
        while (1);
    }

    u8Index = 0;
    while (10 > u8Index)
    {
        // Restart software watchdog
        Swwdg_Start(WdgSwCallback);
        u8SwwdtActive = TRUE;
        // Adjust the timing for reloading watchdog counter
        u32Count = au32CountValue[u8Index];
        while (0 != (u32Count--))
        {
            continue;
        }
        __disable_irq();
        // Clear interrupt and reload watchdog counter
        Swwdg_Feed();
        __enable_irq();
        // Insert cycle for interrupt
        PDL_WAIT_LOOP_HOOK();
        // If watchdog is in-active, error is occured
        if (FALSE == u8SwwdtActive)
        {
            // Error status set
            au8ErrorStatus[u8Index] = 1;
        }
        u8Index++;
        __disable_irq();
        // If watchdog is active...
        if (TRUE == u8SwwdtActive)
        {
            // Stop software watchdog and clear interrupt
            WdgSwCallback();
        }
        __enable_irq();
    }

    while (1);
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
