/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file adc.c
 **
 ** A detailed description is available at 
 ** @link AdcGroup ADC description @endlink
 **
 ** History:
 **   - 2014-01-14  1.2  Edison Zhang First version
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "adc.h"

#if (defined(PDL_PERIPHERAL_ADC_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup AdcGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/// Look-up table for all enabled ADC instances and their internal data
stc_adc_instance_data_t m_astcAdcInstanceDataLut[ADC_INSTANCE_COUNT] =
{
  #if (PDL_PERIPHERAL_ENABLE_ADC0 == PDL_ON)
  { 
     &ADC0,  // pstcInstance
     {0, 0 ,0, 0, 0 ,0}           // stcInternData (not initialized yet)
  },
  #endif
  #if (PDL_PERIPHERAL_ENABLE_ADC1 == PDL_ON)
  {  
     &ADC1,  // pstcInstance
     {0, 0 ,0, 0, 0 ,0}            // stcInternData (not initialized yet)
  },
  #endif
  #if (PDL_PERIPHERAL_ENABLE_ADC2 == PDL_ON)
  {  
     &ADC2,  // pstcInstance
     {0, 0 ,0, 0, 0 ,0}            // stcInternData (not initialized yet)
  }
  #endif
};

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON)
static void Adc_InitNvic(volatile stc_adcn_t* pstcAdc);
static void Adc_DeInitNvic(volatile stc_adcn_t* pstcAdc);
static stc_adc_intern_data_t* AdcGetInternDataPtr(volatile stc_adcn_t* pstcAdc);
#endif

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/******************************************************************************/
/* Local Functions                                                            */
/******************************************************************************/

#if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON)

/**
 ******************************************************************************
 ** \brief Return the internal data for a certain ADC instance.
 **
 ** \param pstcAdc Pointer to ADC instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled (or not known)
 **
 ******************************************************************************/
static stc_adc_intern_data_t* AdcGetInternDataPtr(volatile stc_adcn_t* pstcAdc) 
{
    uint8_t u8Instance;
   
    for (u8Instance = 0u; u8Instance < ADC_INSTANCE_COUNT; u8Instance++)
    {
        if (pstcAdc == m_astcAdcInstanceDataLut[u8Instance].pstcInstance)
        {
            return &m_astcAdcInstanceDataLut[u8Instance].stcInternData;
        }
    }

    return NULL;
}
      
/**
 ******************************************************************************
 ** \brief ADC instance interrupt service routine
 **
 ** ADC instance interrupt service routine, clear interrupt cause and
 **
 ** \param  pstcAdc              Pointer to ADC instance
 ** \param  pstcAdcInternData    Pointer to ADC intern data
 **        
 ******************************************************************************/
void AdcIrqHandler( volatile stc_adcn_t*   pstcAdc,
                    stc_adc_intern_data_t* pstcAdcInternData)
{
    if (pstcAdc->SCCR_f.SOVR)   // Error case (FIFO overflow)
    {
        pstcAdc->SCCR_f.SOVR = 0u;

        if (pstcAdcInternData->pfnScanErrorCallback != NULL)
        {
            pstcAdcInternData->pfnScanErrorCallback();
        }
    }

    if (pstcAdc->PCCR_f.POVR)  // Error case (Priority FIFO overflow)
    {
        pstcAdc->PCCR_f.POVR = 0u;

        if (pstcAdcInternData->pfnPrioErrorCallback != NULL)
        {
            pstcAdcInternData->pfnPrioErrorCallback();
        }
    }

    if (pstcAdc->ADCR_f.PCIF)  // Priority Conversion
    {
        pstcAdc->ADCR_f.PCIF = 0u;

        if (pstcAdcInternData->pfnPrioCallback != NULL)
        {
            // Callback argument points to priority FIFO. User has to empty it in
            // the callback function, because the FIFO depth is known.
            pstcAdcInternData->pfnPrioCallback((volatile uint32_t*)&pstcAdc->PCFD);
        }
    }

    if (pstcAdc->ADCR_f.SCIF)   // Scan conversion interrupt request?
    {
        pstcAdc->ADCR_f.SCIF = 0u;

        if (pstcAdcInternData->pfnScanCallback != NULL)
        {
            // Callback argument points to scan FIFO. User has to empty it in
            // the callback function, because the FIFO depth is known. 
            pstcAdcInternData->pfnScanCallback((volatile uint32_t*)&pstcAdc->SCFD);
        }
    }

    if (pstcAdc->ADCR_f.CMPIF)  // Compare result interrupt request?
    {
        pstcAdc->ADCR_f.CMPIF = 0u;

        if (pstcAdcInternData->pfnComparisonCallback != NULL)
        {
            pstcAdcInternData->pfnComparisonCallback();
        }
    }

    if (pstcAdc->WCMRCIF_f.RCINT)  // Range result interrupt request?
    {
        pstcAdc->WCMRCIF_f.RCINT = 0u;

        if (pstcAdcInternData->pfnRangeComparisonCallback != NULL)
        {
            pstcAdcInternData->pfnRangeComparisonCallback();
        }
    }

    return;
} // AdcIrqHandler

/**
 ******************************************************************************
 ** \brief Set NVIC Interrupt depending on ADC instance
 **
 ** \param pstcAdc Pointer to ADC instance
 **
 ******************************************************************************/
static void Adc_InitNvic(volatile stc_adcn_t* pstcAdc)
{
#if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON)  
    if (pstcAdc == (volatile stc_adcn_t*)(&ADC0))
    {
        NVIC_ClearPendingIRQ(ADC0_IRQn);
        NVIC_EnableIRQ(ADC0_IRQn);
        NVIC_SetPriority(ADC0_IRQn, PDL_IRQ_LEVEL_ADC0);   
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON)     
    if (pstcAdc == (volatile stc_adcn_t*)(&ADC1))
    {
        NVIC_ClearPendingIRQ(ADC1_IRQn);
        NVIC_EnableIRQ(ADC1_IRQn);
        NVIC_SetPriority(ADC1_IRQn, PDL_IRQ_LEVEL_ADC1);      
    }
#endif
#if (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON)     
    if (pstcAdc == (volatile stc_adcn_t*)(&ADC2))
    {
        NVIC_ClearPendingIRQ(ADC2_LCD_IRQn);
        NVIC_EnableIRQ(ADC2_LCD_IRQn);
        NVIC_SetPriority(ADC2_LCD_IRQn, PDL_IRQ_LEVEL_ADC2_LCD);      
    } 
#endif    
} // Adc_InitInterrupt

/**
 ******************************************************************************
 ** \brief Clear NVIC Interrupt depending on ADC instance
 **
 ** \param pstcAdc Pointer to ADC instance
 **
 ******************************************************************************/
static void Adc_DeInitNvic(volatile stc_adcn_t* pstcAdc)
{
    /* if SCIE, PCIE, CMPIE or OVRIE is not 0, return */
    if((pstcAdc->ADCR & 0x0Fu) != 0x00u) 
    {
        return;
    }
    
    /* if RCOIE is not 0, return */
    if(pstcAdc->WCMPCR_f.RCOIE != 0)
    {
        return;
    }
    
    /* Only when all of ADC interrupt sources are disable, disable ADC IRQ */
#if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON) 
    if (pstcAdc == (volatile stc_adcn_t*)(&ADC0))
    {
        NVIC_ClearPendingIRQ(ADC0_IRQn);
        NVIC_DisableIRQ(ADC0_IRQn);
        NVIC_SetPriority(ADC0_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);   
    }
#endif 
#if (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON)    
    if (pstcAdc == (volatile stc_adcn_t*)(&ADC1))
    {
        NVIC_ClearPendingIRQ(ADC1_IRQn);
        NVIC_DisableIRQ(ADC1_IRQn);
        NVIC_SetPriority(ADC1_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);      
    }
#endif
#if (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON)    
    if (pstcAdc == (volatile stc_adcn_t*)(&ADC2))
    {
        NVIC_ClearPendingIRQ(ADC2_LCD_IRQn);
        NVIC_DisableIRQ(ADC2_LCD_IRQn);
        NVIC_SetPriority(ADC2_LCD_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);      
    }
#endif    
} // Adc_DeInitInterrupt
#endif // #if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_ADC2_LCD == PDL_ON)

/******************************************************************************/
/* Global Functions                                                           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief Initialize ADC
 **
 ** This function initializes an ADC module
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  pstcConfig       ADC module configuration 
 **
 ** \retval Ok                    ADC initialized normally
 ** \retval ErrorInvalidParameter if one of following cases matchs:
 **                               - pstcAdc == NULL
 **                               - pstcConfig == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Adc_Init( volatile stc_adcn_t* pstcAdc,
                      stc_adc_config_t*    pstcConfig )
{
    stc_adc0_sccr_field_t  stcSCCR;
    stc_adc0_pccr_field_t  stcPCCR;
    stc_adc0_adst0_field_t stcADST0;
    stc_adc0_adst1_field_t stcADST1;
    stc_adc0_wcmpcr_field_t stcWCMPCR;
    
    //stc_adc0_pcis_field_t  stcPCIS;
    uint8_t u8PCIS = 0;
    uint8_t u8CMPCR = 0;

    PDL_ZERO_STRUCT(stcSCCR);
    PDL_ZERO_STRUCT(stcPCCR);
    PDL_ZERO_STRUCT(stcADST0);
    PDL_ZERO_STRUCT(stcADST1);
    PDL_ZERO_STRUCT(stcWCMPCR);

    // Check for NULL pointer
    if ((pstcAdc    == NULL) ||
        (pstcConfig == NULL))
    {
        return ErrorInvalidParameter ;
    }  

    // Disable ADC in any case first
    pstcAdc->ADCEN_f.ENBL = 0u;

    // Clear all relevant registers
    pstcAdc->ADCR = 0u;
    pstcAdc->ADSR = 0u;
    pstcAdc->SCCR = 0u;

    // Set Sampling Time Selection Register
    pstcAdc->ADSS0 = (uint8_t)(0x000000FFul & pstcConfig->u32SamplingTimeSelect.u32AD_CHn);
    pstcAdc->ADSS1 = (uint8_t)(0x0000FF00ul & (pstcConfig->u32SamplingTimeSelect.u32AD_CHn >> 8u));
    pstcAdc->ADSS2 = (uint8_t)(0x00FF0000ul & (pstcConfig->u32SamplingTimeSelect.u32AD_CHn >> 16u));
    pstcAdc->ADSS3 = (uint8_t)(0xFF000000ul & (pstcConfig->u32SamplingTimeSelect.u32AD_CHn >> 24u));

    // Set Sampling Times Config. 0
    switch (pstcConfig->enSamplingTimeN0)
    {
        case Value1:
            stcADST0.STX0 = 0u;
            break;
        case Value4:
            stcADST0.STX0 = 1u;
            break;
        case Value8:
            stcADST0.STX0 = 2u;
            break;
        case Value16:
            stcADST0.STX0 = 3u;
            break;
        case Value32:
            stcADST0.STX0 = 4u;
            break;
        case Value64:
            stcADST0.STX0 = 5u;
            break;
        case Value128:
            stcADST0.STX0 = 6u;
            break;
        case Value256:
            stcADST0.STX0 = 7u;
            break;
        default:
            return ErrorInvalidParameter;
    }

    if (pstcConfig->u8SamplingTime0 > 31u)
    {
        return ErrorInvalidParameter;
    }
    
    stcADST0.ST0 = pstcConfig->u8SamplingTime0;

    pstcAdc->ADST0_f = stcADST0;

    // Set Sampling Times Config. 1
    switch (pstcConfig->enSamplingTimeN1)
    {
        case Value1:   
            stcADST1.STX1 = 0u;
            break;
        case Value4:
            stcADST1.STX1 = 1u; 
            break;
        case Value8:
            stcADST1.STX1 = 2u; 
            break;
        case Value16:
            stcADST1.STX1 = 3u; 
            break;
        case Value32:
            stcADST1.STX1 = 4u; 
            break;
        case Value64:
            stcADST1.STX1 = 5u; 
            break;
        case Value128:
            stcADST1.STX1 = 6u; 
            break;
        case Value256:
            stcADST1.STX1 = 7u; 
            break;
        default:
            return ErrorInvalidParameter;
    }

    if (pstcConfig->u8SamplingTime1 > 31u)
    {
        return ErrorInvalidParameter;
    }

    stcADST1.ST1 = pstcConfig->u8SamplingTime1;

    pstcAdc->ADST1_f = stcADST1;

    // Frequency Division for ADC Instance
    pstcAdc->ADCT = pstcConfig->u8SamplingMultiplier;

    // MSB, LSB alignment
    pstcAdc->ADSR_f.FDAS = (TRUE == pstcConfig->bLsbAlignment) ? 1u : 0u;
    
    // Set enable time
    pstcAdc->ADCEN &= 0x00FFu;
    pstcAdc->ADCEN |= (uint16_t)((uint16_t)pstcConfig->u8EnableTime << 8u);
  
    if(TRUE == pstcConfig->bScanInit)
    {
        // Set Scan Conversion Input Selection Register
        pstcAdc->SCIS0 = (uint8_t)(0x000000FFul & pstcConfig->stcScanInit.u32ScanCannelSelect.u32AD_CHn);
        pstcAdc->SCIS1 = (uint8_t)(0x0000FF00ul & (pstcConfig->stcScanInit.u32ScanCannelSelect.u32AD_CHn >> 8u));
        pstcAdc->SCIS2 = (uint8_t)(0x00FF0000ul & (pstcConfig->stcScanInit.u32ScanCannelSelect.u32AD_CHn >> 16u));
        pstcAdc->SCIS3 = (uint8_t)(0xFF000000ul & (pstcConfig->stcScanInit.u32ScanCannelSelect.u32AD_CHn >> 24u));

        // Conversion mode single, repeated
        switch (pstcConfig->stcScanInit.enScanMode)
        {
            case ScanSingleConversion:
                stcSCCR.RPT = 0u;
                break;
            case ScanRepeatConversion:
                stcSCCR.RPT = 1u;
                break;
            default:
                return ErrorInvalidParameter;
        }

        // Scan Conversion Timer Start Enable and Timer Source
        if (TRUE == pstcConfig->stcScanInit.bScanTimerStartEnable)
        {
            stcSCCR.SHEN = 1u;
            pstcAdc->SCTSL = (uint8_t) pstcConfig->stcScanInit.enScanTimerTrigger;
        }

        // Update hardware
        pstcAdc->SCCR_f = stcSCCR;

        // Scan Conversion FIFO Depth
        if (pstcConfig->stcScanInit.u8ScanFifoDepth > 15u)
        {
            return ErrorInvalidParameter;
        }

        pstcAdc->SFNS = pstcConfig->stcScanInit.u8ScanFifoDepth;

    } 
  
    if(TRUE == pstcConfig->bPrioInit)  
    {  
        // Priority external Trigger Analog Input
        stcPCCR.ESCE = 0u;

        // Priority external Trigger Start Enable
        stcPCCR.PEEN = (TRUE == pstcConfig->stcPrioInit.bPrioExtTrigStartEnable) ? 1u : 0u;

        // Priority Conversion Timer Start Enable and Timer Source
        if (TRUE == pstcConfig->stcPrioInit.bPrioTimerStartEnable)
        {
            stcPCCR.PHEN = 1u;
            pstcAdc->PRTSL = (uint8_t) pstcConfig->stcPrioInit.enPrioTimerTrigger;
        }

        // Update Hardware
        pstcAdc->PCCR_f = stcPCCR;

        // Priority Conversion FIFO Stage Count Setup
        pstcAdc->PFNS = pstcConfig->stcPrioInit.u8PrioFifoDepth;

        // Priority Conversion Input Selection
        if (pstcConfig->stcPrioInit.u8PrioLevel1AnalogChSel > 7u)
        {
            return ErrorInvalidParameter;
        }
        u8PCIS = pstcConfig->stcPrioInit.u8PrioLevel1AnalogChSel & 0x07u;

        if (pstcConfig->stcPrioInit.u8PrioLevel2AnalogChSel > 31u)
        {
            return ErrorInvalidParameter;
        }
        u8PCIS |= (uint8_t)((uint8_t)(0x1Fu & pstcConfig->stcPrioInit.u8PrioLevel2AnalogChSel) << 3u);

        pstcAdc->PCIS = u8PCIS;
    }
  
    if (TRUE == pstcConfig->bComparisonEnable)
    {
        // Comparison Value
        pstcAdc->CMPD = (((pstcConfig->stcComparisonInit.u16CompareValue >> 2u)) << 6u);
        
        if (pstcConfig->stcComparisonInit.u8CompareChannel > 31u)
        {
            return ErrorInvalidParameter;
        }
        
        // Comparison Control
        if(pstcConfig->stcComparisonInit.bCompIrqEqualGreater == TRUE)
        {
            u8CMPCR |= 0x40u;
        }
        
        if(TRUE == pstcConfig->stcComparisonInit.bCompareAllChannels)
        {
            u8CMPCR |= 0x20u;
        }

        u8CMPCR |= 0x80u; // Enable comparison function
        u8CMPCR |= (0x1Fu & pstcConfig->stcComparisonInit.u8CompareChannel);
        
        pstcAdc->CMPCR = u8CMPCR;  
    }
  
    if (TRUE == pstcConfig->bRangeComparisonEnable)
    {
        // Set lower and upper value of the range
        pstcAdc->WCMPDH = ((pstcConfig->stcRangeComparisonInit.u16UpperLimitRangeValue >> 2u) << 6u);
        pstcAdc->WCMPDL = ((pstcConfig->stcRangeComparisonInit.u16LowerLimitRangeValue >> 2u) << 6u);

        if (pstcConfig->stcRangeComparisonInit.u8RangeCountValue > 7u)
        {
            return ErrorInvalidParameter;
        }

        // Set compare times
        stcWCMPCR.RCODC = pstcConfig->stcRangeComparisonInit.u8RangeCountValue;
        // In-range compare or out-range compare
        stcWCMPCR.RCOIRS = pstcConfig->stcRangeComparisonInit.bWithinRange;
        // Enable range compare function
        stcWCMPCR.RCOE = 1;
        // Compare 1 channel or all
        pstcAdc->WCMPSR_f.WCMD = pstcConfig->stcRangeComparisonInit.bRangeCompareAllChannels;

        if (pstcConfig->stcRangeComparisonInit.u8RangeComapreChannel > 31u)
        {
            return ErrorInvalidParameter;
        }

        // Set the channel compared (according to single channel compare mode)
        pstcAdc->WCMPSR_f.WCCH |= pstcConfig->stcRangeComparisonInit.u8RangeComapreChannel;

        // update hardware
        pstcAdc->WCMPCR_f = stcWCMPCR;
    }
 
  return Ok;
} // Adc_Init

/**
 ******************************************************************************
 ** \brief De-Initialize ADC
 **
 ** This function deinitializes an ADC module
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    ADC instance fully disabled and reset
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t Adc_DeInit( volatile stc_adcn_t* pstcAdc )
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }

    pstcAdc->ADCEN   = 0u;   // Disable ADC (including ENBL bit)
    pstcAdc->ADCR    = 0u;
    pstcAdc->ADSR    = 0u;
    pstcAdc->SCCR    = 0u;
    pstcAdc->SFNS    = 0u;
    pstcAdc->SCIS0   = 0u;
    pstcAdc->SCIS1   = 0u;
    pstcAdc->SCIS2   = 0u;
    pstcAdc->SCIS3   = 0u;
    pstcAdc->PCCR    = 0u;
    pstcAdc->PFNS    = 0u;
    pstcAdc->PCFD    = 0u;
    pstcAdc->PCIS    = 0u;
    pstcAdc->CMPD    = 0u;
    pstcAdc->CMPCR   = 0u;
    pstcAdc->ADSS0   = 0u;
    pstcAdc->ADSS1   = 0u;
    pstcAdc->ADSS2   = 0u;
    pstcAdc->ADSS3   = 0u;
    pstcAdc->ADST0   = 0u;
    pstcAdc->ADST1   = 0u;
    pstcAdc->ADCEN   = 0u;
    pstcAdc->WCMPDH  = 0u;
    pstcAdc->WCMPCR  = 0u;
    pstcAdc->WCMPDL  = 0u;
    pstcAdc->WCMPSR  = 0u;
    pstcAdc->WCMRCOT = 0u;
    pstcAdc->WCMRCIF = 0u;
    pstcAdc->SCTSL   = 0u;
    pstcAdc->PRTSL   = 0u;

    return Ok;
} // Adc_DeInit

#if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON)
/**
 ******************************************************************************
 ** \brief Enable ADC interrupt
 **
 ** This function enable the ADC interrupts which are selected.
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  pstcIntSel       ADC interrupt selection
 ** \param [in]  pstcIntCallback  ADC interrupt callback functions     
 **
 ** \retval Ok                    Enable interrupt normally
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 **                               pstcIntSel == NULL
 **                               pstcIntCallback == NULL     
 **                               Other invalid configuration
 ******************************************************************************/      
en_result_t      Adc_EnableInt(volatile stc_adcn_t* pstcAdc, 
                               stc_adc_int_sel_t* pstcIntSel,
                               stc_adc_int_callback_t* pstcIntCallback)
{
    // Pointer to internal data
    stc_adc_intern_data_t* pstcAdcInternData ;  
  
    // Check for NULL pointer
    if ((pstcAdc  == NULL) || (pstcIntSel == NULL) || (pstcIntCallback == NULL))
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcAdcInternData = AdcGetInternDataPtr( pstcAdc ) ;
    
    // Check for instance available or not
    if(pstcAdcInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bScanIrq == TRUE)
    {
        pstcAdcInternData->pfnScanCallback = pstcIntCallback->pfnScanCallback;
        pstcAdc->ADCR_f.SCIE = 1;
    }
    
    if(pstcIntSel->bPrioIrq == TRUE)
    {
        pstcAdcInternData->pfnPrioCallback = pstcIntCallback->pfnPrioCallback;
        pstcAdc->ADCR_f.PCIE = 1;
    }
    
    if(pstcIntSel->bComparisonIrq == TRUE)
    {
        pstcAdcInternData->pfnComparisonCallback = pstcIntCallback->pfnComparisonCallback;
        pstcAdc->ADCR_f.CMPIE = 1;
    }
  
    if(pstcIntSel->bRangeComparisonIrq == TRUE)
    {
        pstcAdcInternData->pfnRangeComparisonCallback = pstcIntCallback->pfnRangeComparisonCallback;
        pstcAdc->WCMPCR_f.RCOIE = 1;
    }
    
    if(pstcIntSel->bFifoOverrunIrq == TRUE)
    {
        pstcAdcInternData->pfnScanErrorCallback = pstcIntCallback->pfnScanErrorCallback;
        pstcAdcInternData->pfnPrioErrorCallback = pstcIntCallback->pfnPrioErrorCallback;
        pstcAdc->ADCR_f.OVRIE = 1;
    }
    
    Adc_InitNvic(pstcAdc);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable ADC interrupt
 **
 ** This function disable the ADC interrupts which are selected.
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  pstcIntSel       ADC interrupt selection 
 **
 ** \retval Ok                    Disable interrupt normally
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 **                               pstcIntSel == NULL
 **                               pstcIntCallback == NULL     
 **                               Other invalid configuration
 ******************************************************************************/    
en_result_t   Adc_DisableInt(volatile stc_adcn_t* pstcAdc, 
                             stc_adc_int_sel_t* pstcIntSel)
{
    // Pointer to internal data
    stc_adc_intern_data_t* pstcAdcInternData ;  
  
    // Check for NULL pointer
    if ((pstcAdc  == NULL) || (pstcIntSel == NULL))
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcAdcInternData = AdcGetInternDataPtr( pstcAdc ) ;
    
    // Check for instance available or not
    if(pstcAdcInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bScanIrq == TRUE)
    {
        pstcAdcInternData->pfnScanCallback = NULL;
        pstcAdc->ADCR_f.SCIE = 0;
    }
    
    if(pstcIntSel->bPrioIrq == TRUE)
    {
        pstcAdcInternData->pfnPrioCallback = NULL;
        pstcAdc->ADCR_f.PCIE = 0;
    }
    
    if(pstcIntSel->bComparisonIrq == TRUE)
    {
        pstcAdcInternData->pfnComparisonCallback = NULL;
        pstcAdc->ADCR_f.CMPIE = 0;
    }
  
    if(pstcIntSel->bRangeComparisonIrq == TRUE)
    {
        pstcAdcInternData->pfnRangeComparisonCallback = NULL;
        pstcAdc->WCMPCR_f.RCOIE = 0;
    }
    
    if(pstcIntSel->bFifoOverrunIrq == TRUE)
    {
        pstcAdcInternData->pfnScanErrorCallback = NULL;
        pstcAdcInternData->pfnPrioErrorCallback = NULL;
        pstcAdc->ADCR_f.OVRIE = 0;
    }
    
    Adc_DeInitNvic(pstcAdc);
    
    return Ok;
}
#endif

/**
 ******************************************************************************
 ** \brief Get ADC interrupt flag
 **
 ** This function gets the ADC interrupt flag which is selected.
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  enInt            ADC interrupt type
 ** \arg         AdcScanInt              Scan conversion interrupt flag
 ** \arg         AdcPrioInt              Priority scan conversion interrupt flag
 ** \arg         AdcScanFifoOverrunInt   Scan FIFO overrun interrupt flag
 ** \arg         AdcPrioFifoOverrunInt   Priority FIFO overrun interrupt flag
 ** \arg         AdcComparisonInt        Comparison interrupt flag
 ** \arg         AdcRangeComparisonInt   Range comparison interrupt flag
 **
 ** \retval TRUE                  The data count in scan FIFO match scan FIFO stage [AdcScanInt]   
 **                               The data count in priority FIFO match priority FIFO stage [AdcPrioInt]
 **                               Scan FIFO overruns [AdcScanFifoOverrunInt]
 **                               Priority FIFO overruns [AdcPrioFifoOverrunInt]
 **                               The comparison result match the condition set in the Adc_Init() [AdcComparisonInt]
 **                               The range comparison result match the condition set in the Adc_Init() [AdcRangeComparisonInt]
 ** \retval FALSE                 The data count in scan FIFO don't match scan FIFO stage [AdcScanInt]
 **                               The data count in priority FIFO don't match priority FIFO stage [AdcPrioInt]
 **                               Scan FIFO don't overrun [AdcScanFifoOverrunInt]
 **                               Priority FIFO don't overrun [AdcPrioFifoOverrunInt]
 **                               The comparison result don't match the condition set in the Adc_Init() [AdcComparisonInt]
 **                               The range comparison result don't match the condition set in the Adc_Init() [AdcRangeComparisonInt]
 **
 ******************************************************************************/    
boolean_t   Adc_GetIntFlag(volatile stc_adcn_t* pstcAdc, en_adc_int_t enInt)
{
    boolean_t bRet = FALSE;
    switch(enInt)
    {
        case AdcScanInt:
            bRet = (pstcAdc->ADCR_f.SCIF == 1) ? 1 : 0;
            break;
        case AdcPrioInt:
            bRet = (pstcAdc->ADCR_f.PCIF == 1) ? 1 : 0;
            break;
        case AdcScanFifoOverrunInt:
            bRet = (pstcAdc->SCCR_f.SOVR == 1) ? 1 : 0;
            break;
        case AdcPrioFifoOverrunInt:
            bRet = (pstcAdc->PCCR_f.POVR == 1) ? 1 : 0;
            break;    
        case AdcComparisonInt:
            bRet = (pstcAdc->ADCR_f.CMPIF == 1) ? 1 : 0;
            break;
        case AdcRangeComparisonInt:
            bRet = (pstcAdc->WCMRCIF_f.RCINT == 1) ? 1 : 0;
            break;
        default:
            break;
    }
    
    return bRet;
}

/**
 ******************************************************************************
 ** \brief Clear ADC interrupt flag
 **
 ** This function clears the ADC interrupt which is selected.
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  enInt            Interrupt type
 ** \arg         AdcScanInt              Scan conversion interrupt flag
 ** \arg         AdcPrioInt              Priority scan conversion interrupt flag
 ** \arg         AdcScanFifoOverrunInt   Scan FIFO overrun interrupt flag
 ** \arg         AdcPrioFifoOverrunInt   Priority FIFO overrun interrupt flag
 ** \arg         AdcComparisonInt        Comparison interrupt flag
 ** \arg         AdcRangeComparisonInt   Range comparison interrupt flag
 **
 ** \retval Ok                    Disable interrupt normally
 ** \retval ErrorInvalidParameter pstcAdc == NULL 
 **                               Other invalid configuration
 ******************************************************************************/
en_result_t Adc_ClrIntFlag(volatile stc_adcn_t* pstcAdc, en_adc_int_t enInt)
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    switch(enInt)
    {
        case AdcScanInt:
            pstcAdc->ADCR_f.SCIF = 0;
            break;
        case AdcPrioInt:
            pstcAdc->ADCR_f.PCIF = 0;
            break;
        case AdcScanFifoOverrunInt:
            pstcAdc->SCCR_f.SOVR = 0;
            break;
        case AdcPrioFifoOverrunInt:
            pstcAdc->PCCR_f.POVR = 0;
            break;    
        case AdcComparisonInt:
            pstcAdc->ADCR_f.CMPIF = 0;
            break;
        case AdcRangeComparisonInt:
            pstcAdc->WCMRCIF_f.RCINT = 0;
            break;
        default:
            return ErrorInvalidParameter ;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable ADC and wait for ADC operation ready
 **
 ** This function enables an ADC instance and waits for readiness
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    ADC instance enabled and ready
 ** \retval ErrorTimeout          ADC instance not ready
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t Adc_Enable( volatile stc_adcn_t* pstcAdc )
{
    uint32_t u32TimeOutCounter = PDL_ADC_READY_WAIT_COUNT;

    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }

    // Enable ADC
    pstcAdc->ADCEN_f.ENBL = 1u;

    // Polling for readiness
    while (u32TimeOutCounter)
    {
        if (TRUE == pstcAdc->ADCEN_f.READY)
        {
            return Ok;
        }

        u32TimeOutCounter--;
    }

    return ErrorTimeout;
} // Adc_Enable

/**
 ******************************************************************************
 ** \brief Disable ADC
 **
 ** This function disables an ADC operation
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    ADC instance disabled
 ** \retval ErrorTimeout          ADC instance not ready
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t     Adc_Disable( volatile stc_adcn_t* pstcAdc )
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    // Disable ADC
    pstcAdc->ADCEN_f.ENBL = 0u;
    
    return Ok;
}
                   
/**
 ******************************************************************************
 ** \brief ADC Scan Software Start Trigger
 **
 ** This function starts an AD Scan Conversion by Software trigger
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    ADC instance triggered (or re-triggered)
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t Adc_SwTriggerScan( volatile stc_adcn_t* pstcAdc )
{
  // Check for NULL pointer
  if (pstcAdc  == NULL)
  {
    return ErrorInvalidParameter ;
  }
  
  pstcAdc->SCCR_f.SSTR = 1u;     // Trigger AD Conversion
  
  return Ok;
} // Adc_SwStart

/**
 ******************************************************************************
 ** \brief Stop Scan conversion with repeat mode
 **
 ** This function stops an AD Scan Conversion with repeat mode
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    ADC scan conversion stopped
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t  Adc_StopScanRepeat( volatile stc_adcn_t* pstcAdc )
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pstcAdc->SCCR_f.RPT = 0;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief ADC Priority Software Start Trigger
 **
 ** This function starts an AD Priority Conversion by Software trigger
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    ADC instance triggered (or re-triggered)
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t Adc_SwTriggerPrio( volatile stc_adcn_t* pstcAdc )
{
  // Check for NULL pointer
  if (pstcAdc  == NULL)
  {
    return ErrorInvalidParameter ;
  }
  
  pstcAdc->PCCR_f.PSTR = 1u;     // Trigger AD Conversion
  
  return Ok;
}
                      
/**
 ******************************************************************************
 ** \brief ADC Software Stop
 **
 ** This function requests a stop of the ADC
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    ADC instance stop request
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t Adc_ForceStop( volatile stc_adcn_t* pstcAdc )
{
  // Check for NULL pointer
  if (pstcAdc  == NULL)
  {
    return ErrorInvalidParameter ;
  }
  
  pstcAdc->ADSR_f.ADSTP = 1u;
  
  return Ok;
} // Adc_SwStop

/**
 ******************************************************************************
 ** \brief Get ADC conversion status
 **
 ** This function gets the status when ADC is in conversion
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  enStatus         ADC status type
 ** \arg ScanFifoEmptyStatus      Scan FIFO empty status
 ** \arg ScanFifoFullStatus       Scan FIFO full status
 ** \arg PrioFifoEmptyStatus      Priority FIFO empty status
 ** \arg PrioFifoFullStatus       Priority FIFO full status
 ** \arg Prio2PendingStatus       Priority conversion (level 2) pending status
 ** \arg PrioStatus               Priority conversion status
 ** \arg ScanStatus               Scan conversion status
 ** \arg RangeThresholdExcessFlag Range comparison threshold exceed status
 **
 ** \retval TRUE                  If one of following cases match:
 **                               - The Scan FIFO is empty [enStatus = ScanFifoEmptyStatus]   
 **                               - The Scan FIFO is full [enStatus = ScanFifoFullStatus]
 **                               - Priority FIFO is empty [enStatus = PrioFifoEmptyStatus]
 **                               - Priority FIFO is full [enStatus = PrioFifoFullStatus]
 **                               - Priority level 2 conversion is pending [enStatus = Prio2PendingStatus]
 **                               - Priority conversion is in progress [enStatus = PrioStatus]
 **                               - Scan conversion is in progress [enStatus = ScanStatus]
 **                               - The sample data beyond the upper limit threshold [enStatus = RangeThresholdExcessFlag]
 ** \retval FALSE                 If one of following cases match:
 **                               - The Scan FIFO isn't empty [enStatus = ScanFifoEmptyStatus] 
 **                               - The Scan FIFO isn't full [enStatus = ScanFifoFullStatus]
 **                               - Priority FIFO isn't empty [enStatus = PrioFifoEmptyStatus]
 **                               - Priority FIFO isn't full [enStatus = PrioFifoFullStatus]
 **                               - Priority level 2 conversion isn't pending [enStatus = Prio2PendingStatus]
 **                               - Priority conversion isn't in progress [enStatus = PrioStatus]
 **                               - The sample data below the lower limit threshold [enStatus = RangeThresholdExcessFlag]
 ******************************************************************************/  
boolean_t   Adc_GetStatus(volatile stc_adcn_t* pstcAdc, en_adc_status_t enStatus)
{
    boolean_t bRet = FALSE;
    switch (enStatus)
    {
        case ScanFifoEmptyStatus:
            bRet = (pstcAdc->SCCR_f.SEMP == 1) ? TRUE : FALSE;
            break;
        case ScanFifoFullStatus:
            bRet = (pstcAdc->SCCR_f.SFUL == 1) ? TRUE : FALSE;
            break;
        case PrioFifoEmptyStatus:
            bRet = (pstcAdc->PCCR_f.PEMP == 1) ? TRUE : FALSE;
            break;
        case PrioFifoFullStatus:
            bRet = (pstcAdc->PCCR_f.PFUL == 1) ? TRUE : FALSE;
            break;
        case Prio2PendingStatus:
            bRet = (pstcAdc->ADSR_f.PCNS == 1) ? TRUE : FALSE;
            break;
        case PrioStatus:  
            bRet = (pstcAdc->ADSR_f.PCS == 1) ? TRUE : FALSE;
            break;
        case ScanStatus: 
            bRet = (pstcAdc->ADSR_f.SCS == 1) ? TRUE : FALSE;
            break;
        case RangeThresholdExcessFlag:
            bRet = (pstcAdc->WCMRCOT_f.RCOOF == 1) ? TRUE : FALSE;
            break;
        default:
            break;
    }
    
    return bRet;
}

/**
 ******************************************************************************
 ** \brief Reads out Scan Conversion FIFO
 **
 ** This function reads out the Scan Conversion FIFO.
 **
 ** \pre Adc_ScanFifioStatus() should be called before
 **
 ** \param [in]  pstcAdc        ADC instance
 **
 ** \return                     Recent Scan Conversion FIFO value including
 **                             INVL, RS1, RS0, and Channel data as is.
 **                             If pstcAdc == NULL 0xFFFFFFFF is returned.
 ******************************************************************************/
uint32_t Adc_GetScanFifo( volatile stc_adcn_t* pstcAdc )
{
  // Check for NULL pointer
  if (pstcAdc  == NULL)
  {
    return 0xFFFFFFFFu;
  }
  
  return pstcAdc->SCFD;
} // Adc_ReadScanFifo

/**
 ******************************************************************************
 ** \brief Clear scan FIFO
 **
 ** This function clears the FIFO for scan conversion
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    Scan FIFO cleared
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t    Adc_ClrScanFifo(volatile stc_adcn_t* pstcAdc)
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pstcAdc->SCCR_f.SFCLR = 1;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Extract scan conversion data information from FIFO data
 **
 ** This function extracts the scan conversion data information from FIFO data
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  u32FifoData      FIFO data
 ** \param [out] pstcFifo         Pointer to FIFO information structure
 **
 ** \retval Ok                    Scan FIFO data extracted
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcAdc == NULL
 **                               - pstcFifo == NULL
 ******************************************************************************/
en_result_t  Adc_ExtractScanFifo(volatile stc_adcn_t* pstcAdc, 
                                 uint32_t u32FifoData, 
                                 stc_scan_fifo_info_t* pstcFifo)
{
    // Check for NULL pointer
    if ((pstcAdc  == NULL) || (pstcFifo == NULL))
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcAdc->ADSR_f.FDAS == 0)  
    {
        pstcFifo->u16Data = (uint16_t)(u32FifoData >> 20u);
    }
    else
    {
        pstcFifo->u16Data = (uint16_t)(u32FifoData >> 16u);
    }
    
    if((u32FifoData & 0x00001000u) == 0x00001000u)
    {
        pstcFifo->bInValid = 1;
    }
    else
    {
        pstcFifo->bInValid = 0;
    }
    
    if((u32FifoData & 0x00000300u) == 0x00000000u)
    {
        pstcFifo->enStartCause = ScanStartBySw;
    }
    else if((u32FifoData & 0x00000300u) == 0x00000100u)
    {
        pstcFifo->enStartCause = ScanStartByTimer;
    }
    
    pstcFifo->u8Ch = (uint8_t)(u32FifoData & 0x0000001Fu);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Reads out Priority Conversion FIFO
 **
 ** This function reads out the Priority Conversion FIFO.
 **
 ** \pre Adc_PrioFifioStatus() should be called before
 **
 ** \param [in]  pstcAdc        ADC instance
 **
 ** \return                     Recent Priority Conversion FIFO value including
 **                             INVL, RS2, RS1, RS0, and Channel data as is.
 **                             If pstcAdc == NULL 0xFFFFFFFF is returned.
 ******************************************************************************/
uint32_t Adc_GetPrioFifo( volatile stc_adcn_t* pstcAdc )
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return 0xFFFFFFFFu;
    }

    return pstcAdc->PCFD;
} // Adc_ReadPrioFifo

/**
 ******************************************************************************
 ** \brief Clear priority FIFO
 **
 ** This function clears the FIFO for priority conversion
 **
 ** \param [in]  pstcAdc          ADC instance
 **
 ** \retval Ok                    Priority FIFO cleared
 ** \retval ErrorInvalidParameter pstcAdc == NULL
 ******************************************************************************/
en_result_t  Adc_ClrPrioFifo(volatile stc_adcn_t* pstcAdc)
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pstcAdc->PCCR_f.PFCLR = 1;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Extract priority conversion data information from FIFO data
 **
 ** This function extracts the priority conversion data information from FIFO data
 **
 ** \param [in]  pstcAdc          ADC instance
 ** \param [in]  u32FifoData      FIFO data
 ** \param [out] pstcFifo         Pointer to FIFO information structure
 **
 ** \retval Ok                    Priority FIFO data extracted
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcAdc == NULL
 **                               - pstcFifo == NULL
 ******************************************************************************/
en_result_t    Adc_ExtractPrioFifo(volatile stc_adcn_t* pstcAdc, 
                                   uint32_t u32FifoData, 
                                   stc_prio_fifo_info_t* pstcFifo)
{
    // Check for NULL pointer
    if (pstcAdc  == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcAdc->ADSR_f.FDAS == 0)  
    {
        pstcFifo->u16Data = (uint16_t)(u32FifoData >> 20u);
    }
    else
    {
        pstcFifo->u16Data = (uint16_t)(u32FifoData >> 16u);
    }
    
    
    if((u32FifoData & 0x00001000u) == 0x00001000u)
    {
        pstcFifo->bInValid = 1;
    }
    else
    {
        pstcFifo->bInValid = 0;
    }
    
    if((u32FifoData & 0x00000700u) == 0x00000100u)
    {
        pstcFifo->enStartCause = PrioStartBySw;
    }
    else if((u32FifoData & 0x00000700u) == 0x000000200u)
    {
        pstcFifo->enStartCause = PrioStartByTimer;
    }
    else if((u32FifoData & 0x00000700u) == 0x000000400u)
    {
        pstcFifo->enStartCause = PrioStartByExtTrig;
    }
    pstcFifo->u8Ch = (uint8_t)(u32FifoData & 0x0000001Fu);
    
    return Ok;
}


//@} // AdcGroup

#endif // #if (defined(PDL_PERIPHERAL_ADC_ACTIVE))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
