/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file bt.c
 **
 ** A detailed description is available at 
 ** @link BtGroup BT Module description @endlink
 **
 ** History:
 **   - 2014-01-09  1.0  Edison Zhang  First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "bt.h"

#if (defined(PDL_PERIPHERAL_BT_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup BtGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
/// Look-up table for all enabled BT instances and their internal data
stc_bt_instance_data_t m_astcBtInstanceDataLut[BT_INSTANCE_COUNT] =
{
#if (PDL_PERIPHERAL_ENABLE_BT0 == PDL_ON)
    { 
        &BT0,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_BT1 == PDL_ON)
    { 
        &BT1,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_BT2 == PDL_ON)
    { 
        &BT2,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_BT3 == PDL_ON)
    { 
        &BT3,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_BT4 == PDL_ON)
    { 
        &BT4,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_BT5 == PDL_ON)
    { 
        &BT5,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_BT6 == PDL_ON)
    { 
        &BT6,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_BT7 == PDL_ON)
    { 
        &BT7,                 // pstcInstance
        {NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
};

/**
 ******************************************************************************
 ** \brief Return the internal data for a certain BT instance.
 **
 ** \param pstcBt Pointer to BT instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled (or not known)
 **
 ******************************************************************************/
static stc_bt_intern_data_t* BtGetInternDataPtr(volatile stc_btn_t* pstcBt) 
{
    uint32_t u32Instance;
   
    for (u32Instance = 0; u32Instance < BT_INSTANCE_COUNT; u32Instance++)
    {
        if (pstcBt == m_astcBtInstanceDataLut[u32Instance].pstcInstance)
        {
            return &m_astcBtInstanceDataLut[u32Instance].stcInternData;
        }
    }

    return NULL;
}

#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON)
/**
 ******************************************************************************
 ** \brief BT interrupt service routine
 **
 ** \param pstcBt BT instance
 ** \param pstcBtInternData Pointer to BT internal data      
 **        
 ******************************************************************************/      
void Bt_IrqHandler( volatile stc_btn_t* pstcBt,
                    stc_bt_intern_data_t* pstcBtInternData) 
{
    uint8_t u8Irq;
    
    // Get Interrupt Request
    u8Irq = pstcBt->STC; 
     /*
      * Trigger Interrupt (PWM,PPG,Reload timer)
      * Measurement completion Interrupt (PWC)
      */
    if ((u8Irq & (1ul<<2)) == (1ul<<2)) 
    {
        //  Clear Interrupt  
        pstcBt->STC &= ~(1ul<<2); 

        if (pstcBtInternData->pfnBit2IntCallback != NULL) 
        {
            // Call CallBackIrq 
            pstcBtInternData->pfnBit2IntCallback();
        }
    }
    
    // Duty Match Interrupt (PWM)   
    if ((u8Irq & (1ul<<1)) == (1ul<<1)) 
    {
        // Clear Interrupt 
        pstcBt->STC &= ~(1ul<<1); 

        if (pstcBtInternData->pfnBit1IntCallback != NULL) 
        {
            // Call CallBackIrq
            pstcBtInternData->pfnBit1IntCallback();
        }
    }
  
    /* 
     * Underflow Interrupt (PWM,PPG,Reload timer)
     * Overflow Interrupt (PWC) 
     */
    if ((u8Irq & 1ul) == 1ul) 
    {
        // Clear Interrupt 
        pstcBt->STC &= ~1ul;

        if (pstcBtInternData->pfnBit0IntCallback != NULL) 
        {
            // Call CallBackIrq
            pstcBtInternData->pfnBit0IntCallback();
        }
    }   

}      
  
/**
 ******************************************************************************
 ** \brief Set NVIC Interrupt depending on BT instance
 **
 ** \param pstcBt Pointer to BT instance
 **
 ******************************************************************************/
static void Bt_InitNvic(volatile stc_btn_t* pstcBt)
{
    if ((pstcBt == (volatile stc_btn_t*)(&BT0)) || (pstcBt == (volatile stc_btn_t*)(&BT1)) ||
        (pstcBt == (volatile stc_btn_t*)(&BT2)) || (pstcBt == (volatile stc_btn_t*)(&BT3)) ||
        (pstcBt == (volatile stc_btn_t*)(&BT4)) || (pstcBt == (volatile stc_btn_t*)(&BT5)) || 
        (pstcBt == (volatile stc_btn_t*)(&BT6)) || (pstcBt == (volatile stc_btn_t*)(&BT7)))
    {
        NVIC_ClearPendingIRQ(BTIM0_7_FLASH_IRQn); 
        NVIC_EnableIRQ(BTIM0_7_FLASH_IRQn); 
        NVIC_SetPriority(BTIM0_7_FLASH_IRQn, PDL_IRQ_LEVEL_BT0_7_FLASH);
    }
} 

/**
 ******************************************************************************
 ** \brief Clear NVIC Interrupt depending on BT instance
 **
 ** \param pstcBt Pointer to BT instance
 **
 ******************************************************************************/
static void Bt_DeInitNvic(volatile stc_btn_t* pstcBt)
{  
    if ((pstcBt == (volatile stc_btn_t*)(&BT0)) || (pstcBt == (volatile stc_btn_t*)(&BT1)) ||
        (pstcBt == (volatile stc_btn_t*)(&BT2)) || (pstcBt == (volatile stc_btn_t*)(&BT3)) ||
        (pstcBt == (volatile stc_btn_t*)(&BT4)) || (pstcBt == (volatile stc_btn_t*)(&BT5)) || 
        (pstcBt == (volatile stc_btn_t*)(&BT6)) || (pstcBt == (volatile stc_btn_t*)(&BT7)))
    {
        /* only when interrupts of ch0~ch7 and Flash are all disabled, disable NVIC */ 
        if((FM0P_BT0_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_BT1_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_BT2_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_BT3_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_BT4_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_BT5_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_BT6_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_BT7_PWM->STC & 0x70u) != 0x00u)
        {
            return;
        }
        if((FM0P_FLASH_IF->FICR & 0x03u) != 0x00u)
        {
            return;
        }
        NVIC_ClearPendingIRQ(BTIM0_7_FLASH_IRQn);
        NVIC_DisableIRQ(BTIM0_7_FLASH_IRQn);
        NVIC_SetPriority(BTIM0_7_FLASH_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);       
    }   
    
    return;
} 

#endif 

/**
 ******************************************************************************
 ** \brief Configure BT IO mode
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIoMode         BT IO mode
 ** \arg         BtIoMode0        BT IO mode 0
 ** \arg         BtIoMode1        BT IO mode 1
 ** \arg         BtIoMode2        BT IO mode 2
 ** \arg         BtIoMode3        BT IO mode 3
 ** \arg         BtIoMode4        BT IO mode 4
 ** \arg         BtIoMode5        BT IO mode 5
 ** \arg         BtIoMode6        BT IO mode 6
 ** \arg         BtIoMode7        BT IO mode 7
 ** \arg         BtIoMode8        BT IO mode 8
 **
 ** \retval Ok                    BT IO mode has been set successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 **                               Other invalid configuration
 ******************************************************************************/
en_result_t Bt_ConfigIOMode(volatile stc_btn_t* pstcBt, en_bt_io_mode_t enIoMode)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check for NULL pointer
    if ((pstcBt == NULL) || (enIoMode > BtIoMode8))
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if((pstcBt == &BT0) || (pstcBt == &BT1))
    {
        FM0P_BTIOSEL03->BTSEL0123_f.SEL01 = (uint8_t)enIoMode; 
    }
    else if((pstcBt == &BT2) || (pstcBt == &BT3))
    {
        FM0P_BTIOSEL03->BTSEL0123_f.SEL23 = (uint8_t)enIoMode; 
    }
    else if((pstcBt == &BT4) || (pstcBt == &BT5))
    {
        FM0P_BTIOSEL47->BTSEL4567_f.SEL45 = (uint8_t)enIoMode;
    }
    else if((pstcBt == &BT6) || (pstcBt == &BT7))
    {
        FM0P_BTIOSEL47->BTSEL4567_f.SEL67 = (uint8_t)enIoMode;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Select timer function of BT
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enTimerMode      Timer mode
 ** \arg         BtResetMode      Reset mode (Initial mode)
 ** \arg         BtPwmMode        PWM mode
 ** \arg         BtPpgMode        PPG mode 
 ** \arg         BtRtMode         Reload timer mode
 ** \arg         BtPwcMode        PWC mode
 **
 ** \retval Ok                    BT timer mode has been selected successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 **                               Other invalid configuration
 ******************************************************************************/
en_result_t Bt_SelTimerMode(volatile stc_btn_t* pstcBt, en_bt_timer_mode_t enTimerMode)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pstcBt->TMCR &= ~(7ul<<4);
    
    // Reset BT at first
    pstcBt->TMCR_f.FMD = 0u;
    
    // Set BT mode
    switch (enTimerMode)
    {
        case BtPwmMode:
            pstcBt->TMCR_f.FMD = 1u;
            break;
        case BtPpgMode:
            pstcBt->TMCR_f.FMD = 2u;
            break;
        case BtRtMode:
            pstcBt->TMCR_f.FMD = 3u;
            break;
        case BtPwcMode:
            pstcBt->TMCR_f.FMD = 4u;
            break;
        case BtResetMode:
            break;
        default:
            return ErrorInvalidParameter ;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Initialize PWM function of BT
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcPwmConfig    Pointer to PWM configuration
 **
 ** \retval Ok                    PWM function has been configured successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 **                               Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Pwm_Init(volatile stc_btn_t* pstcBt, stc_bt_pwm_config_t* pstcPwmConfig)
{
    volatile FM0P_BT_PWM_TypeDef* pRegBt = (volatile FM0P_BT_PWM_TypeDef*)pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    uint16_t u16Pres;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    // Check configuration parameter
    if((pstcPwmConfig->enPres > PwmPres1Div2048)       ||
       (pstcPwmConfig->enRestartEn > PwmRestartEnable) ||
       (pstcPwmConfig->enOutputMask > PwmOutputMask)   || 
       (pstcPwmConfig->enExtTrig > PwmExtTrigBoth)     || 
       (pstcPwmConfig->enOutputPolarity > PwmPolarityHigh) ||
       (pstcPwmConfig->enMode > PwmOneshot) )
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    u16Pres = pstcPwmConfig->enPres;  // Workaround for MISRA-C 2004 rule 12.7
    
    // Clock prescaler setting
    if(pstcPwmConfig->enPres < PwmPres1Div512)
    {
        pRegBt->TMCR2_f.CKS3 = 0u;
        pRegBt->TMCR_f.CKS = u16Pres;
    }
    else
    {
        pRegBt->TMCR2_f.CKS3 = 1u;
        pRegBt->TMCR_f.CKS = (u16Pres & 0x07u);
    }
    // Restart enable setting
    pRegBt->TMCR_f.RTGEN = (uint8_t)pstcPwmConfig->enRestartEn;
    // Output Mask setting
    pRegBt->TMCR_f.PMSK = (uint8_t)pstcPwmConfig->enOutputMask;
    // Output polarity setting
    pRegBt->TMCR_f.OSEL = (uint8_t)pstcPwmConfig->enOutputPolarity;
    // External trigger setting
    pRegBt->TMCR_f.EGS = (uint8_t)pstcPwmConfig->enExtTrig;
    // Mode setting
    pRegBt->TMCR_f.MDSE = (uint8_t)pstcPwmConfig->enMode;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief De-Initialize PWM function of BT
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    PWM function has been de-initialized successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 **                               Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Pwm_DeInit(volatile stc_btn_t* pstcBt)
{
    volatile FM0P_BT_PWM_TypeDef* pRegBt = (volatile FM0P_BT_PWM_TypeDef*)pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pRegBt->TMCR = 0x0000u;
    pRegBt->TMCR2 = 0x0000u;
    pRegBt->PCSR = 0x0000u;
    pRegBt->STC = 0x0000u;
    pRegBt->PDUT = 0x0000u;
#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
    Bt_DeInitNvic(pstcBt);
#endif    
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable PWM timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Enable PWM timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Pwm_EnableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->TMCR_f.CTEN = 1;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable PWM timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Disable PWM timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Pwm_DisableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->TMCR_f.CTEN = 0;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Start PWM timer by software
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Start PWM timer successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Pwm_EnableSwTrig(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->TMCR_f.STRG = 1;
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Enable PWM timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcIntSel       Pointer to interrupt selection structure        
 ** \param [in]  pstcIntCallback  Pointer to interrupt callback function
 **
 ** \retval Ok                    Enable PWM timer interrupt successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - pfnIntCallback == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Pwm_EnableInt(volatile stc_btn_t* pstcBt, stc_pwm_int_sel_t* pstcIntSel, 
                             stc_pwm_int_cb_t*  pstcIntCallback)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bPwmDutyMatchInt > 1) ||
        (pstcIntSel->bPwmTrigInt > 1) ||
        (pstcIntSel->bPwmUnderflowInt > 1) ||  
        (pstcIntCallback == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bPwmTrigInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = pstcIntCallback->pfnPwmTrigIntCallback;
        ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.TGIE = 1;
    }
    if(pstcIntSel->bPwmDutyMatchInt == 1)
    {
        pstcBtInternData->pfnBit1IntCallback = pstcIntCallback->pfnPwmDutyMatchIntCallback;
        ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.DTIE = 1;
    }
    if(pstcIntSel->bPwmUnderflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = pstcIntCallback->pfnPwmUnderflowIntCallback;
        ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.UDIE = 1;
    }

    Bt_InitNvic(pstcBt);    
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable PWM timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcIntSel       Pointer to interrupt selection structure   
 **
 ** \retval Ok                    Disable PWM timer interrupt successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 **                               Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Pwm_DisableInt(volatile stc_btn_t* pstcBt, stc_pwm_int_sel_t* pstcIntSel)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bPwmDutyMatchInt > 1) ||
        (pstcIntSel->bPwmTrigInt > 1) ||
        (pstcIntSel->bPwmUnderflowInt > 1))
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bPwmTrigInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = NULL;
        ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.TGIE = 0;
    }
    if(pstcIntSel->bPwmDutyMatchInt == 1)
    {
        pstcBtInternData->pfnBit1IntCallback = NULL;
        ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.DTIE = 0;
    }
    if(pstcIntSel->bPwmUnderflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = NULL;
        ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.UDIE = 0;
    }
    
    if(((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.TGIE == 1)
    {
        return Ok;
    }
    
    if(((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.DTIE == 1)
    {
        return Ok;
    }
    
    if(((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.UDIE == 1)
    {
        return Ok;
    }

    Bt_DeInitNvic(pstcBt); 
    
    return Ok;
}

#endif  

/**
 ******************************************************************************
 ** \brief Get interrupt flag of PWM timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIntType        PWM timer interrupt type
 ** \arg         PwmTrigInt       PWM trigger detection interrupt
 ** \arg         PwmDutyMatchInt  PWM duty match interrupt 
 ** \arg         PwmUnderflowInt  PWM underflow interrupt   
 **
 ** \retval PdlSet                Interrupt flag is set
 ** \retval PdlClr                Interrupt flag is clear
 ******************************************************************************/
en_int_flag_t Bt_Pwm_GetIntFlag(volatile stc_btn_t* pstcBt, en_bt_pwm_int_t enIntType)
{
    en_int_flag_t enFlag; 
    
    switch (enIntType)
    {
        case PwmTrigInt:
            enFlag = ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.TGIR ? PdlSet : PdlClr;
            break;
        case PwmDutyMatchInt:
            enFlag = ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.DTIR ? PdlSet : PdlClr;
            break;
        case PwmUnderflowInt:
            enFlag = ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.UDIR ? PdlSet : PdlClr;
            break;
        default:
            break;
    }
    
    return enFlag;
}

/**
 ******************************************************************************
 ** \brief Clear interrupt flag of PWM timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIntType        PWM timer interrupt type
 ** \arg         PwmTrigInt       PWM trigger detection interrupt
 ** \arg         PwmDutyMatchInt  PWM duty match interrupt 
 ** \arg         PwmUnderflowInt  PWM underflow interrupt   
 **
 ** \retval ErrorInvalidParameter pstcBt == NULL
 **                               enIntType > PwmUnderflowInt
 ** \retval Ok                    Clear interrupt flag successfully
 ******************************************************************************/
en_result_t Bt_Pwm_ClrIntFlag(volatile stc_btn_t* pstcBt, en_bt_pwm_int_t enIntType)
{
    if((pstcBt == NULL) || (enIntType > PwmUnderflowInt))
    {
        return ErrorInvalidParameter ;
    }
  
    switch (enIntType)
    {
        case PwmTrigInt:
            ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.TGIR = 0;
            break;
        case PwmDutyMatchInt:
            ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.DTIR = 0;
            break;
        case PwmUnderflowInt:
            ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->STC_f.UDIR = 0;
            break;
        default:
            break;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Write Cycle value of PWM timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  u16Cycle         Cycle value
 **
 ** \retval Ok                    Write Cycle value successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Pwm_WriteCycleVal(volatile stc_btn_t* pstcBt, uint16_t u16Cycle)
{
    // Check parameter
    if ((pstcBt == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->PCSR = u16Cycle;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Write duty value of PWM timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  u16Duty          Dury value
 **
 ** \retval Ok                    Write duty value successfully
 ** \retval ErrorInvalidParameter pstcBt == NUL
 ******************************************************************************/
en_result_t Bt_Pwm_WriteDutyVal(volatile stc_btn_t* pstcBt, uint16_t u16Duty)
{
    // Check parameter
    if ((pstcBt == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->PDUT = u16Duty;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read current count value of PWM timer 
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Current count value
 ******************************************************************************/
uint16_t Bt_Pwm_ReadCurCnt(volatile stc_btn_t* pstcBt)
{ 
    return ((volatile FM0P_BT_PWM_TypeDef*)pstcBt)->TMR;
}

/**
 ******************************************************************************
 ** \brief Initialize PPG function of BT
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcPpgConfig    Pointer to PPG configuration
 **
 ** \retval Ok                    PPG function has been configured successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Ppg_Init(volatile stc_btn_t* pstcBt, stc_bt_ppg_config_t* pstcPpgConfig)
{
    volatile FM0P_BT_PPG_TypeDef* pRegBt = (volatile FM0P_BT_PPG_TypeDef*)pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    uint16_t u16Pres;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    // Check configuration parameter
    if((pstcPpgConfig->enPres > PpgPres1Div2048)       ||
       (pstcPpgConfig->enRestartEn > PpgRestartEnable) ||
       (pstcPpgConfig->enOutputMask > PpgOutputMask)   || 
       (pstcPpgConfig->enExtTrig > PpgExtTrigBoth)     || 
       (pstcPpgConfig->enOutputPolarity > PpgPolarityHigh) ||
       (pstcPpgConfig->enMode > PpgOneshot) )
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    u16Pres = pstcPpgConfig->enPres; // Workaround for MISRA-C 2004 rule 12.7

    // Clock prescaler setting
    if(pstcPpgConfig->enPres < PpgPres1Div512)
    {
        pRegBt->TMCR2_f.CKS3 = 0u;
        pRegBt->TMCR_f.CKS = u16Pres;
    }
    else
    {
        pRegBt->TMCR2_f.CKS3 = 1u;
        pRegBt->TMCR_f.CKS = (u16Pres & 0x7u);
    }
    // Restart enable setting
    pRegBt->TMCR_f.RTGEN = (uint8_t)pstcPpgConfig->enRestartEn;
    // Output Mask setting
    pRegBt->TMCR_f.PMSK = (uint8_t)pstcPpgConfig->enOutputMask;
    // Output polarity setting
    pRegBt->TMCR_f.OSEL = (uint8_t)pstcPpgConfig->enOutputPolarity;
    // External trigger setting    
    pRegBt->TMCR_f.EGS = (uint8_t)pstcPpgConfig->enExtTrig;
    // Mode setting
    pRegBt->TMCR_f.MDSE = (uint8_t)pstcPpgConfig->enMode;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief De-Initialize PPG function of BT
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    PPG function has been de-initialized successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/                    
en_result_t Bt_Ppg_DeInit(volatile stc_btn_t* pstcBt)
{
    volatile FM0P_BT_PPG_TypeDef* pRegBt = (volatile FM0P_BT_PPG_TypeDef*)pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pRegBt->TMCR = 0x0000u;
    pRegBt->TMCR2 = 0x0000u;
    pRegBt->STC = 0x0000u;
    pRegBt->PRLL = 0x0000u;
    pRegBt->PRLH = 0x0000u;
    
#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
    Bt_DeInitNvic(pstcBt);
#endif    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable PPG timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Enable PPG timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Ppg_EnableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->TMCR_f.CTEN = 1;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable PPG timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Disable PPG timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Ppg_DisableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->TMCR_f.CTEN = 0;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Start PPG timer by software
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Start PPG timer successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Ppg_EnableSwTrig(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->TMCR_f.STRG = 1;
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Enable PPG timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcIntSel       Pointer to interrupt selection structure 
 ** \param [in]  pstcIntCallback  Pointer to interrupt callback function
 **
 ** \retval Ok                    Enable PPG timer interrupt successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - pfnIntCallback == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Ppg_EnableInt(volatile stc_btn_t* pstcBt, stc_ppg_int_sel_t* pstcIntSel, 
                             stc_ppg_int_cb_t*  pstcIntCallback)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bPpgTrigInt > 1) ||
        (pstcIntSel->bPpgUnderflowInt > 1) ||  
        (pstcIntCallback == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bPpgTrigInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = pstcIntCallback->pfnPpgTrigIntCallback;
        ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.TGIE = 1;
    }
    if(pstcIntSel->bPpgUnderflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = pstcIntCallback->pfnPpgUnderflowIntCallback;
        ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.UDIE = 1;
    }
    
    Bt_InitNvic(pstcBt);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable PPG timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcIntSel       Pointer to interrupt selection structure   
 **
 ** \retval Ok                    Disable PPG timer interrupt successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Ppg_DisableInt(volatile stc_btn_t* pstcBt, stc_ppg_int_sel_t* pstcIntSel)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bPpgTrigInt > 1) ||
        (pstcIntSel->bPpgUnderflowInt > 1)  )
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bPpgTrigInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = NULL;
        ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.TGIE = 0;
    }
    if(pstcIntSel->bPpgUnderflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = NULL;
        ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.UDIE = 0;
    }
    
    if(((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.TGIE == 1)
    {
        return Ok;
    }
    
    if(((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.UDIE == 1)
    {
        return Ok;
    }
         
    Bt_DeInitNvic(pstcBt);
         
    return Ok;
}

#endif

/**
 ******************************************************************************
 ** \brief Get interrupt flag of PPG timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIntType        PPG timer interrupt type
 ** \arg         PpgTrigInt       PPG trigger detection interrupt
 ** \arg         PpgUnderflowInt  PPG underflow interrupt
 **
 ** \retval PdlSet                Interrupt flag is set
 ** \retval PdlClr                Interrupt flag is clear
 ******************************************************************************/
en_int_flag_t Bt_Ppg_GetIntFlag(volatile stc_btn_t* pstcBt, en_bt_ppg_int_t enIntType)
{
    en_int_flag_t enFlag; 
    
    switch (enIntType)
    {
        case PpgTrigInt:
            enFlag = ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.TGIR ? PdlSet : PdlClr;
            break;
        case PpgUnderflowInt:
            enFlag = ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.UDIR ? PdlSet : PdlClr;
            break;
        default:
            break;
    }
    
    return enFlag;
}

/**
 ******************************************************************************
 ** \brief Clear interrupt flag of PPG timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIntType        PPG timer interrupt type
 ** \arg         PpgTrigInt       PPG trigger detection interrupt
 ** \arg         PpgUnderflowInt  PPG underflow interrupt
 **
 ** \retval ErrorInvalidParameter If one of following cases match:
 **                               - pstcBt == NULL
 **                               - enIntType > PwmUnderflowInt
 ** \retval Ok                    Clear interrupt flag successfully
 ******************************************************************************/
en_result_t Bt_Ppg_ClrIntFlag(volatile stc_btn_t* pstcBt, en_bt_ppg_int_t enIntType)
{
    if((pstcBt == NULL) || (enIntType > PpgUnderflowInt))
    {
        return ErrorInvalidParameter;
    }
  
    switch (enIntType)
    {
        case PpgTrigInt:
            ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.TGIR = 0;
            break;
        case PpgUnderflowInt:
            ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->STC_f.UDIR = 0;
            break;
        default:
            break;
    }
    
    return Ok;
}
                    
/**
 ******************************************************************************
 ** \brief Write low width count value of PPG timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  u16Val           Low width count value
 **
 ** \retval Ok                    Write low width count value successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Ppg_WriteLowWidthVal(volatile stc_btn_t* pstcBt, uint16_t u16Val)
{
    // Check parameter
    if ((pstcBt == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->PRLL = u16Val;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Write high width count value of PPG timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  u16Val           High width count value
 **
 ** \retval Ok                    Write high width count value successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Ppg_WriteHighWidthVal(volatile stc_btn_t* pstcBt, uint16_t u16Val)
{
    // Check parameter
    if ((pstcBt == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->PRLH = u16Val;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read current count value of PPG timer 
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Current count value
 ******************************************************************************/
uint16_t Bt_Ppg_ReadCurCnt(volatile stc_btn_t* pstcBt)
{ 
    return ((volatile FM0P_BT_PPG_TypeDef*)pstcBt)->TMR;
}

/**
 ******************************************************************************
 ** \brief Initialize RT function of BT
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcRtConfig     Pointer to RT configuration
 **
 ** \retval Ok                    RT function has been configured successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Rt_Init(volatile stc_btn_t* pstcBt, stc_bt_rt_config_t* pstcRtConfig)
{
    volatile FM0P_BT_RT_TypeDef* pRegBt = (volatile FM0P_BT_RT_TypeDef*) pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    uint16_t u16Pres, u16ExtTrig;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    // Check configuration parameter
    if((pstcRtConfig->enPres > RtPres1Div2048)           ||
       (pstcRtConfig->enSize > RtSize32Bit)              ||
       (pstcRtConfig->enExtTrig > RtExtTiggerHighLevel)  || 
       (pstcRtConfig->enOutputPolarity > RtPolarityHigh) ||
       (pstcRtConfig->enMode > RtOneshot) )
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    // Workaround for MISRA-C 2004 rule 12.7
    u16Pres = pstcRtConfig->enPres;
    
    // Clock prescaler setting
    if(pstcRtConfig->enPres < RtPres1Div512)
    {
        pRegBt->TMCR2_f.CKS3 = 0;
        pRegBt->TMCR_f.CKS = u16Pres;
    }
    else
    {
        pRegBt->TMCR2_f.CKS3 = 1;
        pRegBt->TMCR_f.CKS = (u16Pres & 0x7u);
        
    }
    // Timer size setting
    pRegBt->TMCR_f.T32 = (uint8_t)pstcRtConfig->enSize;
    // Output polarity setting
    pRegBt->TMCR_f.OSEL = (uint8_t)pstcRtConfig->enOutputPolarity;
    
    // Workaround for MISRA-C 2004 rule 12.7
    u16ExtTrig = pstcRtConfig->enExtTrig;
    
    // External trigger setting
    if(pstcRtConfig->enExtTrig <= RtExtTiggerBothEdge)
    {
        pRegBt->TMCR2_f.GATE = 0u;        
        pRegBt->TMCR_f.EGS = u16ExtTrig;
    }
    else if (pstcRtConfig->enExtTrig == RtExtTiggerLowLevel)
    {
        pRegBt->TMCR2_f.GATE = 1u;        
        pRegBt->TMCR_f.EGS = 0u;
    }
    else if (pstcRtConfig->enExtTrig == RtExtTiggerHighLevel)
    {
        pRegBt->TMCR2_f.GATE = 1u;
        pRegBt->TMCR_f.EGS = 1u;
    }
    // Mode setting
    pRegBt->TMCR_f.MDSE = (uint8_t)pstcRtConfig->enMode;
    
    return Ok;
}


/**
 ******************************************************************************
 ** \brief De-Initialize RT function of BT
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    RT function has been de-initialized successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/                    
en_result_t Bt_Rt_DeInit(volatile stc_btn_t* pstcBt)
{
    volatile FM0P_BT_RT_TypeDef* pRegBt = (volatile FM0P_BT_RT_TypeDef*)pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pRegBt->TMCR = 0x0000u;
    pRegBt->TMCR2 = 0x0000u;
    pRegBt->STC = 0x0000u;
    pRegBt->PCSR = 0x0000u;
    
#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
    Bt_DeInitNvic(pstcBt);
#endif    
    return Ok;
}
        
        
/**
 ******************************************************************************
 ** \brief Enable Reload timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Enable RT timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Rt_EnableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->TMCR_f.CTEN = 1;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable Reload timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Disable RT timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Rt_DisableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->TMCR_f.CTEN = 0;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Start Reload timer by software
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Start Reload timer successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Rt_EnableSwTrig(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->TMCR_f.STRG = 1;
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Enable Reload timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcIntSel       Pointer to interrupt select structure
 ** \param [in]  pstcIntCallback  Pointer to interrupt callback function
 **
 ** \retval Ok                    Enable Reload timer interrupt successfully
 ** \retval ErrorInvalidParameter If one of followng cases matchs:
 **                               - pstcBt == NULL
 **                               - pfnIntCallback == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Rt_EnableInt(volatile stc_btn_t* pstcBt, stc_rt_int_sel_t* pstcIntSel, 
                            stc_rt_int_cb_t*  pstcIntCallback)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bRtTrigInt > 1) ||
        (pstcIntSel->bRtUnderflowInt > 1) ||  
        (pstcIntCallback == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bRtTrigInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = pstcIntCallback->pfnRtTrigIntCallback;
        ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.TGIE = 1;
    }
    if(pstcIntSel->bRtUnderflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = pstcIntCallback->pfnRtUnderflowIntCallback;
        ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.UDIE = 1;
    }

    Bt_InitNvic(pstcBt);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable Reload timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcIntSel       Pointer to interrupt select structure
 **
 ** \retval Ok                    Disable Reload timer interrupt successfully
 ** \retval ErrorInvalidParameter If one of followng cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Rt_DisableInt(volatile stc_btn_t* pstcBt, stc_rt_int_sel_t* pstcIntSel)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bRtTrigInt > 1) ||
        (pstcIntSel->bRtUnderflowInt > 1)  )
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bRtTrigInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = NULL;
        ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.TGIE = 0;
    }
    if(pstcIntSel->bRtUnderflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = NULL;
        ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.UDIE = 0;
    }
    
    Bt_DeInitNvic(pstcBt);
    
    return Ok;
}

#endif

/**
 ******************************************************************************
 ** \brief Get interrupt flag of Reload timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIntType        Reload timer interrupt type
 ** \arg         RtTrigInt        Reload timer trigger detection flag
 ** \arg         RtUnderflowInt   Reload timer underflow interrupt flag
 **
 ** \retval PdlSet                Interrupt flag is set
 ** \retval PdlClr                Interrupt flag is clear
 ******************************************************************************/
en_int_flag_t Bt_Rt_GetIntFlag(volatile stc_btn_t* pstcBt, en_bt_rt_int_t enIntType)
{
    en_int_flag_t enFlag; 
    
    switch (enIntType)
    {
        case RtTrigInt:
            enFlag = ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.TGIR ? PdlSet : PdlClr;
            break;
        case RtUnderflowInt:
            enFlag = ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.UDIR ? PdlSet : PdlClr;
            break;
        default:
            break;
    }
    
    return enFlag;
}

/**
 ******************************************************************************
 ** \brief Clear interrupt flag of Reload timer 
 **
 ** \param [in]  pstcBt             BT instance
 ** \param [in]  enIntType          Reload timer interrupt type
 ** \arg         RtTrigInt          Reload timer trigger detection flag
 ** \arg         RtUnderflowInt     Reload timer underflow interrupt flag
 **
 ** \retval ErrorInvalidParameter   If one of following cases matchs:
 **                                 - pstcBt == NULL
 **                                 - enIntType > RtUnderflowInt
 ** \retval Ok                      Clear interrupt flag successfully
 ******************************************************************************/
en_result_t Bt_Rt_ClrIntFlag(volatile stc_btn_t* pstcBt, en_bt_rt_int_t enIntType)
{
    if((pstcBt == NULL) || (enIntType > RtUnderflowInt))
    {
        return ErrorInvalidParameter;
    }
    
    switch (enIntType)
    {
        case RtTrigInt:
            ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.TGIR = 0;
            break;
        case RtUnderflowInt:
            ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->STC_f.UDIR = 0;
            break;
        default:
            break;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Write count cycle of Reload timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  u16Val           Cycle value
 **
 ** \retval Ok                    Write count cycle successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Rt_WriteCycleVal(volatile stc_btn_t* pstcBt, uint16_t u16Val)
{
    // Check parameter
    if ((pstcBt == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->PCSR = u16Val;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read current count value of Reload timer 
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Current count value
 ******************************************************************************/
uint16_t Bt_Rt_ReadCurCnt(volatile stc_btn_t* pstcBt)
{ 
    return ((volatile FM0P_BT_RT_TypeDef*)pstcBt)->TMR;
}

/**
 ******************************************************************************
 ** \brief Initialize PWC function of BT
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcPwcConfig    Pointer to PWC configuration
 **
 ** \retval Ok                    PWC function has been configured successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Pwc_Init(volatile stc_btn_t* pstcBt, stc_bt_pwc_config_t* pstcPwcConfig)
{
    volatile FM0P_BT_PWC_TypeDef* pRegBt = (volatile FM0P_BT_PWC_TypeDef*) pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    uint16_t u16Pres;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    // Check configuration parameter
    if((pstcPwcConfig->enPres > PwcPres1Div2048)           ||
       (pstcPwcConfig->enSize > PwcSize32Bit)              ||
       (pstcPwcConfig->enMeasureEdge > PwcMeasureFallingToRising)  || 
       (pstcPwcConfig->enMode > PwcOneshot) )
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    // workaround for MISRA-C 2004 rule 12.7
    u16Pres = pstcPwcConfig->enPres;
    
    // Clock prescaler setting
    if(pstcPwcConfig->enPres < PwcPres1Div512)
    {
        pRegBt->TMCR2_f.CKS3 = 0u;
        pRegBt->TMCR_f.CKS = u16Pres;
    }
    else
    {
        pRegBt->TMCR2_f.CKS3 = 1u;
        pRegBt->TMCR_f.CKS = (u16Pres & 0x07u);
    }
    // Timer size setting
    pRegBt->TMCR_f.T32 = (uint8_t)pstcPwcConfig->enSize;
    // Measurement mode setting
    pRegBt->TMCR_f.EGS = (uint8_t)pstcPwcConfig->enMeasureEdge;
    // Mode setting
    pRegBt->TMCR_f.MDSE = (uint8_t)pstcPwcConfig->enMode;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief De-Initialize PWC function of BT
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    PWC function has been de-initialized successfully
 ** \retval ErrorInvalidParameter If one of followng cases matchs:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/                    
en_result_t Bt_Pwc_DeInit(volatile stc_btn_t* pstcBt)
{
    volatile FM0P_BT_PWC_TypeDef* pRegBt = (volatile FM0P_BT_PWC_TypeDef*)pstcBt;
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    pRegBt->TMCR = 0x0000u;
    pRegBt->TMCR2 = 0x0000u;
    pRegBt->STC = 0x0000u;
    
#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
    Bt_DeInitNvic(pstcBt);
#endif    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable PWC timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Enable PWC timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Pwc_EnableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->TMCR_f.CTEN = 1;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable PWC timer counting
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Ok                    Disable PWC timer counting successfully
 ** \retval ErrorInvalidParameter pstcBt == NULL
 ******************************************************************************/
en_result_t Bt_Pwc_DisableCount(volatile stc_btn_t* pstcBt)
{
    // Check for NULL pointer
    if (pstcBt == NULL)
    {
        return ErrorInvalidParameter ;
    }  
    
    ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->TMCR_f.CTEN = 0;
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) ||  (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Enable PWC timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  pstcIntSel       Pointer to interrupt selection structure
 ** \param [in]  pstcIntCallback  Pointer to interrupt callback function
 **
 ** \retval Ok                    Enable PWC timer interrupt successfully
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - pstcBt == NULL
 **                               - pfnIntCallback == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Pwc_EnableInt(volatile stc_btn_t* pstcBt, stc_pwc_int_sel_t* pstcIntSel, 
                             stc_pwc_int_cb_t*  pstcIntCallback)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bPwcMeasCmpInt > 1) ||
        (pstcIntSel->bPwcMeasOverflowInt > 1) || 
        (pstcIntCallback == NULL))
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bPwcMeasCmpInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = pstcIntCallback->pfnPwcMeasCmpCallback;
        ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.EDIE = 1;
    }
    if(pstcIntSel->bPwcMeasOverflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = pstcIntCallback->pfnPwcMeasOverflowCallback;
        ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.OVIE = 1;
    }
    
    Bt_InitNvic(pstcBt);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable PWC timer interrupt
 **
 ** \param [in]  pstcBt           BT instance
  ** \param [in]  pstcIntSel      Pointer to interrupt selection structure
 **
 ** \retval Ok                    Disable PWC timer interrupt successfully
 ** \retval ErrorInvalidParameter If one of following cases match:
 **                               - pstcBt == NULL
 **                               - Other invalid configuration
 ******************************************************************************/
en_result_t Bt_Pwc_DisableInt(volatile stc_btn_t* pstcBt, stc_pwc_int_sel_t* pstcIntSel)
{
    // Pointer to internal data
    stc_bt_intern_data_t* pstcBtInternData ;
    // Check parameter
    if ((pstcBt == NULL) ||
        (pstcIntSel->bPwcMeasCmpInt > 1) ||
        (pstcIntSel->bPwcMeasOverflowInt > 1))
    {
        return ErrorInvalidParameter ;
    }  
    
    // Get pointer to internal data structure ...
    pstcBtInternData = BtGetInternDataPtr( pstcBt ) ;
    
    if(pstcBtInternData == NULL)
    {
        return ErrorInvalidParameter ;
    }
    
    if(pstcIntSel->bPwcMeasCmpInt == 1)
    {
        pstcBtInternData->pfnBit2IntCallback = NULL;
        ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.EDIE = 0;
    }
    if(pstcIntSel->bPwcMeasOverflowInt == 1)
    {
        pstcBtInternData->pfnBit0IntCallback = NULL;
        ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.OVIE = 0;
    }
    
    if(((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.EDIE == 1)
    {
        return Ok;
    }
    
    if(((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.OVIE == 1)
    {
        return Ok;
    }
     
    Bt_DeInitNvic(pstcBt);
    
    return Ok;
}
#endif

/**
 ******************************************************************************
 ** \brief Get interrupt flag of PWC timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIntType        PWC timer interrupt type
 ** \arg         PwcMeasureCompleteInt   PWC measure complete interrupt 
 ** \arg         PwcMeasureOverflowInt   PWC overflow interrupt
 **
 ** \retval PdlSet                Interrupt flag is set
 ** \retval PdlClr                Interrupt flag is clear
 ******************************************************************************/
en_int_flag_t Bt_Pwc_GetIntFlag(volatile stc_btn_t* pstcBt, en_bt_pwc_int_t enIntType)
{
    en_int_flag_t enFlag; 
    
    switch (enIntType)
    {
        case PwcMeasureCompleteInt:
            enFlag = ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.EDIR ? PdlSet : PdlClr;
            break;
        case PwcMeasureOverflowInt:
            enFlag = ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.OVIR ? PdlSet : PdlClr;
            break;
        default:
            break;
    }
    
    return enFlag;
}

/**
 ******************************************************************************
 ** \brief Clear interrupt flag of PWC timer 
 **
 ** \param [in]  pstcBt           BT instance
 ** \param [in]  enIntType        PWC timer interrupt type
 ** \arg         PwcMeasureCompleteInt   PWC measure complete interrupt 
 ** \arg         PwcMeasureOverflowInt   PWC overflow interrupt
 **
 ** \retval ErrorInvalidParameter If one of followng cases matchs:
 **                               - pstcBt == NULL
 **                               - enIntType > PwcMeasureOverflowInt
 ** \retval Ok                    Clear interrupt flag successfully
 ******************************************************************************/
en_result_t Bt_Pwc_ClrIntFlag(volatile stc_btn_t* pstcBt, en_bt_pwc_int_t enIntType)
{
    if((pstcBt == NULL) || (enIntType > PwcMeasureOverflowInt))
    {
        return ErrorInvalidParameter;
    }
  
    switch (enIntType)
    {
        case PwcMeasureCompleteInt:
            break;
        case PwcMeasureOverflowInt:
            ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.OVIR = 0;
            break;
        default:
            break;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get error flag of PWC timer
 **
 ** If the measured data is not read out when the next data is coming,
 ** the error flag will be set.
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval PdlSet                Error flag is set
 ** \retval PdlClr                Error flag is clear
 ******************************************************************************/
en_stat_flag_t Bt_Pwc_GetErrorFlag(volatile stc_btn_t* pstcBt)
{
    en_int_flag_t enFlag; 
    
    enFlag = ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->STC_f.ERR ? PdlSet : PdlClr;
    
    return enFlag;
}

/**
 ******************************************************************************
 ** \brief Get 16 bits measure data of PWC timer
 **
 ** This function reads measure data in 16-bit timer mode.
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Measure data       
 ******************************************************************************/
uint16_t Bt_Pwc_Get16BitMeasureData(volatile stc_btn_t* pstcBt)
{ 
    uint16_t u16RegDtbf;
    u16RegDtbf = ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->DTBF;
    return u16RegDtbf;
}

/**
 ******************************************************************************
 ** \brief Get 32 bits measure data of PWC timer
 **
 ** This function reads measure data in 32-bit timer mode, the input BT instance
 ** should point to register base address of even channel. 
 **
 ** \param [in]  pstcBt           BT instance
 **
 ** \retval Measure data 
 ******************************************************************************/
uint32_t Bt_Pwc_Get32BitMeasureData(volatile stc_btn_t* pstcBt)
{ 
    uint16_t u16RegLow, u16RegHigh;
    
    if((pstcBt == &BT1) || (pstcBt == &BT3) || (pstcBt == &BT5) || (pstcBt == &BT7))
    {
        return 0;
    }
    u16RegLow = ((volatile FM0P_BT_PWC_TypeDef*)pstcBt)->DTBF;
    u16RegHigh = ((volatile FM0P_BT_PWC_TypeDef*)((volatile uint8_t*)pstcBt + 0x40))->DTBF;
    return (((uint32_t)u16RegHigh<<16) | (uint32_t)u16RegLow);
}

/**
 ******************************************************************************
 ** \brief Set the Simultaneous Start register of Base timer
 **
 ** This function is only valid in BTIO mode 5 and mode 6, otherwise don't 
 ** use this function!
 **
 ** \param [in]  u16Value           The bit map is shown as following:
 **                                 - Bit0 ----- Ch.0
 **                                 - Bit1 ----- Ch.1
 **                                 - ...
 **                                 - Bit15 ----- Ch.15
 **
 ** \retval None
 ******************************************************************************/
void Bt_SetSimultaneousStart(uint16_t u16Value)
{
    FM0P_SBSSR->BTSSSR = u16Value;
}

//@}

#endif
