/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file exint.h
 **
 ** Headerfile for EXINT functions
 **  
 ** History:
 **   - 2014-02-21  1.0  Edison Zhang  First version.
 **
 ******************************************************************************/

#ifndef __EXINT_H__
#define __EXINT_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "pdl_user.h"

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup ExintGroup External Interrupts (EXINT)
 **
 ** Provided functions of EXINT module:
 **
 ** - Exint_Enable() 
 ** - Exint_Disable() 
 ** - Exint_SetIntDetectMode()
 ** - Exint_GetIntDetectMode()
 ** - Exint_IrqHandler()
 **
 **  Exint_Enable() starts external interrupt detection and Exint_Disable()
 **  stops external interrupt detection.
 ** 
 **  Exint_SetIntDetectMode() sets the detection level of external interrupt.
 **  
 **  Exint_GetIntDetectMode() gets the detection level of external interrupt.
 ** 
 **  Exint_IrqHandler() is external interrupt IRQ handler, which is called
 **  in the interrupts.c
 ** 
 ******************************************************************************/
//@{
  
#if (defined(PDL_PERIPHERAL_EXINT_ACTIVE))  
/******************************************************************************/
/* Global pre-processor symbols/macros ('#define')                            */
/******************************************************************************/
#define EXINT_CHANNEL_NUM        (8u)
#define EXINT_MAX_CH_INDEX       (31u)                             

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/
  
typedef enum en_exint_level
{
    ExIntLowLevel     = 0,  ///< "L" level detection
    ExIntHighLevel    = 1,  ///< "H" level detection
    ExIntRisingEdge   = 2,  ///< Rising edge detection
    ExIntFallingEdge  = 3   ///< Falling edge detection
} en_exint_level_t;

/// Enumeration to define an index for each enabled EXINT instance
typedef enum en_exint_instance_index
{
    ExintInstanceIndexExint0,
    ExintInstanceIndexExint1,
    ExintInstanceIndexExint2,
    ExintInstanceIndexExint3,
    ExintInstanceIndexExint4,
    ExintInstanceIndexExint5,
    ExintInstanceIndexExint6,
    ExintInstanceIndexExint7,
    ExintInstanceIndexExint8,
    ExintInstanceIndexExint9,
    ExintInstanceIndexExint10,
    ExintInstanceIndexExint11,
    ExintInstanceIndexExint12,
    ExintInstanceIndexExint13,
    ExintInstanceIndexExint14,
    ExintInstanceIndexExint15,
    ExintInstanceIndexExint16,
    ExintInstanceIndexExint17,
    ExintInstanceIndexExint18,
    ExintInstanceIndexExint19,
    ExintInstanceIndexExint20,
    ExintInstanceIndexExint21,
    ExintInstanceIndexExint22,
    ExintInstanceIndexExint23,
    ExintInstanceIndexExint24,
    ExintInstanceIndexExint25,
    ExintInstanceIndexExint26,
    ExintInstanceIndexExint27,
    ExintInstanceIndexExint28,
    ExintInstanceIndexExint29,
    ExintInstanceIndexExint30,
    ExintInstanceIndexExint31,
    ExintInstanceIndexMax
} en_exint_instance_index_t;

/******************************************************************************/
/* Global function prototypes (definition in C source)                        */
/******************************************************************************/
/* external interrupt */
en_result_t Exint_Enable(uint8_t u8Ch, func_ptr_t pfnIntCb);
en_result_t Exint_Disable(uint8_t u8Ch);
en_result_t Exint_SetIntDetectMode(uint8_t u8Ch, en_exint_level_t enLevel);
en_exint_level_t Exint_GetIntDetectMode(uint8_t u8Ch);
void Exint_IrqHandler(uint8_t u8Channel);
#endif // #if (defined(PDL_PERIPHERAL_EXINT_ACTIVE))

#if (defined(PDL_PERIPHERAL_NMI_ACTIVE))  
/* NMI */
en_result_t Exint_Nmi_SetIntCallback(func_ptr_t pfnIntCb);
void Nmi_IrqHandler(void);
#endif

//@} // ExintGroup

#ifdef __cplusplus
}
#endif



#endif /* __EXINT_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
