/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file interrupts.h
 **
 ** PDL Interrupt Handlers
 **
 ** History:
 **   - 2013-03-21  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/
#ifndef __INTERRUPTS_H__
#define __INTERRUPTS_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_user.h"

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup PdlInterrupts PDL Interrupt handling
 **
 ** Each device group of the FM0P MCUs may have individual interrupt service
 ** routine vector names. For this reason this file defines the device
 ** depenent vectors, if the corresponding resource is set to active in 
 ** pdl_user.h.
 **
 ** Interrupt and Callback flow in PDL:
 ** \image html Interrupt_flow.png
 **
 ******************************************************************************/
//@{
 
/*****************************************************************************/
/* Check whether interrupt is enable when peripheral is inactive             */
/*****************************************************************************/
// Include adc.h if ADC is active and ADC interrupt is enabled.
#if (PDL_INTERRUPT_ENABLE_ADC0) || (PDL_INTERRUPT_ENABLE_ADC1) || (PDL_INTERRUPT_ENABLE_ADC2)
    #if defined(PDL_PERIPHERAL_ADC_ACTIVE)
        #include "adc\adc.h"
    #else
        #error Don't enable ADC interrupt when it is inactive!
    #endif
#endif    

// Include adc.h if BT is active and BT interrupt is enabled.
#if (PDL_INTERRUPT_ENABLE_BT0) || (PDL_INTERRUPT_ENABLE_BT1) || \
    (PDL_INTERRUPT_ENABLE_BT2) || (PDL_INTERRUPT_ENABLE_BT3) || \
    (PDL_INTERRUPT_ENABLE_BT4) || (PDL_INTERRUPT_ENABLE_BT5) || \
    (PDL_INTERRUPT_ENABLE_BT6) || (PDL_INTERRUPT_ENABLE_BT7)   
    #if defined(PDL_PERIPHERAL_BT_ACTIVE)
        #include "bt\bt.h"
    #else
        #error Don't enable BT interrupt when it is inactive!
    #endif
#endif  

// Include clk.h if clock is active and clock interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)   
    #if defined(PDL_PERIPHERAL_CLK_ACTIVE)
        #include "clk\clk.h"
    #else  
        #error Don't enable clock interrupt when it is inactive!
    #endif
#endif      

// Include csv.h if clock is active and clock interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_CSV == PDL_ON)      
    #if defined(PDL_PERIPHERAL_CSV_ACTIVE)
        #include "csv\csv.h"
    #else 
        #error Don't enable CSV interrupt when it is inactive!
    #endif  
#endif      

// Include dma.h if DMA is active and DMA interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_DMA0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_DMA2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA3 == PDL_ON)  
    #if defined(PDL_PERIPHERAL_DMA_ACTIVE)   
        #include "dma\dma.h"
    #else  
        #error Don't enable DMA interrupt when it is inactive!
    #endif
#endif      

// Include dt.h if DT is active and DT interrupt is enabled.          
#if (PDL_INTERRUPT_ENABLE_DT == PDL_ON)         
    #if defined(PDL_PERIPHERAL_DT_ACTIVE)
        #include "dt\dt.h"
    #else
        #error Don't enable DT interrupt when it is inactive!
    #endif
#endif      

// Include exint.h if EXT INT is active and EXT INT is enabled.          
#if (PDL_INTERRUPT_ENABLE_EXINT0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT7 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT8 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT9 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT10 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT12 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT14 == PDL_ON) || (PDL_INTERRUPT_ENABLE_EXINT15 == PDL_ON)  
    #if defined(PDL_PERIPHERAL_EXINT_ACTIVE)
        #include "exint\exint.h"
    #else
        #error Don't enable external interrupt when it is inactive!
    #endif
#endif      

// Include flash.h if Flash is active and Flash interrupt is enabled.  
#if (PDL_INTERRUPT_ENABLE_FLASH == PDL_ON)      
    #if defined(PDL_PERIPHERAL_FLASH_ACTIVE)
        #include "flash\flash.h"
    #else
        #error Don't enable Flash interrupt when it is inactive!
    #endif
#endif      

// Include lvd.h if LVD is active and LVD interrupt is enabled.       
#if (PDL_INTERRUPT_ENABLE_LVD == PDL_ON)      
    #if defined(PDL_PERIPHERAL_LVD_ACTIVE)
        #include "lvd\lvd.h"
    #else  
        #error Don't enable LVD interrupt when it is inactive!
    #endif
#endif      

// Include mfs.h if MFS is active and MFS interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON)  
    #if defined(PDL_PERIPHERAL_MFS_ACTIVE)
        #include "mfs\mfs.h"    
    #else
        #error Don't enable MFS interrupt when it is inactive!
    #endif
#endif      

// Include mft_frt.h if FRT is active and FRT interrupt is enabled.       
#if (PDL_INTERRUPT_ENABLE_MFT0_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_FRT == PDL_ON)  
    #if defined(PDL_PERIPHERAL_MFT_FRT_ACTIVE)  
        #include "mft\mft_frt.h"
    #else
        #error Don't enable MFT's FRT interrupt when it is inactive!
    #endif
#endif      

// Include mft_ocu.h if OCU is active and OCU interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_MFT0_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_OCU == PDL_ON)
    #if defined(PDL_PERIPHERAL_MFT_OCU_ACTIVE)  
        #include "mft\mft_ocu.h"
    #else
        #error Don't enable MFT's OCU interrupt when it is inactive!
    #endif
#endif      

// Include mft_wfg.h if WFG is active and WFG interrupt is enabled.       
#if (PDL_INTERRUPT_ENABLE_MFT0_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_WFG == PDL_ON)
    #if defined(PDL_PERIPHERAL_MFT_WFG_ACTIVE)   
        #include "mft\mft_wfg.h"
    #else
        #error Don't enable MFT's WFG interrupt when it is inactive!
    #endif
#endif      

// Include mft_icu.h if ICU is active and ICU interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_MFT0_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_ICU == PDL_ON)
    #if defined(PDL_PERIPHERAL_MFT_ICU_ACTIVE)
        #include "mft\mft_icu.h"
    #else
        #error Don't enable MFT's ICU interrupt when it is inactive!
    #endif
#endif      

// Include ppg.h if PPG is active and PPG interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_PPG == PDL_ON)      
    #if defined(PDL_PERIPHERAL_PPG_ACTIVE)
        #include "ppg\ppg.h"  
    #else
        #error Don't enable PPG interrupt when it is inactive!
    #endif
#endif      

// Include qprc.h if QPRC is active and QPRC interrupt is enabled.         
#if (PDL_INTERRUPT_ENABLE_QPRC0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_QPRC1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_QPRC2 == PDL_ON)   
    #if defined(PDL_PERIPHERAL_QPRC_ACTIVE)
        #include "qprc\qprc.h"
    #else
        #error Don't enable QPRC interrupt when it is inactive!
    #endif
#endif      

// Include rtc.h if RTC is active and RTC interrupt is enabled.        
#if (PDL_INTERRUPT_ENABLE_RTC == PDL_ON)      
    #if defined(PDL_PERIPHERAL_RTC_ACTIVE)
        #include "rtc\rtc.h"
    #else
        #error Don't enable RTC interrupt when it is inactive!
    #endif
#endif      

// Include wc.h if WC is active and WC interrupt is enabled.       
#if (PDL_INTERRUPT_ENABLE_WC == PDL_ON)         
    #if defined(PDL_PERIPHERAL_WC_ACTIVE)
        #include "wc\wc.h"
    #else
        #error Don't enable WC interrupt when it is inactive!
    #endif
#endif      

// Include wdg.h if watchdog is active and watchdog interrupt is enabled.      
#if (PDL_INTERRUPT_ENABLE_HWWDG == PDL_ON) || (PDL_INTERRUPT_ENABLE_SWWDG == PDL_ON)       
    #if defined(PDL_PERIPHERAL_WDG_ACTIVE)
        #include "wdg\wdg.h"
    #else
        #error Don't enable watchdog interrupt when it is inactive!
    #endif   
#endif      

//@} // PdlInterrupts
  
#ifdef __cplusplus
}
#endif  
  
#endif /* __INTERRUPTS_H__ */

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
