/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file mft_wfg.c
 **
 ** A detailed description is available at 
 ** @link WfgGroup WFG Module description @endlink
 **  
 ** History:
 **   - 2013-12-21  0.1  Edison Zhang  First version.
 **
 *****************************************************************************/


/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mft_wfg.h"


#if (defined(PDL_PERIPHERAL_MFT_WFG_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup WfgGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/
/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
/// Look-up table for all enabled WFG  instances and their internal data
stc_mft_wfg_instance_data_t m_astcMftWfgInstanceDataLut[WFG_INSTANCE_COUNT] =
{
#if (PDL_PERIPHERAL_ENABLE_MFT0_WFG == PDL_ON)
    { 
        &MFT0_WFG,  // pstcInstance
        {NULL,NULL,NULL,NULL,NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT1_WFG == PDL_ON)
    { 
        &MFT1_WFG,  // pstcInstance
        {NULL,NULL,NULL,NULL,NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT2_WFG == PDL_ON)
    { 
        &MFT2_WFG,  // pstcInstance
        {NULL,NULL,NULL,NULL,NULL}    // stcInternData (not initialized yet)
    },
#endif
};

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
/**
 *****************************************************************************
 ** \brief Return the internal data for a certain WFG instance.
 **
 ** \param pstcWfg Pointer to WFG instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled 
 **         (or not known)
 **
 *****************************************************************************/
static stc_mft_wfg_intern_data_t* MftGetInternDataPtr(volatile stc_mftn_wfg_t* pstcWfg)
{
    uint32_t u32Instance;
   
    for (u32Instance = 0; u32Instance < WFG_INSTANCE_COUNT; u32Instance++)
    {
        if (pstcWfg == m_astcMftWfgInstanceDataLut[u32Instance].pstcInstance)
        {
            return &m_astcMftWfgInstanceDataLut[u32Instance].stcInternData;
        }
    }

    return NULL;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_WFG == PDL_ON)   
/*!
 ******************************************************************************
 ** \brief  WFG interrupt handler sub function
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \param [in]  pstcMftWfgInternData  structure of WFG callback function
 ** \arg    structure of stc_mft_intern_data
 *****************************************************************************
*/
void Mft_Wfg_IrqHandler( volatile  stc_mftn_wfg_t* pstcWfg,
		         stc_mft_wfg_intern_data_t* pstcMftWfgInternData)
{
    func_ptr_t funCallBack;
    uint32_t* ptemp;
    uint8_t ch;

    /* timer interrupt*/
    for(ch=0;ch<3;ch++)
    {
        if(PdlSet == Mft_Wfg_GetTimerIntFlag(pstcWfg,ch))
        {
            /* Clear Interrupt */
            Mft_Wfg_ClrTimerIntFlag(pstcWfg,ch);

            //get peak timer interrupt callback address of each channel
            ptemp = (uint32_t*)&(pstcMftWfgInternData->pfnWfg10TimerCallback);
            funCallBack = (func_ptr_t)(*(ptemp + ch));

            if(funCallBack != NULL)
            {
               funCallBack();
            }
        }
    }

    /* DTIF Interrupt */
    //Digital filter interrupt
    if(PdlSet == Mft_Wfg_GetDigitalFilterIntFlag(pstcWfg))
    {
        //Mft_Wfg_ClrDigitalFilterIntFlag(pstcMft);
        funCallBack = pstcMftWfgInternData->pfnWfgDigtalFilterCallback;
        if(funCallBack != NULL)
        {
           funCallBack();
        }
    }

    //analog filter interrupt
    if(PdlSet == Mft_Wfg_GetAnalogFilterIntFlag(pstcWfg))
    {
        //Mft_Wfg_ClrAnalogFilterIntFlag(pstcMft);
        funCallBack = pstcMftWfgInternData->pfnWfgAnalogFilterCallback;
        if(funCallBack != NULL)
        {
           funCallBack();
        }
    }
}

/**
 ******************************************************************************
 ** \brief Device dependent initialization of interrupts according CMSIS with
 **        level defined in pdl.h
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \retval Ok    Successful initialization
 **
 ******************************************************************************/
static void Mft_Wfg_InitIrq( volatile stc_mftn_wfg_t* pstcWfg )
{
    if ((pstcWfg == (volatile stc_mftn_wfg_t*)(&MFT0_WFG)) || 
        (pstcWfg == (volatile stc_mftn_wfg_t*)(&MFT1_WFG)) ||
        (pstcWfg == (volatile stc_mftn_wfg_t*)(&MFT2_WFG)))  
    {
        NVIC_ClearPendingIRQ(WFG_DTIF_IRQn);
        NVIC_EnableIRQ(WFG_DTIF_IRQn);
        NVIC_SetPriority(WFG_DTIF_IRQn, PDL_IRQ_LEVEL_MFT_WFG);
    }

    return;
}

/**
 ******************************************************************************
 ** \brief Device dependent de-initialization of interrupts according CMSIS
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \retval Ok    Successful initialization
 **
 ******************************************************************************/
static void Mft_Wfg_DeInitIrq( volatile stc_mftn_wfg_t* pstcWfg )
{
    if ((pstcWfg == (volatile stc_mftn_wfg_t*)(&MFT0_WFG)) || 
        (pstcWfg == (volatile stc_mftn_wfg_t*)(&MFT1_WFG)) ||
        (pstcWfg == (volatile stc_mftn_wfg_t*)(&MFT2_WFG))) 
    {
        if((pstcWfg->NZCL & 0x7300) != 0x7300) // DIMA=DIMB=WIM10=WIM32=WIM54=1 ?
        {
            return;
        }
        
        NVIC_ClearPendingIRQ(WFG_DTIF_IRQn);
        NVIC_DisableIRQ(WFG_DTIF_IRQn);
        NVIC_SetPriority(WFG_DTIF_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
    }
    

    return;
}

#endif

/*!
 ******************************************************************************
 ** \brief  configure WFG mode
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 ** \param [in] enMode enum of WFG mode
 ** \arg    en_mft_wfg_mode_t
 **
 ** \retval Ok                    WFG mode is configured
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_ConfigMode( volatile stc_mftn_wfg_t* pstcWfg,
                               uint8_t u8CoupleCh, en_mft_wfg_mode_t enMode)
{
    volatile stc_mft_wfg_wfsa10_field_t* pstcWFSA;
    stc_mft_wfg_intern_data_t* pstcWfgInternData;
    
    // Check for NULL pointer
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }
    // Get pointer to internal data structure ...
    pstcWfgInternData = MftGetInternDataPtr( pstcWfg ) ;
    
    // ... and check for NULL
    if ( pstcWfgInternData != NULL )
    {
        pstcWfgInternData->pfnWfg10TimerCallback = NULL;
        pstcWfgInternData->pfnWfg32TimerCallback = NULL;
        pstcWfgInternData->pfnWfg54TimerCallback = NULL;
    }

    //Get pointer of current channel WFG register address
    pstcWFSA = (volatile stc_mft_wfg_wfsa10_field_t*)((volatile uint8_t*)pstcWfg + 0xA4UL + u8CoupleCh*4);
    
    switch(enMode)
    {
        case WfgThroughMode:
            pstcWFSA->TMD = 0u;
            break;
        case WfgRtPpgMode:
            pstcWFSA->TMD = 1u;
            break;
        case WfgTimerPpgMode:
            pstcWFSA->TMD = 2u;
            break;
        case WfgRtDeadTimerMode:
            pstcWFSA->TMD = 4u;
            break;
        case WfgRtDeadTimerFilterMode:
            pstcWFSA->TMD = 5u;
            break;
        case WfgPpgDeadTimerFilterMode:
            pstcWFSA->TMD = 6u;
            break;
        case WfgPpgDeadTimerMode:
            pstcWFSA->TMD = 7u;
            break;
        default:
            return ErrorInvalidParameter;
    }

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  configure WFG control bit
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 ** \param [in] pstcCtrlBits structure of control bit
 ** \arg    stc_wfg_ctrl_bits_t, GTEN, PSEL,PGEN,DMOD
 **
 ** \retval Ok                    Control bits are set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_ConfigCtrlBits( volatile stc_mftn_wfg_t* pstcWfg,uint8_t u8CoupleCh,
                                    stc_wfg_ctrl_bits_t* pstcCtrlBits)
{
    volatile stc_mft_wfg_wfsa10_field_t* pstcWFSA;

    // Check for NULL pointer
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel WFG register address
    pstcWFSA = (volatile stc_mft_wfg_wfsa10_field_t*)((volatile uint8_t*)pstcWfg + 0xA4UL + u8CoupleCh*4);

    //set register
    if(pstcCtrlBits->enDmodBit == DmodBit0B)
    {
        pstcWFSA->DMOD = 0u;
    }
    else
    {;
        pstcWFSA->DMOD = 1u;
    }
    
    pstcWFSA->PGEN = (uint8_t)pstcCtrlBits->enPgenBits;
    pstcWFSA->PSEL = (uint8_t)pstcCtrlBits->enPselBits;
    pstcWFSA->GTEN = (uint8_t)pstcCtrlBits->enGtenBits; 

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  init timer clock
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 ** \param [in] enClk count clock cycle to PCLK multiplied
 ** \arg    en_wfg_timer_clock_t
 **
 ** \retval Ok                    WFG timer is initialized
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_InitTimerClock( volatile stc_mftn_wfg_t* pstcWfg,uint8_t u8CoupleCh,
                                   en_wfg_timer_clock_t enClk)
{
    volatile stc_mft_wfg_wfsa10_field_t* pstcWFSA;

    // Check for NULL pointer
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel WFG register address
    pstcWFSA = (volatile stc_mft_wfg_wfsa10_field_t*)((volatile uint8_t*)pstcWfg + 0xA4UL + u8CoupleCh*4);

    //set count clock    
    pstcWFSA->DCK = (uint16_t)enClk;

    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_WFG == PDL_ON)  
/*!
 ******************************************************************************
 ** \brief  enable WFG timer interrupt
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 ** \param [in] pfnCallback callback function
 ** \arg    func_ptr_t
 **
 ** \retval Ok                    WFG timer interrupt is enabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_EnableTimerInt( volatile stc_mftn_wfg_t* pstcWfg,
                                   uint8_t u8CoupleCh, func_ptr_t pfnCallback)
{
    stc_mft_wfg_intern_data_t* pstcWfgInternData;

    // Check for NULL pointer
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure ...
    pstcWfgInternData = MftGetInternDataPtr( pstcWfg ) ;

    switch(u8CoupleCh)
    {
        case MFT_WFG_CH10:
            pstcWfgInternData->pfnWfg10TimerCallback = pfnCallback;
            pstcWfg->NZCL_f.WIM10 = 0;
            break;
        case MFT_WFG_CH32:
            pstcWfgInternData->pfnWfg32TimerCallback = pfnCallback;
            pstcWfg->NZCL_f.WIM32 = 0;
            break;
        case MFT_WFG_CH54:
            pstcWfgInternData->pfnWfg54TimerCallback = pfnCallback;
            pstcWfg->NZCL_f.WIM54 = 0;
            break;
        default:
            break;
    }
 
    Mft_Wfg_InitIrq(pstcWfg);
 
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  disable WFG timer interrupt
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 **
 ** \retval Ok                    WFG timer interrupt is disabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_DisableTimerInt( volatile stc_mftn_wfg_t* pstcWfg, uint8_t u8CoupleCh)
{
    stc_mft_wfg_intern_data_t* pstcWfgInternData;

    // Check for NULL pointer
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure ...
    pstcWfgInternData = MftGetInternDataPtr( pstcWfg ) ;

    switch(u8CoupleCh)
    {
        case MFT_WFG_CH10:
            pstcWfgInternData->pfnWfg10TimerCallback = NULL;
            pstcWfg->NZCL_f.WIM10 = 1;
            break;
        case MFT_WFG_CH32:
            pstcWfgInternData->pfnWfg32TimerCallback = NULL;
            pstcWfg->NZCL_f.WIM32 = 1;
            break;
        case MFT_WFG_CH54:
            pstcWfgInternData->pfnWfg54TimerCallback = NULL;
            pstcWfg->NZCL_f.WIM54 = 1;
            break;
        default:
            break;
    }
  
    Mft_Wfg_DeInitIrq(pstcWfg);

    return Ok;
}

#endif   

/*!
 ******************************************************************************
 ** \brief  Start WFG timer
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 **
 ** \retval Ok                    Start timer successfully
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/          
en_result_t Mft_Wfg_StartTimer(volatile stc_mftn_wfg_t* pstcWfg, uint8_t u8CoupleCh)
{
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }
    
    switch(u8CoupleCh)
    {
        case MFT_WFG_CH10:
            pstcWfg->WFIR_f.TMIE10 = 1;
            break;
        case MFT_WFG_CH32:
            pstcWfg->WFIR_f.TMIE32 = 1;
            break;
        case MFT_WFG_CH54:
            pstcWfg->WFIR_f.TMIE54 = 1;
            break;
        default:
            break;
    }
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  Stop WFG timer
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 **
 ** \retval Ok                    Stop timer successfully
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/  
en_result_t Mft_Wfg_StopTimer(volatile stc_mftn_wfg_t* pstcWfg, uint8_t u8CoupleCh)
{
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }
    
    switch(u8CoupleCh)
    {
        case MFT_WFG_CH10:
            pstcWfg->WFIR_f.TMIS10 = 1;
            break;
        case MFT_WFG_CH32:
            pstcWfg->WFIR_f.TMIS32 = 1;
            break;
        case MFT_WFG_CH54:
            pstcWfg->WFIR_f.TMIS54 = 1;
            break;
        default:
            break;
    }
    
    return Ok;
}


/*!
 ******************************************************************************
 ** \brief  get WFG timer interrupt flag
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 **
 ** \retval PdlSet WFG Interrupt flag is set 
 ** \retval PdlClr WFG Interrupt flag is clear
 **
 *****************************************************************************
*/
en_int_flag_t Mft_Wfg_GetTimerIntFlag( volatile stc_mftn_wfg_t* pstcWfg,
                                       uint8_t u8CoupleCh)
{
    en_int_flag_t retval = PdlClr;

    switch(u8CoupleCh)
    {
        case MFT_WFG_CH10:     
            (pstcWfg->WFIR_f.TMIF10 == 1) ? (retval = PdlSet) : (retval = PdlClr);
            break;
        case MFT_WFG_CH32: 
            (pstcWfg->WFIR_f.TMIF32 == 1) ? (retval = PdlSet) : (retval = PdlClr);
            break;
        case MFT_WFG_CH54:     
            (pstcWfg->WFIR_f.TMIF54 == 1) ? (retval = PdlSet) : (retval = PdlClr); 
            break;
        default:
            break;
    }

    return retval;
}

/*!
 ******************************************************************************
 ** \brief  clear WFG timer interrupt flag
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 **
 ** \retval Ok                    WFG timer interrupt flag is clear
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_ClrTimerIntFlag( volatile stc_mftn_wfg_t* pstcWfg, uint8_t u8CoupleCh)
{
    // Check for NULL pointer
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    //set register value
    switch(u8CoupleCh)
    {
        case MFT_WFG_CH10:
            pstcWfg->WFIR_f.TMIC10 = 1;
            break;
        case MFT_WFG_CH32:
            pstcWfg->WFIR_f.TMIC32 = 1;
            break;
        case MFT_WFG_CH54:
            pstcWfg->WFIR_f.TMIC54 = 1;
            break;
        default:
            break;
    }

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  write timer count cycle
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 ** \param [in] u16CycleA WFTA value
 ** \arg    0~65535
 ** \param [in]  u16CycleB  WFTB value
 ** \arg    0~65535
 **
 ** \retval Ok                    Timer count cycle is written
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_WriteTimerCountCycle( volatile stc_mftn_wfg_t* pstcWfg,
                                         uint8_t u8CoupleCh,
                                         uint16_t u16CycleA,
                                         uint16_t u16CycleB)
{
    volatile uint16_t* pWFTA;
    volatile uint16_t* pWFTB;

    // Check for NULL pointer
    if ( (pstcWfg == NULL)
         || (u8CoupleCh >= MFT_WFG_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel WFG register address
    pWFTA = (volatile uint16_t*)((volatile uint8_t*)pstcWfg + 0x90UL + u8CoupleCh*8);
    pWFTB = (volatile uint16_t*)((volatile uint8_t*)pstcWfg + 0x92UL + u8CoupleCh*8);

    //set the register
    *pWFTA = u16CycleA;
    *pWFTB = u16CycleB;

    return Ok;

}

/*!
 ******************************************************************************
 ** \brief  set cycle of WFG timer
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 ** \param [in]  u16Count  wfg pulse counter value
 ** \arg    0~65535
 **
 ** \retval Ok                    cycle of WFG timer is set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_SetTimerCycle( volatile stc_mftn_wfg_t* pstcWfg,
                                      uint8_t u8CoupleCh, uint16_t u16Count)
{
    volatile uint16_t* pWFTF;

    //Get pointer of current channel WFG register address
    pWFTF = (volatile uint16_t*)((volatile uint8_t*)pstcWfg + 0x8EUL + u8CoupleCh*8);

    *pWFTF =u16Count;
    return Ok;
}
/*!
 ******************************************************************************
 ** \brief  get cycle of WFG timer
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in]  u8CoupleCh  channel of WFG couple
 ** \arg    MFT_WFG_CH10 ~ MFT_WFG_CH54
 **
 ** \return Cycle value of WFG timer
 **
 *****************************************************************************
*/
uint16_t Mft_Wfg_GetTimerCurCycle( volatile stc_mftn_wfg_t* pstcWfg, uint8_t u8CoupleCh)
{
    uint16_t retval;
    volatile uint16_t* pWFTF;

    //Get pointer of current channel WFG register address
    pWFTF = (volatile uint16_t*)((volatile uint8_t*)pstcWfg + 0x8EUL + u8CoupleCh*8);

    retval = *pWFTF;
    return retval;
}

/*!
 ******************************************************************************
 ** \brief  configure NZCL
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in] pstcNzclConfig  configure of NZCL
 **
 ** \retval Ok                    NZCL is configured
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_ConfigNzcl( volatile stc_mftn_wfg_t* pstcWfg,
                               stc_wfg_nzcl_config_t* pstcNzclConfig)
{
    // Check for NULL pointer
    if ( pstcWfg == NULL)
    {
        return ErrorInvalidParameter ;
    }

    //set register value
    pstcWfg->NZCL_f.DTIEA = pstcNzclConfig->bEnDigitalFilter;
    pstcWfg->NZCL_f.DTIEB = pstcNzclConfig->bEnAnalogFilter;
    pstcWfg->NZCL &= ~(7ul<<1);
    pstcWfg->NZCL |= (uint16_t)((uint16_t)pstcNzclConfig->enDigitalFilterWidth <<1 );
    if(pstcWfg == &MFT0_WFG)
    {
        bFM0P_GPIO_EPFR01_DTTI0C = pstcNzclConfig->bSwitchToGpio;
    }
    else if (pstcWfg == &MFT1_WFG)
    {
        bFM0P_GPIO_EPFR02_DTTI1C = pstcNzclConfig->bSwitchToGpio;
    }
    else if(pstcWfg == &MFT2_WFG)
    {
        bFM0P_GPIO_EPFR03_DTTI2C = pstcNzclConfig->bSwitchToGpio;
    }
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_WFG == PDL_ON) 
/*!
 ******************************************************************************
 ** \brief  Enable DTIF interrupt
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in] pstcIntSel  DTIF interrupt selection
 ** \arg    structure of DTIF interrupt selection
 ** \param [in] pstcCallback  Pointer to DTIF interrupt callback function array
 ** \arg    structure of stc_wfg_nzcl_config_t
 **
 ** \retval Ok                    DTIF interrupt is enabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - pstcIntSel == NULL
 **                               - pstcCallback == NULL                        
 **
 ******************************************************************************/
en_result_t Mft_Wfg_EnableDtifInt( volatile stc_mftn_wfg_t* pstcWfg, 
                                   stc_dtif_int_sel_t* pstcIntSel, 
                                   stc_dtif_int_cb_t* pstcCallback)
{
    stc_mft_wfg_intern_data_t* pstcWfgInternData;
    // Check for NULL pointer
    if ((pstcWfg == NULL ) || 
        (pstcIntSel->bDtifAnalogFilterInt > 1) ||
        (pstcIntSel->bDtifDigitalFilterInt > 1)) 
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcWfgInternData = MftGetInternDataPtr( pstcWfg ) ;
    
    if(TRUE == pstcIntSel->bDtifDigitalFilterInt)
    {
        pstcWfg->NZCL_f.DIMA = 0;
        pstcWfgInternData->pfnWfgDigtalFilterCallback = pstcCallback->pfnWfgDigtalFilterCallback;

    }
    if(TRUE == pstcIntSel->bDtifAnalogFilterInt)
    {
        pstcWfg->NZCL_f.DIMB = 0;
        pstcWfgInternData->pfnWfgAnalogFilterCallback = pstcCallback->pfnWfgAnalogFilterCallback;
    }
    
    Mft_Wfg_InitIrq(pstcWfg);
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  Disable DTIF interrupt
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 ** \param [in] pstcIntSel  DTIF interrupt selection
 ** \arg    structure of DTIF interrupt selection
 **
 ** \retval Ok                    Interrupt disable normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - pstcIntSel == NULL                  
 **
 ******************************************************************************/
en_result_t Mft_Wfg_DisableDtifInt( volatile stc_mftn_wfg_t* pstcWfg, 
                                    stc_dtif_int_sel_t* pstcIntSel)
{
    stc_mft_wfg_intern_data_t* pstcWfgInternData;
    // Check for NULL pointer
    if ( (pstcWfg == NULL) || 
         (pstcIntSel->bDtifAnalogFilterInt > 1) ||
         (pstcIntSel->bDtifDigitalFilterInt > 1)  
       )
    {
        return ErrorInvalidParameter ;
    }
    
    // Get pointer to internal data structure ...
    pstcWfgInternData = MftGetInternDataPtr( pstcWfg ) ;
    
    if(TRUE == pstcIntSel->bDtifDigitalFilterInt)
    {
        pstcWfg->NZCL_f.DIMA = 1;
        pstcWfgInternData->pfnWfgDigtalFilterCallback = NULL;

    }
    if(TRUE == pstcIntSel->bDtifAnalogFilterInt)
    {
        pstcWfg->NZCL_f.DIMB = 1;
        pstcWfgInternData->pfnWfgAnalogFilterCallback = NULL;
    }
    
    Mft_Wfg_DeInitIrq(pstcWfg);
    
    return Ok;
}

#endif

/*!
 ******************************************************************************
 ** \brief  set software trigger DTIF
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \retval Ok                    DTIF is triggered by software
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_SwTiggerDtif( volatile stc_mftn_wfg_t* pstcWfg)
{
    // Check for NULL pointer
    if ( (pstcWfg == NULL))
    {
        return ErrorInvalidParameter ;
    }

    pstcWfg->NZCL_f.SDTI = 1;
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  get Digital filter interrupt flag
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \retval PdlClr Digital Filter interrupt flag is clear
 ** \retval PdlSet Digital Filter interrupt flag is set
 **
 *****************************************************************************
*/
en_int_flag_t Mft_Wfg_GetDigitalFilterIntFlag( volatile stc_mftn_wfg_t* pstcWfg)
{
    en_int_flag_t retval = PdlClr;

    //get interrupt flag via digital noise-canceler   
    (pstcWfg->WFIR_f.DTIFA == 1) ? (retval = PdlSet) : (retval = PdlClr);
    
    return retval;
}

/*!
 ******************************************************************************
 ** \brief  clear Digital filter interrupt flag
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \retval Ok                    Digital filter interrupt flag is clear
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_ClrDigitalFilterIntFlag( volatile stc_mftn_wfg_t* pstcWfg)
{
    // Check for NULL pointer
    if ( pstcWfg == NULL)
    {
        return ErrorInvalidParameter ;
    }

    pstcWfg->WFIR_f.DTICA = 1;
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  get Analog Filter interrupt flag
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \retval PdlClr Analog Filter interrupt flag is clear
 ** \retval PdlSet Analog Filter interrupt flag is set
 **
 *****************************************************************************
*/
en_int_flag_t Mft_Wfg_GetAnalogFilterIntFlag( volatile stc_mftn_wfg_t* pstcWfg)
{
    en_int_flag_t retval;

    //get interrupt flag via analog noise filter
    (pstcWfg->WFIR_f.DTIFB == 1) ? (retval = PdlSet) : (retval = PdlClr);
    
    return retval;
}
/*!
 ******************************************************************************
 ** \brief  clear Analog Filter interrupt flag
 **
 ** \param [in] pstcWfg Pointer to WFG instance
 **
 ** \retval Ok                    Analog Filter interrupt flag is clear
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8CoupleCh out of range
 **
 *****************************************************************************
*/
en_result_t Mft_Wfg_ClrAnalogFilterIntFlag( volatile stc_mftn_wfg_t* pstcWfg)
{
    // Check for NULL pointer
    if ( pstcWfg == NULL)
    {
        return ErrorInvalidParameter ;
    }

    pstcWfg->WFIR_f.DTICB = 1;
    return Ok;
}

//@}

#endif // #if (defined(PDL_PERIPHERAL_$$X_ACTIVE))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
